@@ -1,6 +1,6 @@
 /* h316_fhd.c: H316/516 fixed head simulator
 
-   Copyright (c) 2003-2008, Robert M. Supnik
+   Copyright (c) 2003-2015, Robert M. Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -25,7 +25,10 @@
 
    fhd          516-4400 fixed head disk
 
-   15-May-06    RMS     Fixed bug in autosize attach (reported by David Gesswein)
+   03-Sep-13    RMS     Added explicit void * cast
+   03-Jul-13    RLA     compatibility changes for extended interrupts
+   19-Mar-12    RMS     Fixed declaration of chan_req (Mark Pizzolato)
+   15-May-06    RMS     Fixed bug in autosize attach (David Gesswein)
    04-Jan-04    RMS     Changed sim_fsize calling sequence
 
    These head-per-track devices are buffered in memory, to minimize overhead.
@@ -72,7 +75,8 @@
 #define OTA_CW1         1                               /* expecting CW1 */
 #define OTA_CW2         2                               /* expecting CW2 */
 
-extern int32 dev_int, dev_enb, chan_req;
+extern int32 dev_int, dev_enb;
+extern uint32 chan_req;
 extern int32 stop_inst;
 extern uint32 dma_ad[DMA_MAX];
 
@@ -111,7 +115,7 @@ uint32 fhd_csword (uint32 cs, uint32 ch);
    fhd_reg      register list
 */
 
-DIB fhd_dib = { FHD, IOBUS, 1, &fhdio };
+DIB fhd_dib = { FHD, 1, IOBUS, IOBUS, INT_V_FHD, INT_V_NONE, &fhdio, 0 };
 
 UNIT fhd_unit = { 
     UDATA (&fhd_svc, UNIT_FIX+UNIT_ATTABLE+UNIT_BUFABLE+UNIT_MUSTBUF,
@@ -371,7 +375,7 @@ uint32 tk = CW1_GETTK (fhd_cw1);                        /* track */
 uint32 ca = CW2_GETCA (fhd_cw2);                        /* char addr */
 uint32 wa = ca >> 1;                                    /* word addr */
 uint32 ba = (((sf * FH_NUMTK) + tk) * FH_NUMWD) + wa;   /* buffer offset */
-uint16 *fbuf = uptr->filebuf;                           /* buffer base */
+uint16 *fbuf = (uint16 *) uptr->filebuf;                /* buffer base */
 uint32 wd;
 
 if (fhd_bad_wa (wa))                                    /* addr bad? */
