@@ -1,6 +1,6 @@
 /* i1401_cpu.c: IBM 1401 CPU simulator
 
-   Copyright (c) 1993-2008, Robert M. Supnik
+   Copyright (c) 1993-2017, Robert M. Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -23,18 +23,24 @@
    used in advertising or otherwise to promote the sale, use or other dealings
    in this Software without prior written authorization from Robert M Supnik.
 
-   11-Jul-08    RMS     Added missing A magtape modifier (from Van Snyder)
-                        Fixed tape indicator implementation (from Bob Abeles)
-                        Fixed bug in ZA and ZS (from Bob Abeles)
+   13-Mar-17    RMS     Fixed MTF length checking (COVERITY)
+   30-Jan-15    RMS     Fixed treatment of overflow (Ken Shirriff)
+   08-Oct-12    RMS     Clear storage and branch preserves B register (Van Snyder)
+   19-Mar-11    RMS     Reverted multiple tape indicator implementation
+   20-Jan-11    RMS     Fixed branch on EOT indicator per hardware (Van Snyder)
+   07-Nov-10    RMS     Fixed divide not to clear word marks in quotient
+   24-Apr-10    RMS     Revised divide algorithm (Van Snyder)
+   11-Jul-08    RMS     Added missing A magtape modifier (Van Snyder)
+                        Fixed tape indicator implementation (Bob Abeles)
+                        Fixed bug in ZA and ZS (Bob Abeles)
    07-Jul-07    RMS     Removed restriction on load-mode binary tape
    28-Jun-07    RMS     Added support for SS overlap modifiers
-   22-May-06    RMS     Fixed format error in CPU history (found by Peter Schorn)
-   06-Mar-06    RMS     Fixed bug in divide (found by Van Snyder)
-   22-Sep-05    RMS     Fixed declarations (from Sterling Garwood)
-   01-Sep-05	RMS     Removed error stops in MCE
+   22-May-06    RMS     Fixed format error in CPU history (Peter Schorn)
+   06-Mar-06    RMS     Fixed bug in divide (Van Snyder)
+   22-Sep-05    RMS     Fixed declarations (Sterling Garwood)
+   01-Sep-05    RMS     Removed error stops in MCE
    16-Aug-05    RMS     Fixed C++ declaration and cast problems
-   02-Jun-05    RMS     Fixed SSB-SSG clearing on RESET
-                        (reported by Ralph Reinke)
+   02-Jun-05    RMS     Fixed SSB-SSG clearing on RESET (Ralph Reinke)
    14-Nov-04    WVS     Added column binary support, debug support
    06-Nov-04    RMS     Added instruction history
    12-Jul-03    RMS     Moved ASCII/BCD tables to included file
@@ -174,6 +180,14 @@ typedef struct {
                         PCQ_ENTRY; \
                         IS = AS;
 
+#define BRANCH_CS       if (ADDR_ERR (AS)) { \
+                            reason = STOP_INVBR; \
+                            break; \
+                            } \
+                        PCQ_ENTRY; \
+                        IS = AS;
+
+
 uint8 M[MAXMEMSIZE] = { 0 };                            /* main memory */
 int32 saved_IS = 0;                                     /* saved IS */
 int32 AS = 0;                                           /* AS */
@@ -193,11 +207,7 @@ int32 hst_lnt = 0;                                      /* history length */
 InstHistory *hst = NULL;                                /* instruction history */
 t_bool conv_old = 0;                                    /* old conversions */
 
-extern int32 sim_int_char;
 extern int32 sim_emax;
-extern t_value *sim_eval;
-extern FILE *sim_deb;
-extern uint32 sim_brk_types, sim_brk_dflt, sim_brk_summ; /* breakpoint info */
 
 t_stat cpu_ex (t_value *vptr, t_addr addr, UNIT *uptr, int32 sw);
 t_stat cpu_dep (t_value val, t_addr addr, UNIT *uptr, int32 sw);
@@ -210,8 +220,8 @@ t_stat cpu_show_conv (FILE *st, UNIT *uptr, int32 val, void *desc);
 int32 store_addr_h (int32 addr);
 int32 store_addr_t (int32 addr);
 int32 store_addr_u (int32 addr);
-int32 div_add (int32 ap, int32 bp, int32 aend);
-int32 div_sub (int32 ap, int32 bp, int32 aend);
+int32 div_add (int32 ap, int32 bp);
+int32 div_sub (int32 ap, int32 bp);
 void div_sign (int32 dvrc, int32 dvdc, int32 qp, int32 rp);
 t_stat iomod (int32 ilnt, int32 mod, const int32 *tptr);
 t_stat iodisp (int32 dev, int32 unit, int32 flag, int32 mod);
@@ -225,9 +235,6 @@ extern t_stat inq_io (int32 flag, int32 mod);
 extern t_stat mt_io (int32 unit, int32 flag, int32 mod);
 extern t_stat dp_io (int32 fnc, int32 flag, int32 mod);
 extern t_stat mt_func (int32 unit, int32 flag, int32 mod);
-extern t_bool mt_testind (void);
-extern t_stat sim_activate (UNIT *uptr, int32 delay);
-extern t_stat fprint_sym (FILE *of, t_addr addr, t_value *val, UNIT *uptr, int32 sw);
 
 /* CPU data structures
 
@@ -263,6 +270,7 @@ REG cpu_reg[] = {
     { FLDATA (IOCHK, iochk, 0) },
     { FLDATA (PRCHK, prchk, 0) },
     { FLDATA (HBPEND, hb_pend, 0) },
+    { BRDATA (IND, ind, 8, 32, 64), REG_HIDDEN + PV_LEFT },
     { BRDATA (ISQ, pcq, 10, 14, PCQ_SIZE), REG_RO+REG_CIRC },
     { DRDATA (ISQP, pcq_p, 6), REG_HRO },
     { ORDATA (WRU, sim_int_char, 8) },
@@ -318,7 +326,7 @@ const int32 op_table[64] = {
     L1 | L2 | L4 | L5,                                  /* write and read */
     L1 | L2 | L4 | L5,                                  /* punch */
     L1 | L4,                                            /* read and punch */
-    L1 | L2 | L4 | L5,                                  /* write and read */
+    L1 | L2 | L4 | L5,                                  /* write and punch */
     L1 | L2 | L4 | L5,                                  /* write, read, punch */
     L1,                                                 /* 10: read feed */
     L1,                                                 /* punch feed */
@@ -431,7 +439,7 @@ static const int32 ind_table[64] = {
     0, 0, 0, 0, 0, 0, 0, 0,                             /* 10 - 17 */
     0, 0, 0, 0, 0, 0, 0, 0,                             /* 20 - 27 */
     0, 1, 1, 0, 1, 0, 0, 0,                             /* 30 - 37 */
-    0, 0, 1, 0, 0, 0, 0, 0,                             /* 40 - 47 */
+    0, 0, 0, 0, 0, 0, 0, 0,                             /* 40 - 47 */
     0, 0, 1, 0, 1, 0, 0, 0,                             /* 50 - 57 */
     0, 0, 0, 0, 0, 0, 0, 0,                             /* 60 - 67 */
     0, 0, 1, 0, 0, 0, 0, 0                              /* 70 - 77 */
@@ -513,12 +521,11 @@ static const int32 mtf_mod[] = {
 
 t_stat sim_instr (void)
 {
-extern int32 sim_interval;
 int32 IS, ilnt, flags;
 int32 op, xa, t, wm, ioind, dev, unit;
 int32 a, b, i, k, asave, bsave;
 int32 carry, lowprd, sign, ps;
-int32 quo, ahigh, qs;
+int32 quo, qs;
 int32 qzero, qawm, qbody, qsign, qdollar, qaster, qdecimal;
 t_stat reason, r1, r2;
 
@@ -714,7 +721,7 @@ CHECK_LENGTH:
    1            chained A and B
    2,3          invalid A-address
    4            chained B address
-   5,6          invalid B-address
+   5,6          invalid B-address - checked in fetch
    7            normal
    8+           normal + modifier
 */
@@ -766,9 +773,9 @@ CHECK_LENGTH:
    Instruction lengths:
 
    1            chained
-   2,3          invalid A-address
+   2,3          invalid A-address - checked in fetch
    4            self (B-address = A-address)
-   5,6          invalid B-address
+   5,6          invalid B-address - checked in fetch
    7            normal
    8+           normal + ignored modifier
 */
@@ -840,12 +847,6 @@ CHECK_LENGTH:
             BRANCH;
             }
         else if (ilnt == 5) {                           /* branch on ind? */
-            if (D == IN_END) {                          /* tape indicator */
-                if (mt_testind ()) {                    /* test, reset */
-                    BRANCH;
-                    }
-                else break;
-                }
             if (ind[D]) {                               /* test indicator */
                 BRANCH;
                 }
@@ -956,9 +957,11 @@ CHECK_LENGTH:
         M[BS] = b;                                      /* store result */
         MM (BS);
         if (b & WM) {                                   /* b wm? done */
-            if (qsign && (carry == 0))                  /* compl, no carry? */
+            if ((qsign != 0) && (carry == 0))           /* eff sub and no carry? */
                 M[bsave] = WM + ((b & ZONE) ^ ABIT) + sum_table[10 - t];
-            break;
+            if ((qsign == 0) && (carry != 0))           /* eff add and carry */
+                 ind[IN_OVF] = 1;                       /* overflow */
+             break;
             }
         do {
             if (a & WM)                                 /* A WM? char = 0 */
@@ -974,7 +977,8 @@ CHECK_LENGTH:
             if ((b & WM) && (qsign == 0)) {             /* last, no recomp? */
                 M[BS] = WM + sum_table[t] +             /* zone add */
                     (((a & ZONE) + b + (carry? ABIT: 0)) & ZONE);
-                ind[IN_OVF] = carry;                    /* ovflo if carry */
+                if (carry != 0)                         /* carry out? */
+                    ind[IN_OVF] = 1;                    /* ovflo if carry */
                 }
             else M[BS] = (b & WM) + sum_table[t];       /* normal add */
             MM (BS);
@@ -1006,7 +1010,7 @@ CHECK_LENGTH:
                 ind[IN_HGH] = col_table[b & CHAR] > col_table [a & CHAR];
                 ind[IN_LOW] = ind[IN_HGH] ^ 1;
                 }
-            MM (AS);                                /* decr pointers */
+            MM (AS);                                    /* decr pointers */
             MM (BS);
             } while ((wm & WM) == 0);                   /* stop on A, B WM */
         if ((a & WM) && !(b & WM)) {                    /* short A field? */
@@ -1024,7 +1028,7 @@ CHECK_LENGTH:
    WR           write and read                          if branch
    P            punch a card                            if branch
    RP           read and punch                          if branch
-   WP   :       write and punch                         if branch
+   WP           write and punch                         if branch
    WRP          write read and punch                    if branch
    RF           read feed (nop)
    PF           punch feed (nop)
@@ -1145,19 +1149,24 @@ CHECK_LENGTH:
 
    Instruction lengths:
 
-   1-3          invalid I/O address
+   1-3          invalid I/O address - checked here
    4            normal, d-character is unit
-   5            normal
+   5            normal, d-character is last character
    6+           normal, d-character is last character
 */
 
     case OP_MTF:                                        /* magtape function */
-        if (ilnt < 4)                                   /* too short? */
+        if (ilnt < 4) {                                 /* too short? */
             reason = STOP_INVL;
-        else if (ioind != BCD_PERCNT)                   /* valid dev addr? */
-            reason = STOP_INVA;
-        else if (reason = iomod (ilnt, D, mtf_mod))     /* valid modifier? */
             break;
+            }
+      if (ioind != BCD_PERCNT) {                        /* valid dev addr? */
+            reason = STOP_INVA;
+            break;
+            }
+        if (reason = iomod (ilnt, D, mtf_mod))          /* valid modifier? */
+            break;
+
         if (dev == IO_MT)                               /* BCD? */
             reason = mt_func (unit, 0, D);
         else if (dev == IO_MTB)                         /* binary? */
@@ -1193,9 +1202,9 @@ CHECK_LENGTH:
    Instruction lengths:
 
    1            chained
-   2,3          invalid A-address
+   2,3          invalid A-address - checked in fetch
    4            self (B-address = A-address)
-   5,6          invalid B-address
+   5,6          invalid B-address - checked in fetch
    7            normal
    8+           normal + ignored modifier
 */
@@ -1369,9 +1378,9 @@ CHECK_LENGTH:
    Instruction lengths:
 
    1            chained
-   2,3          invalid A-address
+   2,3          invalid A-address - checked in fetch
    4            self (B-address = A-address)
-   5,6          invalid B-address
+   5,6          invalid B-address - checked in fetch
    7            normal
    8+           normal + ignored modifier
 */
@@ -1432,43 +1441,40 @@ CHECK_LENGTH:
    - AS points to the low order divisor digit.
    - BS points to the high order dividend digit.
    - The low order dividend digit is identified by sign (zone) bits.
-   - To the left of the dividend is a zero field of length LS + 1.
-   The low quotient is at low dividend - LS - 1.  As BS points to the
-   high dividend, the low dividend is at BS + LD - 1, so the low
-   quotient is at BS + LD - LS - 2.  The longest possible quotient is
-   LD - LS + 1, so the first possible non-zero quotient bit will be
-   found as BS - 2.
+   - To the left of the dividend is a (zero) field of length LS + 1.
+   So the quotient starts as BS - LS - 1.
+   The divide process starts with a subdividend that begins at BS - LS
+   and ends at BS.  (Note that the subdividend is one digit wider than
+   the divisor, to allow for borrows during the divide process.)  This
+   means that non-zero digits in the "zero" field to the left of the
+   dividend CAN affect the divide.
 
-   This pointer calculation assumes that the divisor has no leading zeroes.
-   For each leading zero, the start of the quotient will be one position
-   further left.
-
-   Start by locating the high order non-zero digit of the divisor.  This
-   also tests for a divide by zero.
+   Start by computing the length of the divisor and testing for divide
+   by zero.
 
    Instruction lengths:
 
    1            chained
-   2,3          invalid A-address
+   2,3          invalid A-address - checked in fetch
    4            self (B-address = A-address)
-   5,6          invalid B-address
+   5,6          invalid B-address - checked in fetch
    7            normal
    8+           normal + ignored modifier
 */
 
     case OP_DIV:
         asave = AS;
-        ahigh = -1;
-        do {
+        t = 0;                                          /* assume all 0's */
+        do {                                            /* scan divisor */
             a = M[AS];                                  /* get dvr char */
-            if ((a & CHAR) != BCD_ZERO)                 /* mark non-zero */
-                ahigh = AS;
+            if ((bcd_to_bin[a & DIGIT]) != 0)           /* mark non-zero */
+                t = 1;
             MM (AS);
             }
         while ((a & WM) == 0);
         if (reason)                                     /* address err? */
             break;
-        if (ahigh < 0) {                                /* div by zero? */
+        if (t == 0) {                                   /* div by zero? */
             ind[IN_OVF] = 1;                            /* set ovf indic */
             qs = bsave = BS;                            /* quo, dividend */
             do {
@@ -1485,15 +1491,15 @@ CHECK_LENGTH:
             BS = (BS - 2) - (asave - (AS + 1));         /* final bs */
             break;
             }
-        bsave = BS + (asave - ahigh);                   /* end subdivd */
-        qs = (BS - 2) - (ahigh - (AS + 1));             /* quo start */
+        bsave = BS;                                     /* end subdivd */
+        qs = BS - (asave - AS) - 1;                     /* quo start */
 
 /* Divide loop - done with subroutines to keep the code clean.
    In the loop,
         
-   asave =      low order divisor
-   bsave =      low order subdividend
-   qs   =       current quotient digit
+   asave =      low order divisor (constant)
+   bsave =      low order subdividend (increments)
+   qs   =       current quotient digit (increments)
 */
 
     do {
@@ -1504,11 +1510,13 @@ CHECK_LENGTH:
             }
         b = M[bsave];                                   /* save low divd */
         do {
-            t = div_sub (asave, bsave, ahigh);          /* subtract */
+            t = div_sub (asave, bsave);                 /* subtract */
             quo++;                                      /* incr quo digit */
             } while (t == 0);                           /* until borrow */
-        div_add (asave, bsave, ahigh);                  /* restore */
+        div_add (asave, bsave);                         /* restore */
         quo--;
+        if (quo > 9)                                    /* overflow? */
+            ind[IN_OVF] = 1;                            /* set ovf indic */
         M[qs] = (M[qs] & WM) | sum_table[quo];          /* store quo digit */
         bsave++;                                        /* adv divd, quo */
         qs++;
@@ -1571,6 +1579,9 @@ CHECK_LENGTH:
    5,6          invalid B-address
    7            branch
    8+           one operand, branch ignored
+
+   Note that clear storage and branch does not overwrite the B register,
+   unlike all other branches
 */
 
     case OP_CS:                                         /* clear storage */
@@ -1580,7 +1591,7 @@ CHECK_LENGTH:
         if (BS < 0)                                     /* wrap if needed */
             BS = BS + MEMSIZE;
         if (ilnt == 7) {                                /* branch variant? */
-            BRANCH;
+            BRANCH_CS;                                  /* special branch */
             }
         break;
 
@@ -1695,7 +1706,7 @@ return bin_to_bcd[addr % 10] | (thous << (V_ZONE - 2));
 
 /* div_add - add string for divide */
 
-int32 div_add (int32 ap, int32 bp, int32 aend)
+int32 div_add (int32 ap, int32 bp)
 {
 int32 a, b, c, r;
 
@@ -1706,16 +1717,16 @@ do {
     r = bcd_to_bin[b & DIGIT] +                         /* sum digits + c */
         bcd_to_bin[a & DIGIT] + c;
     c = (r >= 10);                                      /* set carry out */
-    M[bp] = sum_table[r];                               /* store result */
+    M[bp] = (M[bp] & WM) | sum_table[r];                /* store result */
     ap--;
     bp--;
-    } while (ap >= aend);
+    } while ((a & WM) == 0);
 return c;
 }
 
 /* div_sub - substract string for divide */
 
-int32 div_sub (int32 ap, int32 bp, int32 aend)
+int32 div_sub (int32 ap, int32 bp)
 {
 int32 a, b, c, r;
 
@@ -1726,14 +1737,14 @@ do {
     r = bcd_to_bin[b & DIGIT] -                         /* a - b - borrow */
         bcd_to_bin[a & DIGIT] - c;
     c = (r < 0);                                        /* set borrow out */
-    M[bp] = sum_table[r + 10];                          /* store result */
+    M[bp] = (M[bp] & WM) | sum_table[r + 10];           /* store result */
     ap--;
     bp--;
-    } while (ap >= aend);
-b = M[bp] & CHAR;                                       /* borrow position */
-if (b && (b != BCD_ZERO)) {                             /* non-zero? */
+    } while ((a & WM) == 0);
+b = M[bp];                                              /* borrow position */
+if (bcd_to_bin[b & DIGIT] != 0) {                       /* non-zero? */
     r = bcd_to_bin[b & DIGIT] - c;                      /* subtract borrow */
-    M[bp] = sum_table[r];                               /* store result */
+    M[bp] = (M[bp] & WM) | sum_table[r];                /* store result */
     return 0;                                           /* subtract worked */
     }
 return c;                                               /* return borrow */
@@ -1907,8 +1918,6 @@ char *cptr = (char *) desc;
 t_value sim_eval[MAX_L + 1];
 t_stat r;
 InstHistory *h;
-extern t_stat fprint_sym (FILE *ofile, t_addr addr, t_value *val,
-    UNIT *uptr, int32 sw);
 
 if (hst_lnt == 0)                                       /* enabled? */
     return SCPE_NOFNC;
