@@ -1,6 +1,6 @@
 /* i1620_cd.c: IBM 1622 card reader/punch
 
-   Copyright (c) 2002-2008, Robert M. Supnik
+   Copyright (c) 2002-2017, Robert M. Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -26,9 +26,15 @@
    cdr          1622 card reader
    cdp          1622 card punch
 
+   23-Jun-17    RMS     Unattached error does not set RDCHK/WRCHK
+   09-Mar-17    RMS     Guardbanded translation table lookups (COVERITY)
+   31-Jan-15    TFM     Changes to translation tables (Tom McBride)
+   10-Dec-13    RMS     Fixed WA card punch translations (Bob Armstrong)
+                        Fixed card reader EOL processing (Bob Armstrong)
+   19-Mar-12    RMS     Fixed declarations of saved_pc, io_stop (Mark Pizzolato)
    19-Jan-07    RMS     Set UNIT_TEXT flag
-   13-Jul-06    RMS     Fixed card reader fgets call (from Tom McBride)
-                        Fixed card reader boot sequence (from Tom McBride)
+   13-Jul-06    RMS     Fixed card reader fgets call (Tom McBride)
+                        Fixed card reader boot sequence (Tom McBride)
    21-Sep-05    RMS     Revised translation tables for 7094/1401 compatibility
    25-Apr-03    RMS     Revised for extended file support
 
@@ -43,7 +49,7 @@
 extern uint8 M[MAXMEMSIZE];
 extern uint8 ind[NUM_IND];
 extern UNIT cpu_unit;
-extern int32 io_stop;
+extern uint32 io_stop;
 
 char cdr_buf[CD_LEN + 2];
 char cdp_buf[CD_LEN + 2];
@@ -104,102 +110,146 @@ DEVICE cdp_dev = {
     };
 
 /* Data tables.  The card reader presents unusual problems.
-   - Unique codes needed for 11-2-8 (uses !) and 12-7-8 (uses ") .
-   - Can punch both 11 (-) and 11-0 (uses ]).
-   On input, the nul and nl generated by C are converted to
-   spaces; tabs and line feeds are also converted to spaces.
 
+   Some of these translations may seem strange. The 1620 could
+   read and punch cards numerically (one 1620 storage location
+   per card column) or alphabetically (two 1620 storage locations
+   per card column). Even though a card might have contained any
+   possible character (digit, letter, special character), it 
+   could still be read numerically. In this case, some characters 
+   behaved the same as numbers or as record marks. The results 
+   are well defined in IBM documentation. 
+
+   In order to make it possible to prepare card decks for input
+   using normal text editors, ASCII characters have been assigned 
+   to represent 1620 characters that could appear on cards. In most
+   cases, this was easy since the letters, digits and punctuation
+   characters all have equivalent ASCII assignments. Five 1620 
+   characters do not have equivalent ASCII graphics and are 
+   assigned as follows:
+
+     ]  is used to represent a flagged zero
+     |  is used to represent a record mark
+     !  is used to represent a flagged record mark
+     }  is used to represent a group mark
+     "  is used to represent a flagged group mark
+
+   As a concession to some editors, ASCII nul, nl, tab, and lf 
+   characters are accepted and converted to blanks. Also, for
+   the same reason, lower case letters are treated the same as
+   upper case on input. All other ASCII characters not in the
+   1620 character set are treated as errors.
+
+                                                (Tom McBride)
+   
+*/
 /* Card reader (ASCII) to numeric (one digit) */
 
-const char cdr_to_num[128] = {
+const int8 cdr_to_num[128] = {
  0x00,   -1,   -1,   -1,   -1,   -1,   -1,   -1,        /* 00 */
    -1, 0x00, 0x00,   -1,   -1, 0x00,   -1,   -1,
    -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,        /* 10 */
    -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
- 0x00, 0x1A, 0x0F, 0x0B, 0x1B, 0x0C, 0x00, 0x0C,        /*  !"#$%&' */
- 0x0C, 0x0C, 0x1C, 0x00, 0x0B, 0x10, 0x1B, 0x01,        /* ()*+,-./ */
+ 0x00, 0x1A, 0x1F,   -1, 0x1B,   -1,   -1,   -1,        /*  !" $    */
+ 0x0C, 0x0C, 0x1C, 0x00, 0x0B, 0x10, 0x0B, 0x01,        /* ()*+,-./ */
  0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,        /* 01234567 */
- 0x08, 0x09, 0x00, 0x1E, 0x1E, 0x0B, 0x0E, 0x1A,        /* 89:;<=>? */
+ 0x08, 0x09,   -1,   -1,   -1, 0x0B,   -1,   -1,        /* 89   =   */
  0x0C, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,        /* @ABCDEFG */
  0x08, 0x09, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16,        /* HIJKLMNO */
  0x17, 0x18, 0x19, 0x02, 0x03, 0x04, 0x05, 0x06,        /* PQRSTUVW */
- 0x07, 0x08, 0x09, 0x00, 0x0E, 0x10, 0x0A, 0x1F,        /* XYZ[\]^_ */
+ 0x07, 0x08, 0x09,   -1,   -1, 0x10,   -1,   -1,        /* XYZ  ]   */
    -1, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,        /* `abcdefg */
  0x08, 0x09, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16,        /* hijklmno */
  0x17, 0x18, 0x19, 0x02, 0x03, 0x04, 0x05, 0x06,        /* pqrstuvw */
- 0x07, 0x08, 0x09, 0x0F, 0x0A, 0x1F, 0x00,   -1         /* xyz{|}~  */
+ 0x07, 0x08, 0x09,   -1, 0x0A, 0x0F,   -1,   -1         /* xyz |}   */
  };
 
 /* Numeric (flag + digit) to card punch (ASCII) */
 
-const char num_to_cdp[32] = {
+/* Note that all valid digits produce different 
+   codes except that both numeric blanks and flagged 
+   numeric blanks both produce a blank column. (tfm) */
+
+const int8 num_to_cdp[32] = {
  '0', '1', '2', '3', '4', '5', '6', '7',                /* 0 */
- '8', '9', '|', ',', ' ', '"', ' ', '"',
+ '8', '9', '|',  -1, ' ',  -1,  -1, '}',
  ']', 'J', 'K', 'L', 'M', 'N', 'O', 'P',                /* F + 0 */
- 'Q', 'R', '!', '$',  -1,  -1,  -1, '"'
+ 'Q', 'R', '!',  -1, ' ',  -1,  -1, '"'
  };
 
 /* Card reader (ASCII) to alphameric (two digits)
 
-   11-2-8 (!) reads as 5A
-   11-7-8 (_) reads as 5F
-   12-2-8 (?) reads inconsistently (here 02)
-   12-6-8 (<) reads inconsistently (here 5E)
-   12-7-8 (}) reads as 5F
+   ] reads as 50 (flagged zero)
+   | reads as 0A (record mark)
+   ! reads as 5A (flagged record mark)
+   } reads as 0F (group mark)
+   " reads as 5F (flagged group mark)
+
+   As a concession to some editors, ASCII nul, nl, tab, and lf 
+   characters are accepted and converted to blanks. Also, for
+   the same reason, lower case letters are treated the same as
+   upper case on input. All other ASCII characters not in the
+   1620 character set are treated as errors.  
+   
 */
 
-const char cdr_to_alp[128] = {
+const int8 cdr_to_alp[128] = {
  0x00,   -1,   -1,   -1,   -1,   -1,   -1,   -1,        /* 00 */
    -1, 0x00, 0x00,   -1,   -1, 0x00,   -1,   -1,
    -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,        /* 10 */
    -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
- 0x00, 0x5A, 0x0F, 0x33, 0x13, 0x24, 0x10, 0x34,        /*  !"#$%&' */
+ 0x00, 0x5A, 0x5F,   -1, 0x13,   -1,   -1,   -1,        /*  !" $    */
  0x24, 0x04, 0x14, 0x10, 0x23, 0x20, 0x03, 0x21,        /* ()*+,-./ */
  0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77,        /* 01234567 */
- 0x78, 0x79, 0x70, 0x5E, 0x5E, 0x33, 0x0E, 0x02,        /* 89:;<=>? */
+ 0x78, 0x79,   -1,   -1,   -1, 0x33,   -1,   -1,        /* 89   =   */
  0x34, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47,        /* @ABCDEFG */
  0x48, 0x49, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56,        /* HIJKLMNO */
  0x57, 0x58, 0x59, 0x62, 0x63, 0x64, 0x65, 0x66,        /* PQRSTUVW */
- 0x67, 0x68, 0x69, 0x40, 0x0E, 0x50, 0x0A, 0x5F,        /* XYZ[\]^_ */
- 0x50, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47,        /* `abcdefg */
+ 0x67, 0x68, 0x69,   -1,   -1, 0x50,   -1,   -1,        /* XYZ  ]   */
+   -1, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47,        /*  abcdefg */
  0x48, 0x49, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56,        /* hijklmno */
  0x57, 0x58, 0x59, 0x62, 0x63, 0x64, 0x65, 0x66,        /* pqrstuvw */
- 0x67, 0x68, 0x69, 0x0F, 0x0A, 0x5F, 0x60,   -1         /* xyz{|}~  */
+ 0x67, 0x68, 0x69,   -1, 0x0A, 0x0F,   -1,   -1         /* xyz |}   */
  };
 
-/* Alphameric (two digits) to card punch (ASCII).  Oddities:
+/* Alphameric (two digits) to card punch (ASCII).  
 
-        02 -> 12-2-8 (?), symmetric
-        07 -> 12-7-8 (}), reads as 5F
-        12 -> 11-2-8 (!), reads as 5A
-        15 -> 11,0 (`), reads as 50
-        22 -> 0-2-8 (|), reads as 0A
-        32 -> 2-8 (^), reads as 0A
-        5B -> 11-3-8 (=), reads as 13
-        6A -> 0-2-8 (|), reads as 0A
-        6B -> 0-3-8 (,), reads as 23
-        AA -> 0-2-8 (|), reads as 0A
+   All 1620 compilers are know to punch numeric data
+   (i.e. data that it knows will be read back using RNCD)
+   in alphameric mode with WACD. Because of that, there are some
+   alpha to ASCII translations that absolutely MUST work out right
+   or otherwise we won't be able to load the object decks.
 
-   There is no way to punch 0-5-8 (~), 0-6-8 (\),
-   11-5-8 (]), 11-6-8 (;), 11-7-8 (_),
-   12-5-8 ([), or 12-6-8 (<)
+     50 - punches as ] (flagged zero) 
+     0A - punches as | (record mark)
+     0F - punches as } (group mark)
+
+   If a program punches alphameric data that includes a flagged 
+   record mark or flagged group mark, they will be punched 
+   as below. No known IBM compiler punches any of these but
+   some application programs do. The mapping of characters for
+   the card reader and punch is such that a card deck can be 
+   duplicated with a RACD, WACD loop. 
+
+     5A - punches as ! (flagged record mark)
+     5F - punches as " (flagged group mark)
 */
 
-const char alp_to_cdp[256] = {
- ' ',  -1, '?', '.', ')',  -1,  -1, '}',                /* 00 */
-  -1,  -1, '\'', -1,  -1,  -1,  -1, '"',
- '+',  -1, '!', '$', '*', ']',  -1,  -1,                /* 10 */ 
+const int8 alp_to_cdp[256] = {
+ ' ',  -1,  -1, '.', ')',  -1,  -1,  -1,                /* 00 */
+  -1,  -1, '|',  -1,  -1,  -1,  -1, '}',
+ '+',  -1,  -1, '$', '*',  -1,  -1,  -1,                /* 10 */ 
   -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,
- '-', '/', '|', ',', '(',  -1,  -1,  -1,                /* 20 */
+ '-', '/',  -1, ',', '(',  -1,  -1,  -1,                /* 20 */
+  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,
+  -1,  -1,  -1, '=', '@',  -1,  -1,  -1,                /* 30 */
   -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,
-  -1,  -1, '^', '=', '@', ':', ' ',  -1,                /* 30 */
-  -1,  -1, '|',  -1,  -1,  -1,  -1, '"',
   -1, 'A', 'B', 'C', 'D', 'E', 'F', 'G',                /* 40 */
  'H', 'I',  -1,  -1,  -1,  -1,  -1,  -1,
- '_', 'J', 'K', 'L', 'M', 'N', 'O', 'P',                /* 50 */
- 'Q', 'R', '?', '=',  -1,  -1,  -1, '}',
+ ']', 'J', 'K', 'L', 'M', 'N', 'O', 'P',                /* 50 */
+ 'Q', 'R', '!',  -1,  -1,  -1,  -1, '"',
   -1, '/', 'S', 'T', 'U', 'V', 'W', 'X',                /* 60 */
- 'Y', 'Z', '|', ',',  -1,  -1,  -1,  -1,
+ 'Y', 'Z',  -1,  -1,  -1,  -1,  -1,  -1,
  '0', '1', '2', '3', '4', '5', '6', '7',                /* 70 */
  '8', '9',  -1,  -1,  -1,  -1,  -1,  -1,
   -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,                /* 80 */
@@ -207,7 +257,7 @@ const char alp_to_cdp[256] = {
   -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,                /* 90 */
   -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,
   -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,                /* A0 */
-  -1,  -1, '|',  -1,  -1,  -1,  -1,  -1,
+  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,
   -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,                /* B0 */
   -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,
   -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,                /* C0 */
@@ -241,7 +291,7 @@ switch (op) {                                           /* case on op */
         if (r != SCPE_OK)                               /* error? */
             return r;
         for (i = 0; i < CD_LEN; i++) {                  /* transfer to mem */
-            cdc = cdr_to_num[cdr_buf[i]];               /* translate */
+            cdc = cdr_to_num[cdr_buf[i] & 0177];        /* translate */
             if (cdc < 0) {                              /* invalid? */
                 ind[IN_RDCHK] = 1;                      /* set read check */
                 if (io_stop)                            /* set return status */
@@ -258,7 +308,7 @@ switch (op) {                                           /* case on op */
         if (r != SCPE_OK)                               /* error? */
             return r;
         for (i = 0; i < CD_LEN; i++) {                  /* transfer to mem */
-            cdc = cdr_to_alp[cdr_buf[i]];               /* translate */
+            cdc = cdr_to_alp[cdr_buf[i] & 0177];        /* translate */
             if (cdc < 0) {                              /* invalid? */
                 ind[IN_RDCHK] = 1;                      /* set read check */
                 if (io_stop)                            /* set return status */
@@ -278,17 +328,20 @@ switch (op) {                                           /* case on op */
 return sta;
 }
 
-/* Fill card reader buffer - all errors are hard errors */
+/* Fill card reader buffer - all errors are hard errors
+
+   As Bob Armstrong pointed out, this routines needs to account
+   for variants in text file formats, which may terminate lines
+   with cr-lf (Windows), lf (UNIX), or cr (Mac).
+*/
 
 t_stat cdr_read (void)
 {
 int32 i;
 
 ind[IN_LAST] = 0;                                       /* clear last card */
-if ((cdr_unit.flags & UNIT_ATT) == 0) {                 /* attached? */
-    ind[IN_RDCHK] = 1;                                  /* no, error */
+if ((cdr_unit.flags & UNIT_ATT) == 0)                   /* attached? */
     return SCPE_UNATT;
-    }
 
 for (i = 0; i < CD_LEN + 2; i++)                        /* clear buffer */
     cdr_buf[i] = ' ';
@@ -297,10 +350,26 @@ if (feof (cdr_unit.fileref))                            /* eof? */
     return STOP_NOCD;
 if (ferror (cdr_unit.fileref)) {                        /* error? */
     ind[IN_RDCHK] = 1;                                  /* set read check */
-    perror ("CDR I/O error");
+    sim_perror ("CDR I/O error");
     clearerr (cdr_unit.fileref);
     return SCPE_IOERR;
     }
+if ((i = strlen (cdr_buf)) > 0) {                       /* anything at all? */
+    if (cdr_buf[i-1] == '\n') {                         /* line end in \n? */
+        cdr_buf[i-1] = 0;                               /* remove it */
+        }
+    else if (cdr_buf[i-1] == '\r') {                    /* line end in \r? */
+        cdr_buf[i-1] = 0;                               /* remove it */
+        cdr_unit.pos = ftell (cdr_unit.fileref);        /* save position */
+        if (fgetc (cdr_unit.fileref) != '\n')           /* next char not \n? */
+            fseek (cdr_unit.fileref, cdr_unit.pos, SEEK_SET); /* then rewind */  
+        }
+    else {                                              /* line too long */
+        ind[IN_RDCHK] = 1;
+        sim_perror ("CDR line too long");
+        return SCPE_IOERR;
+        }
+    }
 cdr_unit.pos = ftell (cdr_unit.fileref);                /* update position */
 getc (cdr_unit.fileref);                                /* see if more */
 if (feof (cdr_unit.fileref))                            /* eof? set last */
@@ -332,8 +401,8 @@ return SCPE_OK;
 t_stat cdr_boot (int32 unitno, DEVICE *dptr)
 {
 t_stat r;
-int32 old_io_stop;
-extern int32 saved_PC;
+uint32 old_io_stop;
+extern uint32 saved_PC;
 
 old_io_stop = io_stop;
 io_stop = 1;
@@ -361,12 +430,38 @@ uint8 z, d;
 switch (op) {                                           /* decode op */
 
     case OP_DN:
-        return cdp_num (pa, 20000 - (pa % 20000), TRUE); /* dump numeric */
+
+        /* DN punches all characters the same as WN except that a flagged
+           zero is punched as a hyphen (-) instead of a flagged
+           zero ([). Punching begins at the P address and continues until
+           the last digit of the storage module containing the P address
+           has been punched. If the amount of data to be punched is an 
+           exact multiple of 80, the operation ends there. If the last
+           character of the module does not fill out the card, additional
+           characters from the next higher addresses (addressing wraps to
+           back to zero if the operation started in the highest module) are 
+           used to fill out the card.                   (Tom McBride) */
+
+        return cdp_num (pa,                             /* dump numeric */
+                        ((20000 - (pa % 20000) + 79) / 80) * 80,
+                        TRUE);
 
     case OP_WN:
+
+        /* WN always punches exactly 80 characters. If the highest address
+           in the machine is reached before the card is full, addressing 
+           wraps around to zero and continues. The PP function handles
+           this correctly.                              (Tom McBride) */
+
         return cdp_num (pa, CD_LEN, FALSE);             /* write numeric */
 
-    case OP_WA:
+    case OP_WA:                                         /* write alphanumerically */
+
+        /* WA always punches exactly 80 characters. If the highest address
+           in the machine is reached before the card is full, addressing
+           wraps around to zero and continues. The ADDR_A function handles
+           this correctly.                              (Tom McBride) */
+
         for (i = 0; i < CD_LEN; i++) {                  /* one card */
             d = M[pa] & DIGIT;                          /* get digit pair */
             z = M[pa - 1] & DIGIT;
@@ -403,7 +498,8 @@ while (ncd-- >= 0) {                                    /* until done */
         break;
     for (i = 0; i < len; i++) {                         /* one card */
         d = M[pa] & (FLAG | DIGIT);                     /* get char */
-        if (dump && (d == FLAG)) cdc = '-';             /* dump? F+0 is diff */
+        if (dump && (d == FLAG))                        /* dump? F+0 is diff .. */
+            cdc = '-';                                  /* .. punch as hyphen */
         else cdc = num_to_cdp[d];                       /* translate */
         if (cdc < 0) {                                  /* bad char? */
             ind[IN_WRCHK] = 1;                          /* set write check */
@@ -423,10 +519,8 @@ return SCPE_OK;
 
 t_stat cdp_write (uint32 len)
 {
-if ((cdp_unit.flags & UNIT_ATT) == 0) {                 /* attached? */
-    ind[IN_WRCHK] = 1;                                  /* no, error */
+if ((cdp_unit.flags & UNIT_ATT) == 0)                   /* attached? */
     return SCPE_UNATT;
-    }
 
 while ((len > 0) && (cdp_buf[len - 1] == ' '))          /* trim spaces */
     --len;
@@ -437,7 +531,7 @@ fputs (cdp_buf, cdp_unit.fileref);                      /* write card */
 cdp_unit.pos = ftell (cdp_unit.fileref);                /* count char */
 if (ferror (cdp_unit.fileref)) {                        /* error? */
     ind[IN_WRCHK] = 1;
-    perror ("CDP I/O error");
+    sim_perror ("CDP I/O error");
     clearerr (cdp_unit.fileref);
     return SCPE_IOERR;
     }
