@@ -1,6 +1,6 @@
 /* i7094_cpu.c: IBM 7094 CPU simulator
 
-   Copyright (c) 2003-2007, Robert M. Supnik
+   Copyright (c) 2003-2017, Robert M. Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -25,6 +25,12 @@
 
    cpu          7094 central processor
 
+   07-Sep-17    RMS     Fixed sim_eval declaration in history routine (COVERITY)
+   31-Dec-11    RMS     Select traps have priority over protect traps
+                        Added SRI, SPI
+                        Fixed user mode and relocation from CTSS RPQ documentation
+   16-Jul-10    RMS     Fixed user mode protection (Dave Pitts)
+                        Fixed issues in storage nullification mode
    28-Apr-07    RMS     Removed clock initialization
    29-Oct-06    RMS     Added additional expanded core instructions
    17-Oct-06    RMS     Fixed the fix in halt IO wait loop
@@ -55,6 +61,9 @@
    protection, and relocation.  Additional state:
 
    USER                 user mode
+   RELOCM               relocation mode
+   USER_BUF             user mode buffer
+   RELOC_BUF            relocation buffer
    INST_BASE            instruction memory select (A vs B core)
    DATA_BASE            data memory select (A vs B core)
    IND_RELOC<0:6>       relocation value (block number)
@@ -119,8 +128,8 @@
         ch_flags[0..7]  flags for channels A..H
         chtr_enab       channel trap enables
         chtr_inht       channel trap inhibit due to trap (cleared by RCT)
-        chtr_inhi       channel trap inhibit due to XEC, ENAB, RCT, RDS,
-                        or WDS (cleared after one instruction)
+        chtr_inhi       channel trap inhibit due to XEC, ENB, RCT, LRI,
+                        LPI, SEA, SEB (cleared after one instruction)
 
       Channel traps are summarized in variable chtr_pend.
 
@@ -148,6 +157,8 @@
 
 #define HALT_IO_LIMIT   ((2 << 18) + 1)                 /* max wait to stop */
 
+#define EAMASK          (mode_storn? A704_MASK: AMASK)  /* eff addr mask */
+
 t_uint64 *M = NULL;                                     /* memory */
 t_uint64 AC = 0;                                        /* AC */
 t_uint64 MQ = 0;                                        /* MQ */
@@ -175,6 +186,9 @@ uint32 ind_dvc = 0;                                     /* divide check */
 uint32 ind_ioc = 0;                                     /* IO check */
 uint32 cpu_model = I_9X|I_94;                           /* CPU type */
 uint32 mode_user = 0;                                   /* (CTSS) user mode */
+uint32 mode_reloc = 0;                                  /* (CTSS) relocation mode */
+uint32 user_buf = 0;                                    /* (CTSS) user mode buffer */
+uint32 reloc_buf = 0;                                   /* (CTSS) reloc mode buffer */
 uint32 ind_reloc = 0;                                   /* (CTSS) relocation */
 uint32 ind_start = 0;                                   /* (CTSS) prot start */
 uint32 ind_limit = 0;                                   /* (CTSS) prot limit */
@@ -185,7 +199,6 @@ uint32 ht_pend = 0;                                     /* HTR pending */
 uint32 ht_addr = 0;                                     /* HTR address */
 uint32 stop_illop = 1;                                  /* stop on ill op */
 uint32 cpu_astop = 0;                                   /* address stop */
-static uint32 eamask = AMASK;                           /* (dynamic) addr mask */
 
 uint16 pcq[PCQ_SIZE] = { 0 };                           /* PC queue */
 int32 pcq_p = 0;                                        /* PC queue ptr */
@@ -199,11 +212,6 @@ extern uint32 ch_sta[NUM_CHAN];
 extern uint32 ch_flags[NUM_CHAN];
 extern DEVICE mt_dev[NUM_CHAN];
 extern DEVICE ch_dev[NUM_CHAN];
-extern FILE *sim_deb;
-extern int32 sim_int_char;
-extern int32 sim_interval;
-extern int32 sim_switches;
-extern uint32 sim_brk_types, sim_brk_dflt, sim_brk_summ; /* breakpoint info */
 
 /* Forward and external declarations */
 
@@ -304,6 +312,9 @@ REG cpu_reg[] = {
     { FLDATA (CHTR_INHI, chtr_inhi, 0) },
     { ORDATA (CHTR_ENAB, chtr_enab, 30) },
     { FLDATA (USERM, mode_user, 0) },
+    { FLDATA (RELOCM, mode_reloc, 0) },
+    { FLDATA (USERBUF, user_buf, 0) },
+    { FLDATA (RELOCBUF, reloc_buf, 0) },
     { FLDATA (IMEM, inst_base, BCORE_V) },
     { FLDATA (DMEM, data_base, BCORE_V) },
     { GRDATA (RELOC, ind_reloc, 8, VA_N_BLK, VA_V_BLK) },
@@ -569,8 +580,8 @@ const uint8 op_flags[1024] = {
  I_XNR|I_CT, 0         , 0         , 0         ,
  0         , 0         , 0         , 0         ,
  0         , 0         , 0         , 0         ,
- I_XN      , 0         , I_XNR|I_9X, I_XN|I_94 ,        /* -600 */
- 0         , 0         , 0         , 0         ,
+ I_XN      , I_CT      , I_XNR|I_9X, I_XN|I_94 ,        /* -600 */
+ I_CT      , 0         , 0         , 0         ,
  0         , 0         , 0         , 0         ,
  0         , 0         , 0         , 0         ,
  I_XNR|I_9X, 0         , 0         , 0         ,        /* -620 */
@@ -619,7 +630,6 @@ t_bool tracing;
 ch_set_map ();                                          /* set dispatch map */
 if (!(cpu_model & (I_94|I_CT)))                         /* ~7094? MTM always on */
     mode_multi = 1;
-eamask = mode_storn? A704_MASK: AMASK;                  /* set eff addr mask */
 inst_base = inst_base & ~AMASK;                         /* A/B sel is 1b */
 data_base = data_base & ~AMASK;
 ind_reloc = ind_reloc & VA_BLK;                         /* canonical form */
@@ -650,14 +660,14 @@ while (reason == SCPE_OK) {                             /* loop until error */
         }
 
     if (sim_interval <= 0) {                            /* intv cnt expired? */
-        if (reason = sim_process_event ())              /* process events */
+        if ((reason = sim_process_event ()))            /* process events */
             break;
         chtr_pend = chtr_eval (NULL);                   /* eval chan traps */
         }
 
     for (i = 0; ch_req && (i < NUM_CHAN); i++) {        /* loop thru channels */
         if (ch_req & REQ_CH (i)) {                      /* channel request? */
-            if (reason = ch_proc (i))
+            if ((reason = ch_proc (i)))
                 break;
             }
         chtr_pend = chtr_eval (NULL);
@@ -683,8 +693,12 @@ while (reason == SCPE_OK) {                             /* loop until error */
             chtr_inhi = 0;                              /* clear */
             chtr_pend = chtr_eval (NULL);               /* re-evaluate */
             }
+        else if (cpu_model & I_CT) {                    /* CTSS? */
+            mode_user = user_buf;                       /* load modes from buffers */
+            mode_reloc = reloc_buf;
+            }
         oldPC = PC;                                     /* save current PC */
-        PC = (PC + 1) & eamask;                         /* increment PC */
+        PC = (PC + 1) & EAMASK;                         /* increment PC */
         if (!ReadI (oldPC, &IR))                        /* get inst; trap? */
             continue;
         }
@@ -695,7 +709,7 @@ while (reason == SCPE_OK) {                             /* loop until error */
     XEC:
 
     tag = GET_TAG (IR);                                 /* get tag */
-    addr = (uint32) IR & eamask;                        /* get base addr */
+    addr = (uint32) IR & EAMASK;                        /* get base addr */
 
 /* Decrement format instructions */
 
@@ -788,15 +802,15 @@ while (reason == SCPE_OK) {                             /* loop until error */
             continue;
             }
         if (tag && (fl & I_X))                          /* tag and indexable? */
-            ea = (addr - get_xri (tag)) & eamask;       /* do indexing */
+            ea = (addr - get_xri (tag)) & EAMASK;       /* do indexing */
         else ea = addr;
         if (TST_IND (IR) && (fl & I_N)) {               /* indirect? */
             if (!ReadI (ea, &SR))                       /* get ind; trap? */
                 continue;
-            addr = (uint32) SR & eamask;                /* get address */
+            addr = (uint32) SR & EAMASK;                /* get address */
             tagi = GET_TAG (SR);                        /* get tag */
             if (tagi)                                   /* tag? */
-                ea = (addr - get_xri (tagi)) & eamask;  /* do indexing */
+                ea = (addr - get_xri (tagi)) & EAMASK;  /* do indexing */
             else ea = addr;
             }
         if ((fl & I_R) && !Read (ea, &SR))              /* read opnd; trap? */
@@ -901,7 +915,7 @@ while (reason == SCPE_OK) {                             /* loop until error */
         case 00054:                                     /* RFT */
             t = IR & RMASK;
             if ((SI & t) == 0)                          /* if ind off, skip */
-                PC = (PC + 1) & eamask;
+                PC = (PC + 1) & EAMASK;
             break;
 
         case 00055:                                     /* SIR */
@@ -911,7 +925,7 @@ while (reason == SCPE_OK) {                             /* loop until error */
         case 00056:                                     /* RNT */
             t = IR & RMASK;
             if ((SI & t) == t)                          /* if ind on, skip */
-                PC = (PC + 1) & eamask;
+                PC = (PC + 1) & EAMASK;
             break;
 
         case 00057:                                     /* RIR */
@@ -943,20 +957,26 @@ while (reason == SCPE_OK) {                             /* loop until error */
         case 00101:                                     /* (CTSS) TIA */
             if (prot_trap (0))                          /* not user mode? */
                 break;
-            PCQ_ENTRY;
-            PC = ea;
-            inst_base = 0;
+            if (mode_ttrap) {                           /* trap? */
+                WriteTA (TRAP_STD_SAV, oldPC);          /* save PC */
+                TrapXfr (TRAP_TRA_PC);                  /* trap */
+                }
+            else {
+                PCQ_ENTRY;
+                PC = ea;
+                inst_base = 0;
+                }
             break;
 
         case 00114: case 00115: case 00116: case 00117: /* CVR */
             sc = GET_CCNT (IR);
             SR = ea;
             while (sc) {
-                ea = (uint32) ((AC & 077) + SR) & eamask;
+                ea = (uint32) ((AC & 077) + SR) & EAMASK;
                 if (!Read (ea, &SR))
                     break;
                 AC = (AC & AC_S) | ((AC >> 6) & 0017777777777) |
-                    (SR & 0770000000000);
+                    (SR & INT64_C(0770000000000));
                 sc--;
                 }
             if ((sc == 0) && (IR & INST_T_CXR1))
@@ -1152,12 +1172,12 @@ while (reason == SCPE_OK) {                             /* loop until error */
             t2 = SR & MMASK;
             if (s1 ^ s2) {                              /* diff signs? */
                 if (s1)                                 /* AC < mem? skip 2 */
-                    PC = (PC + 2) & eamask;
+                    PC = (PC + 2) & EAMASK;
                 }
             else if (t1 == t2)                          /* equal? skip 1 */
-                PC = (PC + 1) & eamask;
+                PC = (PC + 1) & EAMASK;
             else if ((t1 < t2) ^ s1)                    /* AC < mem, AC +, or */
-                PC = (PC + 2) & eamask;                 /* AC > mem, AC -? */
+                PC = (PC + 2) & EAMASK;                 /* AC > mem, AC -? */
             break;
 
         case 00361:                                     /* ACL */
@@ -1208,7 +1228,7 @@ while (reason == SCPE_OK) {                             /* loop until error */
 
         case 00444:                                     /* OFT */
             if ((SI & SR) == 0)                         /* skip if ind off */
-                PC = (PC + 1) & eamask;
+                PC = (PC + 1) & EAMASK;
             break;
 
         case 00445:                                     /* RIS */
@@ -1217,7 +1237,7 @@ while (reason == SCPE_OK) {                             /* loop until error */
 
         case 00446:                                     /* ONT */
             if ((SI & SR) == SR)                        /* skip if ind on */
-                PC = (PC + 1) & eamask;
+                PC = (PC + 1) & EAMASK;
             break;
 
         case 00460:                                     /* LDA (704) */
@@ -1234,7 +1254,7 @@ while (reason == SCPE_OK) {                             /* loop until error */
 
         case 00520:                                     /* ZET */
             if ((SR & MMASK) == 0)                      /* skip if M 1-35 = 0 */
-                PC = (PC + 1) & eamask;
+                PC = (PC + 1) & EAMASK;
             break;
 
         case 00522:                                     /* XEC */
@@ -1265,6 +1285,9 @@ while (reason == SCPE_OK) {                             /* loop until error */
             if (prot_trap (0))                          /* user mode? */
                 break;
             ind_reloc = ((uint32) SR) & VA_BLK;
+            reloc_buf = 1;                              /* set mode buffer */
+            chtr_inhi = 1;                              /* delay traps */
+            chtr_pend = 0;                              /* no trap now */
             break;
 
         case 00564:                                     /* ENB */
@@ -1348,8 +1371,6 @@ while (reason == SCPE_OK) {                             /* loop until error */
             break;
 
         case 00760:                                     /* PSE */
-            if (prot_trap (0))                          /* user mode? */
-                break;
             reason = op_pse (ea);
             break;
 
@@ -1380,6 +1401,8 @@ while (reason == SCPE_OK) {                             /* loop until error */
 /* Negative instructions */
 
         case 01021:                                     /* ESNT */
+            if (prot_trap (0))                          /* user mode? */
+                break;
             mode_storn = 1;                             /* enter nullification */
             PCQ_ENTRY;
             PC = ea;                                    /* branch, no trap */
@@ -1400,7 +1423,7 @@ while (reason == SCPE_OK) {                             /* loop until error */
         case 01054:                                     /* LFT */
             t = (IR & RMASK) << 18;
             if ((SI & t) == 0)                          /* if ind off, skip */
-                PC = (PC + 1) & eamask;
+                PC = (PC + 1) & EAMASK;
             break;
 
         case 01055:                                     /* SIL */
@@ -1410,7 +1433,7 @@ while (reason == SCPE_OK) {                             /* loop until error */
         case 01056:                                     /* LNT */
             t = (IR & RMASK) << 18;
             if ((SI & t) == t)                          /* if ind on, skip */
-                PC = (PC + 1) & eamask;
+                PC = (PC + 1) & EAMASK;
             break;
 
         case 01057:                                     /* RIL */
@@ -1431,17 +1454,22 @@ while (reason == SCPE_OK) {                             /* loop until error */
         case 01101:                                     /* (CTSS) TIB */
             if (prot_trap (0))                          /* user mode? */
                 break;
-            PCQ_ENTRY;
-            PC = ea;
-            mode_user = 1;
-            inst_base = BCORE_BASE;
+            if (mode_ttrap) {                           /* trap? */
+                WriteTA (TRAP_STD_SAV, oldPC);          /* save PC */
+                TrapXfr (TRAP_TRA_PC);                  /* trap */
+                }
+            else {
+                PCQ_ENTRY;
+                PC = ea;
+                inst_base = BCORE_BASE;
+                }
             break;
 
         case 01114: case 01115: case 01116: case 01117: /* CAQ */
             sc = GET_CCNT (IR);
             SR = ea;
             while (sc) {
-                ea = (uint32) ((MQ >> 30) + SR) & eamask;
+                ea = (uint32) ((MQ >> 30) + SR) & EAMASK;
                 if (!Read (ea, &SR))
                     break;
                 MQ = ((MQ << 6) & DMASK) | (MQ >> 30);
@@ -1485,7 +1513,7 @@ while (reason == SCPE_OK) {                             /* loop until error */
             sc = GET_CCNT (IR);
             SR = ea;
             while (sc) {
-                ea = (uint32) ((MQ >> 30) + SR) & eamask;
+                ea = (uint32) ((MQ >> 30) + SR) & EAMASK;
                 if (!Read (ea, &SR))
                     break;
                 MQ = ((MQ << 6) & DMASK) | (SR >> 30);
@@ -1587,9 +1615,9 @@ while (reason == SCPE_OK) {                             /* loop until error */
         case 01340:                                     /* LAS */
             t = AC & AC_MMASK;                          /* AC Q,P,1-35 */
             if (t < SR)
-                PC = (PC + 2) & eamask;
+                PC = (PC + 2) & EAMASK;
             else if (t == SR)
-                PC = (PC + 1) & eamask;
+                PC = (PC + 1) & EAMASK;
             break;
 
         case 01400:                                     /* SBM */
@@ -1606,7 +1634,7 @@ while (reason == SCPE_OK) {                             /* loop until error */
 
         case 01520:                                     /* NZT */
             if ((SR & MMASK) != 0)
-                PC = (PC + 1) & eamask;
+                PC = (PC + 1) & EAMASK;
             break;
 
         case 01534:                                     /* LXD */
@@ -1624,12 +1652,21 @@ while (reason == SCPE_OK) {                             /* loop until error */
                 break;
             ind_start = ((uint32) SR) & VA_BLK;
             ind_limit = (GET_DEC (SR) & VA_BLK) | VA_OFF;
+            user_buf = 1;                               /* set mode buffer */
+            chtr_inhi = 1;                              /* delay traps */
+            chtr_pend = 0;                              /* no trap now */
             break;
 
         case 01600:                                     /* STQ */
             Write (ea, MQ);
             break;
 
+        case 01601:                                     /* SRI (CTSS) */
+            SR = ind_reloc & VA_BLK;
+            /* add reloc mode in bit 1 */
+            Write (ea, SR);
+            break;
+
         case 01602:                                     /* ORS */
             SR = SR | (AC & DMASK);
             Write (ea, SR);
@@ -1639,7 +1676,14 @@ while (reason == SCPE_OK) {                             /* loop until error */
             SR = (AC & MMASK) | ((AC & AC_S)? SIGN: 0);
             if (!Write (ea, SR))
                 break;
-            Write ((ea + 1) & eamask, MQ);
+            Write ((ea + 1) & EAMASK, MQ);
+            break;
+
+        case 01604:                                     /* SPI (CTSS) */
+            SR = (((t_uint64) (ind_limit & VA_BLK)) << INST_V_DEC) |
+                ((t_uint64) (ind_start & VA_BLK));
+            /* add prot mode in bit 2 */
+            Write (ea, SR);
             break;
 
         case 01620:                                     /* SLQ */
@@ -1687,8 +1731,6 @@ while (reason == SCPE_OK) {                             /* loop until error */
             break;                                      /* -xr -> AC decr */
 
         case 01760:                                     /* MSE */
-            if (prot_trap (0))                          /* user mode? */
-                break;
             reason = op_mse (ea);
             break;
 
@@ -1701,11 +1743,11 @@ while (reason == SCPE_OK) {                             /* loop until error */
                 data_base = BCORE_BASE;
             else if (ea == 043) {                       /* IFT? */
                 if (inst_base == 0)
-                    PC = (PC + 1) & eamask;
+                    PC = (PC + 1) & EAMASK;
                 }
             else if (ea == 044) {                       /* EFT? */
                 if (data_base == 0)
-                    PC = (PC + 1) & eamask;
+                    PC = (PC + 1) & EAMASK;
                 }
             else if (stop_illop)
                 reason = STOP_ILLEG;
@@ -1835,10 +1877,8 @@ while (reason == SCPE_OK) {                             /* loop until error */
 
         case 00640: case 00641: case 00642: case 00643: /* SCHx */
         case 01640: case 01641: case 01642: case 01643:
-            if (prot_trap (0))                          /* user mode? */
-                break;
-             ch = ((op & 03) << 1) | ((op >> 9) & 01);
-             if ((reason = ch_op_store (ch, &SR)) == SCPE_OK)
+            ch = ((op & 03) << 1) | ((op >> 9) & 01);
+            if ((reason = ch_op_store (ch, &SR)) == SCPE_OK)
                 Write (ea, SR);
             break;
 
@@ -1850,7 +1890,7 @@ while (reason == SCPE_OK) {                             /* loop until error */
             break;
 
         case 00762:                                     /* RDS */
-            if (prot_trap (0) || sel_trap (PC))
+            if (sel_trap (0) || prot_trap (PC))         /* select takes priority */
                 break;
             ch = GET_U_CH (IR);
             reason = ch_op_ds (ch, CHSL_RDS, GET_U_UNIT (ea));
@@ -1858,7 +1898,7 @@ while (reason == SCPE_OK) {                             /* loop until error */
             break;
 
         case 00764:                                     /* BSR */
-            if (prot_trap (0) || sel_trap (PC))
+            if (sel_trap (0) || prot_trap (PC))         /* select takes priority */
                 break;
             ch = GET_U_CH (IR);
             reason = ch_op_nds (ch, CHSL_BSR, GET_U_UNIT (ea));
@@ -1866,7 +1906,7 @@ while (reason == SCPE_OK) {                             /* loop until error */
             break;
 
         case 00766:                                     /* WRS */
-            if (prot_trap (0) || sel_trap (PC))
+            if (sel_trap (0) || prot_trap (PC))         /* select takes priority */
                 break;
             ch = GET_U_CH (IR);
             reason = ch_op_ds (ch, CHSL_WRS, GET_U_UNIT (ea));
@@ -1874,7 +1914,7 @@ while (reason == SCPE_OK) {                             /* loop until error */
             break;
 
         case 00770:                                     /* WEF */
-            if (prot_trap (0) || sel_trap (PC))
+            if (sel_trap (0) || prot_trap (PC))         /* select takes priority */
                 break;
             ch = GET_U_CH (IR);
             reason = ch_op_nds (ch, CHSL_WEF, GET_U_UNIT (ea));
@@ -1882,7 +1922,7 @@ while (reason == SCPE_OK) {                             /* loop until error */
             break;
 
         case 00772:                                     /* REW */
-            if (prot_trap (0) || sel_trap (PC))
+            if (sel_trap (0) || prot_trap (PC))         /* select takes priority */
                 break;
             ch = GET_U_CH (IR);
             reason = ch_op_nds (ch, CHSL_REW, GET_U_UNIT (ea));
@@ -1890,7 +1930,7 @@ while (reason == SCPE_OK) {                             /* loop until error */
             break;
 
         case 01764:                                     /* BSF */
-            if (prot_trap (0) || sel_trap (PC))
+            if (sel_trap (0) || prot_trap (PC))         /* select takes priority */
                 break;
             ch = GET_U_CH (IR);
             reason = ch_op_nds (ch, CHSL_BSF, GET_U_UNIT (ea));
@@ -1898,7 +1938,7 @@ while (reason == SCPE_OK) {                             /* loop until error */
             break;
 
         case 01772:                                     /* RUN */
-            if (prot_trap (0) || sel_trap (PC))
+            if (sel_trap (0) || prot_trap (PC))         /* select takes priority */
                 break;
             ch = GET_U_CH (IR);
             reason = ch_op_nds (ch, CHSL_RUN, GET_U_UNIT (ea));
@@ -1906,7 +1946,7 @@ while (reason == SCPE_OK) {                             /* loop until error */
             break;
 
         case 00776:                                     /* SDN */
-            if (prot_trap (0) || sel_trap (PC))
+            if (sel_trap (0) || prot_trap (PC))         /* select takes priority */
                 break;
             ch = GET_U_CH (IR);
             reason = ch_op_nds (ch, CHSL_SDN, GET_U_UNIT (ea));
@@ -1929,13 +1969,13 @@ while (reason == SCPE_OK) {                             /* loop until error */
             t_stat r;
             for (i = 0; (i < HALT_IO_LIMIT) && !ch_qidle (); i++) {
                 sim_interval = 0;
-                if (r = sim_process_event ())           /* process events */
+                if ((r = sim_process_event ()))         /* process events */
                     return r;
                 chtr_pend = chtr_eval (NULL);           /* eval chan traps */
                 while (ch_req) {                        /* until no ch req */
                     for (j = 0; j < NUM_CHAN; j++) {    /* loop thru channels */
                         if (ch_req & REQ_CH (j)) {      /* channel request? */
-                            if (r = ch_proc (j))
+                            if ((r = ch_proc (j)))
                                 return r;
                             }
                         chtr_pend = chtr_eval (NULL);
@@ -1967,9 +2007,9 @@ if (tag) {
             r = r | XR[2];
         if (tag & 4)
             r = r | XR[4];
-        return r & eamask;
+        return r & EAMASK;
         }
-    return XR[tag] & eamask;
+    return XR[tag] & EAMASK;
     }
 return 0;
 }
@@ -1994,9 +2034,9 @@ if (tag) {
         if (tag & 4)
             r = r | XR[4];
         put_xr (tag, r);
-        return r & eamask;
+        return r & EAMASK;
         }
-    return XR[tag] & eamask;
+    return XR[tag] & EAMASK;
     }
 return 0;
 }
@@ -2006,7 +2046,7 @@ return 0;
 void put_xr (uint32 tag, uint32 dat)
 {
 tag = tag & INST_M_TAG;
-dat = dat & eamask;
+dat = dat & EAMASK;
 
 if (tag) {
     if (mode_multi) {
@@ -2071,7 +2111,8 @@ WriteP (pa, mem);
 mode_ctrap = 0;
 mode_strap = 0;
 mode_storn = 0;
-mode_user = 0;
+mode_user = user_buf = 0;
+mode_reloc = reloc_buf = 0;
 inst_base = 0;
 data_base = 0;
 return;
@@ -2123,7 +2164,8 @@ PC = newpc;
 mode_ctrap = 0;
 mode_strap = 0;
 mode_storn = 0;
-mode_user = 0;
+mode_user = user_buf = 0;
+mode_reloc = reloc_buf = 0;
 inst_base = 0;
 data_base = 0;
 return;
@@ -2133,12 +2175,11 @@ return;
 
 t_bool ReadI (uint32 va, t_uint64 *val)
 {
-if (mode_user) {
+if (mode_reloc)
     va = (va + ind_reloc) & AMASK;
-    if ((va < ind_start) || (va > ind_limit)) {
-        prot_trap (0);
-        return FALSE;
-        }
+if (mode_user && ((va < ind_start) || (va > ind_limit))) {
+    prot_trap (0);
+    return FALSE;
     }
 *val = M[va | inst_base];
 return TRUE;
@@ -2148,12 +2189,11 @@ return TRUE;
 
 t_bool Read (uint32 va, t_uint64 *val)
 {
-if (mode_user) {
+if (mode_reloc)
     va = (va + ind_reloc) & AMASK;
-    if ((va < ind_start) || (va > ind_limit))  {
-        prot_trap (0);
-        return FALSE;
-        }
+if (mode_user && ((va < ind_start) || (va > ind_limit))) {
+    prot_trap (0);
+    return FALSE;
     }
 *val = M[va | data_base];
 return TRUE;
@@ -2163,12 +2203,11 @@ return TRUE;
 
 t_bool Write (uint32 va, t_uint64 dat)
 {
-if (mode_user) {
+if (mode_reloc)
     va = (va + ind_reloc) & AMASK;
-    if ((va < ind_start) || (va > ind_limit))  {
-        prot_trap (0);
-        return FALSE;
-        }
+if (mode_user && ((va < ind_start) || (va > ind_limit))) {
+    prot_trap (0);
+    return FALSE;
     }
 M[va | data_base] = dat;
 return TRUE;
@@ -2193,7 +2232,8 @@ mode_storn = 0;
 if (cpu_model & (I_94|I_CT))
     mode_multi = 0;
 else mode_multi = 1;
-mode_user = 0;
+mode_user = user_buf = 0;
+mode_reloc = reloc_buf = 0;
 inst_base = 0;
 data_base = 0;
 ch_req = 0;
@@ -2222,8 +2262,7 @@ if (vptr == NULL)
     return SCPE_ARG;
 if ((sw & (SWMASK ('A') | SWMASK ('B')))? (ea > AMASK): (ea >= MEMSIZE))
     return SCPE_NXM;
-if ((sw & SWMASK ('B')) ||
-    ((sw & SWMASK ('V')) && mode_user && inst_base))
+if (sw & SWMASK ('B'))
     ea = ea | BCORE_BASE;
 *vptr = M[ea] & DMASK;
 return SCPE_OK;
@@ -2362,14 +2401,12 @@ t_stat cpu_fprint_one_inst (FILE *st, uint32 pc, uint32 rpt, uint32 ea,
     t_uint64 ir, t_uint64 ac, t_uint64 mq, t_uint64 si, t_uint64 opnd)
 {
 int32 ch;
-t_value sim_eval;
-extern t_stat fprint_sym (FILE *ofile, t_addr addr, t_value *val,
-    UNIT *uptr, int32 sw);
+extern t_value *sim_eval;
 
-sim_eval = ir;
+sim_eval[0] = ir;
 if (pc & HIST_PC) {                                     /* instruction? */
     fputs ("CPU ", st);
-    fprintf (st, "%05o ", pc & AMASK);
+    fprintf (st, "%05o ", (int)(pc & AMASK));
     if (rpt == 0)
         fprintf (st, "       ");
     else if (rpt < 1000000)
@@ -2384,7 +2421,7 @@ if (pc & HIST_PC) {                                     /* instruction? */
     if (ir & INST_T_DEC)
         fprintf (st, "       ");
     else fprintf (st, "%05o  ", ea);
-    if (fprint_sym (st, pc & AMASK, &sim_eval, &cpu_unit, SWMASK ('M')) > 0) {
+    if (fprint_sym (st, pc & AMASK, sim_eval, &cpu_unit, SWMASK ('M')) > 0) {
         fputs ("(undefined) ", st);
         fprint_val (st, ir, 8, 36, PV_RZRO);
         }
@@ -2395,12 +2432,12 @@ if (pc & HIST_PC) {                                     /* instruction? */
         }
     fputc ('\n', st);                                   /* end line */
     }                                                   /* end if instruction */
-else if (ch = HIST_CH (pc)) {                           /* channel? */
+else if ((ch = HIST_CH (pc))) {                         /* channel? */
     fprintf (st, "CH%c ", 'A' + ch - 1);
-    fprintf (st, "%05o  ", pc & AMASK);
+    fprintf (st, "%05o  ", (int)(pc & AMASK));
     fputs ("                                              ", st);
-    fprintf (st, "%05o  ", ea & AMASK);
-    if (fprint_sym (st, pc & AMASK, &sim_eval, &cpu_unit,
+    fprintf (st, "%05o  ", (int)(ea & AMASK));
+    if (fprint_sym (st, pc & AMASK, sim_eval, &cpu_unit,
         (ch_dev[ch - 1].flags & DEV_7909)? SWMASK ('N'): SWMASK ('I')) > 0) {
         fputs ("(undefined) ", st);
         fprint_val (st, ir, 8, 36, PV_RZRO);
