@@ -1,6 +1,6 @@
 /* i7094_defs.h: IBM 7094 simulator definitions
 
-   Copyright (c) 2003-2008, Robert M Supnik
+   Copyright (c) 2003-2011, Robert M Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -26,13 +26,22 @@
    This simulator incorporates prior work by Paul Pierce, Dave Pitts, and Rob
    Storey.  Tom Van Vleck, Stan Dunten, Jerry Saltzer, and other CTSS veterans
    helped to reconstruct the CTSS hardware RPQ's.  Dave Pitts gets special
-   thanks for patiently coaching me through IBSYS debug. */
+   thanks for patiently coaching me through IBSYS debug.
+   
+   25-Mar-11    RMS     Updated SDC mask based on 7230 documentation
+   22-May-10    RMS     Added check for 64b addresses
 
-#ifndef _I7094_DEFS_H_
-#define _I7094_DEFS_H_  0
+*/
+
+#ifndef I7094_DEFS_H_
+#define I7094_DEFS_H_  0
 
 #include "sim_defs.h"                                   /* simulator defns */
 
+#if defined(USE_ADDR64)
+#error "7094 does not support 64b addresses!"
+#endif
+
 /* Simulator stop codes */
 
 #define STOP_HALT       1                               /* halted */
@@ -115,25 +124,25 @@ typedef struct {
 
 /* Integer */
 
-#define DMASK           0777777777777                   /* data mask */
-#define SIGN            0400000000000                   /* sign */
-#define MMASK           0377777777777                   /* magnitude mask */
-#define LMASK           0777777000000                   /* left mask */
-#define RMASK           0000000777777                   /* right mask */
-#define PMASK           0700000000000                   /* prefix */
-#define XMASK           0077777000000                   /* decrement */
-#define TMASK           0000000700000                   /* tag */
-#define AMASK           0000000077777                   /* address */
-#define SCMASK          0000000000377                   /* shift count mask */
-#define B1              0200000000000                   /* bit 1 */
-#define B9              0000400000000                   /* bit 9 */
+#define DMASK           INT64_C(0777777777777)          /* data mask */
+#define SIGN            INT64_C(0400000000000)          /* sign */
+#define MMASK           INT64_C(0377777777777)          /* magnitude mask */
+#define LMASK           INT64_C(0777777000000)          /* left mask */
+#define RMASK           INT64_C(0000000777777)          /* right mask */
+#define PMASK           INT64_C(0700000000000)          /* prefix */
+#define XMASK           INT64_C(0077777000000)          /* decrement */
+#define TMASK           INT64_C(0000000700000)          /* tag */
+#define AMASK           INT64_C(0000000077777)          /* address */
+#define SCMASK          INT64_C(0000000000377)          /* shift count mask */
+#define B1              INT64_C(0200000000000)          /* bit 1 */
+#define B9              INT64_C(0000400000000)          /* bit 9 */
 
 /* Accumulator is actually 38b wide */
 
-#define AC_S            02000000000000                  /* sign */
-#define AC_Q            01000000000000                  /* Q */
-#define AC_P            00400000000000                  /* P */
-#define AC_MMASK        01777777777777                  /* Q+P+magnitude */
+#define AC_S            INT64_C(02000000000000)         /* sign */
+#define AC_Q            INT64_C(01000000000000)         /* Q */
+#define AC_P            INT64_C(00400000000000)         /* P */
+#define AC_MMASK        INT64_C(01777777777777)         /* Q+P+magnitude */
 
 /* Floating point */
 
@@ -150,8 +159,8 @@ typedef struct {
 
 /* Instruction format */
 
-#define INST_T_DEC      0300000000000                   /* if nz, takes decr */
-#define INST_T_CXR1     0000000100000                   /* if nz, update XR1 */
+#define INST_T_DEC      INT64_C(0300000000000)          /* if nz, takes decr */
+#define INST_T_CXR1     INT64_C(0000000100000)          /* if nz, update XR1 */
 #define INST_V_OPD      33                              /* decrement opcode */
 #define INST_M_OPD      07
 #define INST_V_DEC      18                              /* decrement */
@@ -168,6 +177,8 @@ typedef struct {
 #define INST_M_TAG      07
 #define INST_V_ADDR     0
 #define INST_M_ADDR     077777
+#define INST_V_4B        0
+#define INST_M_4B        017
 
 #define GET_OPD(x)      ((uint32) (((x) >> INST_V_OPD) & INST_M_OPD))
 #define GET_DEC(x)      ((uint32) (((x) >> INST_V_DEC) & INST_M_DEC))
@@ -408,8 +419,8 @@ typedef struct {
 #define CHF_V_LCC       30                              /* loop ctrl counter */
 #define CHF_M_LCC       077
 
-#define CHF_CLR_7909    07775000177                     /* 7909 clear flags */
-#define CHF_SDC_7909    07776000000                     /* 7909 SDC flags */
+#define CHF_CLR_7909    INT64_C(07775000177)            /* 7909 clear flags */
+#define CHF_SDC_7909    INT64_C(07777600000)            /* 7909 SDC flags */
 
 /* Channel characteristics (in dev.flags) */
 
