@@ -1,6 +1,6 @@
 /* id_defs.h: Interdata 16b/32b simulator definitions
 
-   Copyright (c) 2000-2006, Robert M. Supnik
+   Copyright (c) 2000-2012, Robert M. Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -26,6 +26,8 @@
    The author gratefully acknowledges the help of Carl Friend and Al Kossow,
    who provided key documents about the Interdata product line.
 
+   18-Apr-12    RMS     Added clock coschedule prototype
+   22-May-10    RMS     Added check for 64b definitions
    09-Mar-06    RMS     Increased register sets to architectural limit
    25-Jan-04    RMS     Removed local logging support
    22-Sep-03    RMS     Added additional instruction decode types
@@ -34,11 +36,15 @@
    28-Feb-03    RMS     Changed magtape device default to 0x85
 */
 
-#ifndef _ID_DEFS_H_
-#define _ID_DEFS_H_     0
+#ifndef ID_DEFS_H_
+#define ID_DEFS_H_     0
 
 #include "sim_defs.h"                                   /* simulator defns */
 
+#if defined(USE_INT64) || defined(USE_ADDR64)
+#error "Interdata 16/32 does not support 64b values!"
+#endif
+
 /* Simulator stop codes */
 
 #define STOP_RSRV       1                               /* undef instr */
@@ -478,4 +484,6 @@ t_stat set_dev (UNIT *uptr, int32 val, char *cptr, void *desc);
 t_stat show_sch (FILE *st, UNIT *uptr, int32 val, void *desc);
 t_stat show_dev (FILE *st, UNIT *uptr, int32 val, void *desc);
 
+int32 lfc_cosched (int32 wait);
+
 #endif
