@@ -1,6 +1,6 @@
 /* id_fd.c: Interdata floppy disk simulator
 
-   Copyright (c) 2001-2008, Robert M Supnik
+   Copyright (c) 2001-2013, Robert M Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -25,6 +25,9 @@
 
    fd           M46-630 floppy disk
 
+   03-Sep-13    RMS     Added explicit void * cast
+   19-Mar-12    RMS     Fixed macro naming conflict (Mark Pizzolato)
+
    A diskette consists of 77 tracks, each with 26 sectors of 128B.  The
    Interdata floppy uses a logical record numbering scheme from 1 to 2002.
    Physical tracks are numbered 0-76, physical sectors 1-26.
@@ -75,11 +78,11 @@
 
 #define STA_WRP         0x80                            /* *write prot */
 #define STA_DEF         0x40                            /* def track NI */
-#define STA_DEL         0x20                            /* del record */
+#define STA_DLR         0x20                            /* del record */
 #define STA_ERR         0x10                            /* error */
 #define STA_IDL         0x02                            /* idle */
 #define STA_OFL         0x01                            /* fault */
-#define STA_MASK        (STA_DEF|STA_DEL|STA_ERR|STA_BSY|STA_IDL)
+#define STA_MASK        (STA_DEF|STA_DLR|STA_ERR|STA_BSY|STA_IDL)
 #define SET_EX          (STA_ERR)                       /* set EX */
 
 /* Extended status, 6 bytes, * = dynamic */
@@ -113,7 +116,7 @@ uint32 fd_cmd = 0;                                      /* command */
 uint32 fd_db = 0;                                       /* data buffer */
 uint32 fd_bptr = 0;                                     /* buffer pointer */
 uint8 fdxb[FD_NUMBY] = { 0 };                           /* sector buffer */
-uint8 fd_es[FD_NUMDR][ES_SIZE] = { 0 };                 /* ext status */
+uint8 fd_es[FD_NUMDR][ES_SIZE] = { {0} };               /* ext status */
 uint32 fd_lrn = 0;                                      /* log rec # */
 uint32 fd_wdv = 0;                                      /* wd valid */
 uint32 fd_stopioe = 1;                                  /* stop on error */
@@ -224,7 +227,7 @@ switch (op) {                                           /* case IO op */
                 fd_sta = fd_sta | STA_BSY;              /* set busy */
                 }
             else fd_bptr = 0;                           /* just wrap */
-			}
+            }
         if ((ctab[fnc] & C_RD) && fd_arm)               /* if rd & arm, */
             SET_INT (v_FD);                             /* interrupt */
         return fd_db;                                   /* return buf */
@@ -304,7 +307,7 @@ return 0;
 t_stat fd_svc (UNIT *uptr)
 {
 uint32 i, u, tk, sc, crc, fnc, da;
-uint8 *fbuf = uptr->filebuf;
+uint8 *fbuf = (uint8 *) uptr->filebuf;
 
 u = uptr - fd_dev.units;                                /* get unit number */
 fnc = GET_FNC (uptr->FNC);                              /* get function */
@@ -329,7 +332,7 @@ switch (fnc) {                                          /* case on function */
         for (i = 0; i < FD_NUMBY; i++)                  /* read sector */
             fdxb[i] = fbuf[da + i];
         if (fbuf[FD_SIZE  + uptr->LRN - 1]) {           /* deleted? set err */
-            fd_sta = fd_sta | STA_DEL;
+            fd_sta = fd_sta | STA_DLR;
             fd_es[u][0] = fd_es[u][0] | ES0_DEL;
             }
         fd_es[u][2] = GET_SEC (uptr->LRN);              /* set ext sec/trk */
@@ -364,7 +367,7 @@ switch (fnc) {                                          /* case on function */
         if ((uptr->flags & UNIT_BUF) == 0) {            /* not attached? */
             fd_es[u][0] = fd_es[u][0] | ES0_FLT;        /* set err */
             fd_es[u][1] = fd_es[u][1] | ES1_NRDY;
-			}
+            }
         for (i = 0; i < ES_SIZE; i++)                   /* copy to buf */
             fdxb[i] = fd_es[u][i];
         for (i = ES_SIZE; i < FD_NUMBY; i++)
@@ -375,7 +378,7 @@ switch (fnc) {                                          /* case on function */
         if ((uptr->flags & UNIT_BUF) == 0) {            /* not attached? */
             fd_done (u, STA_ERR, ES0_ERR | ES0_FLT, ES1_NRDY);
             return SCPE_OK;
-			}
+            }
         for (i = 0; i < FD_NUMBY; i++) fdxb[i] = 0;     /* clr buf */
         tk = GET_TRK (uptr->LRN);                       /* get track */
         sc = GET_SEC (uptr->LRN);                       /* get sector */
