@@ -168,7 +168,7 @@ DEVICE mt_dev = {
     MT_NUMDR, 10, 31, 1, 16, 8,
     NULL, NULL, &mt_reset,
     &mt_boot, &mt_attach, &mt_detach,
-    &mt_dib, DEV_DISABLE
+    &mt_dib, DEV_DISABLE | DEV_TAPE
     };
 
 /* Magtape: IO routine */
@@ -368,13 +368,13 @@ switch (uptr->UCMD) {                                   /* case on function */
             }
 
         if (mt_bptr) {                                  /* any chars? */
-            if (st = sim_tape_wrrecf (uptr, mtxb, mt_bptr)) /* write, err? */
+            if ((st = sim_tape_wrrecf (uptr, mtxb, mt_bptr)))/* write, err? */
                 r = mt_map_err (uptr, st);              /* map error */
             }
         break;                                          /* record done */
 
     case MTC_WEOF:                                      /* write eof */
-        if (st = sim_tape_wrtmk (uptr))                 /* write tmk, err? */
+        if ((st = sim_tape_wrtmk (uptr)))               /* write tmk, err? */
             r = mt_map_err (uptr, st);                  /* map error */
         mt_sta = mt_sta | STA_EOF;                      /* set eof */
         if (mt_arm[u])                                  /* set intr */
@@ -402,7 +402,7 @@ switch (uptr->UCMD) {                                   /* case on function */
         break;
 
     case MTC_SPCR:                                      /* backspace */
-        if (st = sim_tape_sprecr (uptr, &tbc))          /* skip rec rev, err? */
+        if ((st = sim_tape_sprecr (uptr, &tbc)))        /* skip rec rev, err? */
             r = mt_map_err (uptr, st);                  /* map error */
         break;
         }                                               /* end case */
