@@ -1,6 +1,6 @@
 /* nova_lp.c: NOVA line printer simulator
 
-   Copyright (c) 1993-2005, Robert M. Supnik
+   Copyright (c) 1993-2008, Robert M. Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -25,15 +25,27 @@
 
    lpt          line printer
 
+   04-Jul-07    BKR     DEV_SET/CLR macros now used,
+                        <FF>, <CR>, <LF> output character delay now contingent upon non-zero TIME value,
+                        LPT can now be DISABLED
+   19-Jan-07    RMS     Added UNIT_TEXT
    25-Apr-03    RMS     Revised for extended file support
    30-May-02    RMS     Widened POS to 32b
+
+
+Notes:
+    - data currently masked to 7 bits.
+    - if register TIME is non-zero, then delay TIME events if <FF>, <CR> or <LF> seen
+    - register POS show the current file position
+    - register STOP_IOE determines return value issued if output to unattached LPT is attempted
 */
 
 #include "nova_defs.h"
 
 extern int32 int_req, dev_busy, dev_done, dev_disable;
 
-int32 lpt_stopioe = 0;                                  /* stop on error */
+
+int32 lpt_stopioe = 0;                                  /* stop on error flag */
 
 int32 lpt (int32 pulse, int32 code, int32 AC);
 t_stat lpt_svc (UNIT *uptr);
@@ -48,8 +60,8 @@ t_stat lpt_reset (DEVICE *dptr);
 
 DIB lpt_dib = { DEV_LPT, INT_LPT, PI_LPT, &lpt };
 
-UNIT lpt_unit = {
-    UDATA (&lpt_svc, UNIT_SEQ+UNIT_ATTABLE, 0), SERIAL_OUT_WAIT
+UNIT lpt_unit = {    /* 2007-May-30, bkr */
+    UDATA (&lpt_svc, UNIT_SEQ+UNIT_ATTABLE+UNIT_TEXT, 0), SERIAL_OUT_WAIT
     };
 
 REG lpt_reg[] = {
@@ -72,61 +84,72 @@ DEVICE lpt_dev = {
     &lpt_dib, DEV_DISABLE
     };
 
+
 /* IOT routine */
 
 int32 lpt (int32 pulse, int32 code, int32 AC)
 {
+if (code == ioDOA)
+    lpt_unit.buf = AC & 0177 ;
 
-if (code == ioDOA) lpt_unit.buf = AC & 0177;
-switch (pulse) {                                        /* decode IR<8:9> */
-
-    case iopS:                                          /* start */
-        dev_busy = dev_busy | INT_LPT;                  /* set busy */
-        dev_done = dev_done & ~INT_LPT;                 /* clear done, int */
-        int_req = int_req & ~INT_LPT;
-        if ((lpt_unit.buf != 015) && (lpt_unit.buf != 014) &&
-            (lpt_unit.buf != 012))
-            return (lpt_svc (&lpt_unit) << IOT_V_REASON);
+switch (pulse)
+    {                                                   /* decode IR<8:9> */
+  case iopS:                                            /* start */
+    DEV_SET_BUSY( INT_LPT ) ;
+    DEV_CLR_DONE( INT_LPT ) ;
+    DEV_UPDATE_INTR ;
+    if ( lpt_unit.wait )
+       if (  (lpt_unit.buf == 015)
+          || (lpt_unit.buf == 014)
+          || (lpt_unit.buf == 012)
+          )
+        {
         sim_activate (&lpt_unit, lpt_unit.wait);
-        break;
+        break ;
+        }
+    return (lpt_svc (&lpt_unit) << IOT_V_REASON);
+    break;
 
-    case iopC:                                          /* clear */
-        dev_busy = dev_busy & ~INT_LPT;                 /* clear busy */
-        dev_done = dev_done & ~INT_LPT;                 /* clear done, int */
-        int_req = int_req & ~INT_LPT;
-        sim_cancel (&lpt_unit);                         /* deactivate unit */
-        break;
-        }                                               /* end switch */
+  case iopC:                                            /* clear */
+    DEV_CLR_BUSY( INT_LPT ) ;
+    DEV_CLR_DONE( INT_LPT ) ;
+    DEV_UPDATE_INTR ;
+    sim_cancel (&lpt_unit);                             /* deactivate unit */
+    break;
+    }                                                   /* end switch */
 
 return 0;
 }
 
+
 /* Unit service */
 
 t_stat lpt_svc (UNIT *uptr)
 {
-dev_busy = dev_busy & ~INT_LPT;                         /* clear busy */
-dev_done = dev_done | INT_LPT;                          /* set done */
-int_req = (int_req & ~INT_DEV) | (dev_done & ~dev_disable);
+DEV_CLR_BUSY( INT_LPT ) ;
+DEV_SET_DONE( INT_LPT ) ;
+DEV_UPDATE_INTR ;
 if ((lpt_unit.flags & UNIT_ATT) == 0)                   /* attached? */
     return IORETURN (lpt_stopioe, SCPE_UNATT);
-if (putc (lpt_unit.buf, lpt_unit.fileref) == EOF) {
+fputc (uptr->buf, uptr->fileref);
+uptr->pos = ftell (uptr->fileref);
+if (ferror (uptr->fileref)) {
     perror ("LPT I/O error");
-    clearerr (lpt_unit.fileref);
+    clearerr (uptr->fileref);
     return SCPE_IOERR;
     }
-lpt_unit.pos = lpt_unit.pos + 1;
 return SCPE_OK;
 }
 
+
 /* Reset routine */
 
 t_stat lpt_reset (DEVICE *dptr)
 {
-lpt_unit.buf = 0;
-dev_busy = dev_busy & ~INT_LPT;                         /* clear busy */
-dev_done = dev_done & ~INT_LPT;                         /* clear done, int */
-int_req = int_req & ~INT_LPT;
+lpt_unit.buf = 0;                                       /* (not DG compatible) */
+DEV_CLR_BUSY( INT_LPT ) ;
+DEV_CLR_DONE( INT_LPT ) ;
+DEV_UPDATE_INTR ;
 sim_cancel (&lpt_unit);                                 /* deactivate unit */
 return SCPE_OK;
 }
