@@ -1,6 +1,6 @@
 /* pdp10_rp.c - RH11/RP04/05/06/07 RM02/03/05/80 "Massbus" disk controller
 
-   Copyright (c) 1993-2008, Robert M Supnik
+   Copyright (c) 1993-2017, Robert M Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -25,6 +25,9 @@
 
    rp           RH/RP/RM moving head disks
 
+   13-Mar-17    RMS     Annotated fall through in switch
+   17-Mar-13    RMS     Fixed incorrect copy/paste from pdp11_rp.c
+   08-Dec-12    RMS     UNLOAD does not set ATTN (Mark Pizzolato)
    12-Nov-05    RMS     Fixed DCLR not to clear drive address
    07-Jul-05    RMS     Removed extraneous externs
    18-Mar-05    RMS     Added attached test to detach routine
@@ -32,7 +35,7 @@
    04-Jan-04    RMS     Changed sim_fsize calling sequence
    23-Jul-03    RMS     Fixed bug in read header stub
    25-Apr-03    RMS     Revised for extended file support
-   21-Nov-02    RMS     Fixed bug in bootstrap (reported by Michael Thompson)
+   21-Nov-02    RMS     Fixed bug in bootstrap (Michael Thompson)
    29-Sep-02    RMS     Added variable vector support
                         New data structures
    30-Nov-01    RMS     Added read only unit, extended SET/SHOW support support
@@ -45,7 +48,7 @@
    28-Sep-01    RMS     Fixed interrupt handling for SC/ATA
    23-Aug-01    RMS     Added read/write header stubs for ITS
                         (found by Mirian Crzig Lennox) 
-   13-Jul-01    RMS     Changed fread call to fxread (found by Peter Schorn)
+   13-Jul-01    RMS     Changed fread call to fxread (Peter Schorn)
    14-May-01    RMS     Added check for unattached drive
 
    The "Massbus style" disks consisted of several different large
@@ -70,12 +73,14 @@
 
 #include "pdp10_defs.h"
 #include <math.h>
+#include <assert.h>
 
 #define RP_NUMDR        8                               /* #drives */
 #define RP_NUMWD        128                             /* 36b words/sector */
 #define RP_MAXFR        32768                           /* max transfer */
+#define SPINUP_DLY      (1000*1000)                     /* Spinup delay, usec */
 #define GET_SECTOR(x,d) ((int) fmod (sim_gtime() / ((double) (x)), \
-                    ((double) drv_tab[d].sect)))
+                        ((double) drv_tab[d].sect)))
 #define MBA_RP_CTRL     0                               /* RP drive */
 #define MBA_RM_CTRL     1                               /* RM drive */
 
@@ -85,7 +90,9 @@
 #define UNIT_V_DTYPE    (UNIT_V_UF + 1)                 /* disk type */
 #define UNIT_M_DTYPE    7
 #define UNIT_V_AUTO     (UNIT_V_UF + 4)                 /* autosize */
-#define UNIT_V_DUMMY    (UNIT_V_UF + 5)                 /* dummy flag */
+#define UNIT_V_UTS      (UNIT_V_UF + 5)                 /* Up to speed */
+#define UNIT_UTS        (1u << UNIT_V_UTS)
+#define UNIT_V_DUMMY    (UNIT_V_UF + 6)                 /* dummy flag */
 #define UNIT_WLK        (1 << UNIT_V_WLK)
 #define UNIT_DTYPE      (UNIT_M_DTYPE << UNIT_V_DTYPE)
 #define UNIT_AUTO       (1 << UNIT_V_AUTO)
@@ -238,9 +245,9 @@
 #define GET_DA(c,fs,d)  ((((GET_CY (c) * drv_tab[d].surf) + \
                         GET_SF (fs)) * drv_tab[d].sect) + GET_SC (fs))
 
-/* RPCC - 176736 - current cylinder */
-/* RPER2 - 176740 - error status 2 - drive unsafe conditions - unimplemented */
-/* RPER3 - 176742 - error status 3 - more unsafe conditions - unimplemented */
+/* RPCC -  176736 - current cylinder */
+/* RPER2 - 176740 - error status 2 - drive unsafe conditions */
+/* RPER3 - 176742 - error status 3 - more unsafe conditions */
 /* RPEC1 - 176744 - ECC status 1 - unimplemented */
 /* RPEC2 - 176746 - ECC status 2 - unimplemented */
 
@@ -331,6 +338,8 @@ extern int32 int_req;
 extern int32 ubmap[UBANUM][UMAP_MEMSIZE];               /* Unibus maps */
 extern int32 ubcs[UBANUM];
 extern UNIT cpu_unit;
+extern uint32 fe_bootrh;
+extern int32 fe_bootunit;
 
 int32 rpcs1 = 0;                                        /* control/status 1 */
 int32 rpwc = 0;                                         /* word count */
@@ -366,7 +375,7 @@ t_stat rp_reset (DEVICE *dptr);
 t_stat rp_boot (int32 unitno, DEVICE *dptr);
 t_stat rp_attach (UNIT *uptr, char *cptr);
 t_stat rp_detach (UNIT *uptr);
-void set_rper (int32 flag, int32 drv);
+void set_rper (int16 flag, int32 drv);
 void update_rpcs (int32 flags, int32 drv);
 void rp_go (int32 drv, int32 fnc);
 t_stat rp_set_size (UNIT *uptr, int32 val, char *cptr, void *desc);
@@ -609,12 +618,11 @@ return SCPE_OK;
 
 t_stat rp_wr (int32 data, int32 PA, int32 access)
 {
-int32 cs1f, drv, dtype, i, j;
+int32 cs1f, drv, i, j;
 UNIT *uptr;
 
 cs1f = 0;                                               /* no int on cs1 upd */
 drv = GET_UNIT (rpcs2);                                 /* get current unit */
-dtype = GET_DTYPE (rp_unit[drv].flags);                 /* get drive type */
 uptr = rp_dev.units + drv;                              /* get unit */
 j = (PA >> 1) & 037;                                    /* get reg offset */
 if (reg_in_drive[j] && (rp_unit[drv].flags & UNIT_DIS)) { /* nx disk */
@@ -622,12 +630,12 @@ if (reg_in_drive[j] && (rp_unit[drv].flags & UNIT_DIS)) { /* nx disk */
     update_rpcs (CS1_SC, drv);                          /* request intr */
     return SCPE_OK;
     }
-if (reg_in_drive[j] && sim_is_active (&rp_unit[drv])) { /* unit busy? */
+if (reg_in_drive[j] && sim_is_active (uptr) && (uptr->flags & UNIT_UTS)) { /* unit busy? */
     set_rper (ER1_RMR, drv);                            /* won't write */
     update_rpcs (0, drv);
     return SCPE_OK;
     }
-rmhr[drv] = data;
+rmhr[drv] = (uint16)data;
 
 switch (j) {                                            /* decode PA<5:1> */
 
@@ -650,7 +658,7 @@ switch (j) {                                            /* decode PA<5:1> */
                 rpcs2 = rpcs2 | CS2_NED;                /* set error flag */
                 cs1f = CS1_SC;                          /* req interrupt */
                 }
-            else if (sim_is_active (uptr))
+            else if (sim_is_active (uptr) && (uptr->flags & UNIT_UTS))
                 set_rper (ER1_RMR, drv);                /* won't write */
             else if (data & CS1_GO) {                   /* start op */
                 uptr->FUNC = GET_FNC (data);            /* set func */
@@ -679,7 +687,7 @@ switch (j) {                                            /* decode PA<5:1> */
     case 003:                                           /* RPDA */
         if ((access == WRITEB) && (PA & 1))
             data = data << 8;
-        rpda[drv] = data & ~DA_MBZ;
+        rpda[drv] = (uint16)(data & ~DA_MBZ);
         break;
 
     case 004:                                           /* RPCS2 */
@@ -700,7 +708,7 @@ switch (j) {                                            /* decode PA<5:1> */
     case 006:                                           /* RPER1 */
         if ((access == WRITEB) && (PA & 1))
             data = data << 8;
-        rper1[drv] = data;
+        rper1[drv] = (uint16)data;
         break;
 
     case 007:                                           /* RPAS */
@@ -721,17 +729,17 @@ switch (j) {                                            /* decode PA<5:1> */
     case 012:                                           /* RPMR */
         if ((access == WRITEB) && (PA & 1))
             data = data << 8;
-        rpmr[drv] = data;
+        rpmr[drv] = (uint16)data;
         break;
 
     case 015:                                           /* RPOF */
-        rpof[drv] = data & ~OF_MBZ;
+        rpof[drv] = (uint16)(data & ~OF_MBZ);
         break;
 
     case 016:                                           /* RPDC */
         if ((access == WRITEB) && (PA & 1))
             data = data << 8;
-        rpdc[drv] = data & ~DC_MBZ;
+        rpdc[drv] = (uint16)(data & ~DC_MBZ);
         break;
 
     case 005:                                           /* RPDS */
@@ -793,12 +801,16 @@ switch (fnc) {                                          /* case on function */
         rpda[drv] = 0;
         rpof[drv] = 0;                                  /* clear offset */
     case FNC_PACK:                                      /* pack acknowledge */
+        if ((uptr->flags & UNIT_UTS) == 0) {            /* not attached? */
+            set_rper (ER1_UNS, drv);                    /* unsafe */
+            break;
+            }
         rpds[drv] = rpds[drv] | DS_VV;                  /* set volume valid */
         return;
 
     case FNC_OFFSET:                                    /* offset mode */
     case FNC_RETURN:
-        if ((uptr->flags & UNIT_ATT) == 0) {            /* not attached? */
+        if ((uptr->flags & UNIT_UTS) == 0) {            /* not attached? */
             set_rper (ER1_UNS, drv);                    /* unsafe */
             break;
             }
@@ -811,7 +823,7 @@ switch (fnc) {                                          /* case on function */
         dc = 0;                                         /* seek to 0 */
     case FNC_SEEK:                                      /* seek */
     case FNC_SEARCH:                                    /* search */
-        if ((uptr->flags & UNIT_ATT) == 0) {            /* not attached? */
+        if ((uptr->flags & UNIT_UTS) == 0) {            /* not attached? */
             set_rper (ER1_UNS, drv);                    /* unsafe */
             break;
             }
@@ -834,7 +846,7 @@ switch (fnc) {                                          /* case on function */
     case FNC_WCHK:                                      /* write check */
     case FNC_READ:                                      /* read */
     case FNC_READH:                                     /* read headers */
-        if ((uptr->flags & UNIT_ATT) == 0) {            /* not attached? */
+        if ((uptr->flags & UNIT_UTS) == 0) {            /* not attached? */
             set_rper (ER1_UNS, drv);                    /* unsafe */
             break;
             }
@@ -877,6 +889,13 @@ static d10 dbuf[RP_MAXFR];
 
 dtype = GET_DTYPE (uptr->flags);                        /* get drive type */
 drv = (int32) (uptr - rp_dev.units);                    /* get drv number */
+if ((uptr->flags & UNIT_UTS) == 0) {                    /* Transition to up-to-speed */
+    uptr->flags |= UNIT_UTS;
+    rpds[drv] = DS_ATA | DS_MOL | DS_DPR | DS_RDY |
+               ((uptr->flags & UNIT_WPRT)? DS_WRL: 0);
+    update_rpcs (CS1_SC, drv);
+    return SCPE_OK;
+    }
 rpds[drv] = (rpds[drv] & ~DS_PIP) | DS_RDY;             /* change drive status */
 
 switch (uptr->FUNC) {                                   /* case on function */
@@ -893,6 +912,8 @@ switch (uptr->FUNC) {                                   /* case on function */
 
     case FNC_UNLOAD:                                    /* unload */
         rp_detach (uptr);                               /* detach unit */
+        rpds[drv] &= ~DS_ATA;                           /* Unload does not interrupt */
+        update_rpcs (0, drv);
         break;
 
     case FNC_RECAL:                                     /* recalibrate */
@@ -917,7 +938,7 @@ switch (uptr->FUNC) {                                   /* case on function */
             set_rper (ER1_WLE, drv);                    /* set drive error */
             update_rpcs (CS1_DONE | CS1_TRE, drv);      /* set done, err */
             break;
-            }
+            }                                           /* fall through */
     case FNC_WCHK:                                      /* write check */
     case FNC_READ:                                      /* read */
     case FNC_READH:                                     /* read headers */
@@ -951,7 +972,7 @@ switch (uptr->FUNC) {                                   /* case on function */
                 if ((rpcs2 & CS2_UAI) == 0)
                     ba = ba + 4;
                 }
-            if (fc10 = twc10 & (RP_NUMWD - 1)) {        /* fill? */
+            if ((fc10 = twc10 & (RP_NUMWD - 1))) {      /* fill? */
                 fc10 = RP_NUMWD - fc10;
                 for (i = 0; i < fc10; i++)
                     dbuf[twc10 + i] = 0;
@@ -998,10 +1019,10 @@ switch (uptr->FUNC) {                                   /* case on function */
         if (da >= drv_tab[dtype].size)
             rpds[drv] = rpds[drv] | DS_LST;
         da = da / RP_NUMWD;
-        rpda[drv] = da % drv_tab[dtype].sect;
+        rpda[drv] = (uint16)(da % drv_tab[dtype].sect);
         da = da / drv_tab[dtype].sect;
-        rpda[drv] = rpda[drv] | ((da % drv_tab[dtype].surf) << DA_V_SF);
-        rpdc[drv] = da / drv_tab[dtype].surf;
+        rpda[drv] = (uint16)(rpda[drv] | ((da % drv_tab[dtype].surf) << DA_V_SF));
+        rpdc[drv] = (uint16)(da / drv_tab[dtype].surf);
 
         if (err != 0) {                                 /* error? */
             set_rper (ER1_PAR, drv);                    /* set drive error */
@@ -1021,7 +1042,7 @@ return SCPE_OK;
 
 /* Set drive error */
 
-void set_rper (int32 flag, int32 drv)
+void set_rper (int16 flag, int32 drv)
 {
 rper1[drv] = rper1[drv] | flag;
 rpds[drv] = rpds[drv] | DS_ATA;
@@ -1048,7 +1069,7 @@ uptr = rp_dev.units + drv;                              /* get unit */
 if (rp_unit[drv].flags & UNIT_DIS)
     rpds[drv] = rper1[drv] = 0;
 else rpds[drv] = (rpds[drv] | DS_DPR) & ~DS_PGM;
-if (rp_unit[drv].flags & UNIT_ATT)
+if (rp_unit[drv].flags & UNIT_UTS)
     rpds[drv] = rpds[drv] | DS_MOL;
 else rpds[drv] = rpds[drv] & ~(DS_MOL | DS_VV | DS_RDY);
 if (rper1[drv] | rper2[drv] | rper3[drv])
@@ -1057,15 +1078,17 @@ else rpds[drv] = rpds[drv] & ~DS_ERR;
 
 rpcs1 = (rpcs1 & ~(CS1_SC | CS1_MCPE | CS1_MBZ | CS1_DRV)) | CS1_DVA | flag;
 rpcs1 = rpcs1 | (uptr->FUNC << CS1_V_FNC);
-if (sim_is_active (uptr))
+if (sim_is_active (uptr) && (uptr->flags & UNIT_UTS))
     rpcs1 = rpcs1 | CS1_GO;
 if (rpcs2 & CS2_ERR)
     rpcs1 = rpcs1 | CS1_TRE | CS1_SC;
 else if (rpcs1 & CS1_TRE)
     rpcs1 = rpcs1 | CS1_SC;
 for (i = 0; i < RP_NUMDR; i++) {
-    if (rpds[i] & DS_ATA)
+    if (rpds[i] & DS_ATA) {
         rpcs1 = rpcs1 | CS1_SC;
+        break;
+        }
     }
 if (rpiff || ((rpcs1 & CS1_SC) && (rpcs1 & CS1_DONE) && (rpcs1 & CS1_IE)))
     int_req = int_req | INT_RP;
@@ -1096,14 +1119,23 @@ rpiff = 0;                                              /* clear CSTB INTR */
 int_req = int_req & ~INT_RP;                            /* clear intr req */
 for (i = 0; i < RP_NUMDR; i++) {
     uptr = rp_dev.units + i;
-    sim_cancel (uptr);
     uptr->CYL = uptr->FUNC = 0;
     if (uptr->flags & UNIT_ATT)
+        if (uptr->flags & UNIT_UTS) {
+            sim_cancel (uptr);
         rpds[i] = (rpds[i] & DS_VV) | DS_DPR | DS_RDY | DS_MOL |
                 ((uptr->flags & UNIT_WPRT)? DS_WRL: 0);
-    else if (uptr->flags & UNIT_DIS)
+            } else {
+            if (!sim_is_active (uptr))
+                sim_activate (uptr, SPINUP_DLY);
+            rpds[i] = DS_DPR | ((uptr->flags & UNIT_WPRT)? DS_WRL: 0);
+            }
+    else {
+        sim_cancel (uptr);
+        if (uptr->flags & UNIT_DIS)
         rpds[i] = 0;
     else rpds[i] = DS_DPR;
+    }
     rper1[i] = 0;
     rper2[i] = 0;
     rper3[i] = 0;
@@ -1123,19 +1155,16 @@ return SCPE_OK;
 
 t_stat rp_attach (UNIT *uptr, char *cptr)
 {
-int32 drv, i, p;
+int32 i, p;
 t_stat r;
 
 uptr->capac = drv_tab[GET_DTYPE (uptr->flags)].size;
 r = attach_unit (uptr, cptr);
 if (r != SCPE_OK)
     return r;
-drv = (int32) (uptr - rp_dev.units);                    /* get drv number */
-rpds[drv] = DS_ATA | DS_MOL | DS_RDY | DS_DPR |
-    ((uptr->flags & UNIT_WPRT)? DS_WRL: 0);
-rper1[drv] = 0;
-update_rpcs (CS1_SC, drv);
-
+sim_cancel (uptr);
+uptr->flags &= ~UNIT_UTS;
+sim_activate (uptr, SPINUP_DLY);
 if ((uptr->flags & UNIT_AUTO) == 0)                     /* autosize? */
     return SCPE_OK;
 if ((p = sim_fsize (uptr->fileref)) == 0)
@@ -1147,6 +1176,7 @@ for (i = 0; drv_tab[i].sect != 0; i++) {
         return SCPE_OK;
         }
     }
+/* File is larger than max known disk.  This should probably fail. */
 return SCPE_OK;
 }
 
@@ -1163,11 +1193,14 @@ rpds[drv] = (rpds[drv] & ~(DS_MOL | DS_RDY | DS_WRL | DS_VV | DS_OF)) |
     DS_ATA;
 if (sim_is_active (uptr)) {                             /* unit active? */
     sim_cancel (uptr);                                  /* cancel operation */
+    if (uptr->flags & UNIT_UTS) {
     rper1[drv] = rper1[drv] | ER1_OPI;                  /* set drive error */
     if (uptr->FUNC >= FNC_WCHK)                         /* data transfer? */
         rpcs1 = rpcs1 | CS1_DONE | CS1_TRE;             /* set done, err */
     }
-update_rpcs (CS1_SC, drv);                              /* request intr */
+    }
+uptr->flags &= ~UNIT_UTS;
+update_rpcs (0, drv);                                  /* request intr */
 return detach_unit (uptr);
 }
 
@@ -1183,111 +1216,164 @@ uptr->capac = drv_tab[dtype].size;
 return SCPE_OK;
 }
 
-/* Device bootstrap */
+/* Device bootstrap
+ * The DEC and ITS versions are word-for-word identical, except that
+ * the DEC RDIO/WRIO are replaced by IORDQ and IOWRQ.  This is hand
+ * assembled code, so please always make changes in both.
+ * Due to a typo in the KS Console rom, block 010 is read for the
+ * alternate HOM block.  The correct block is 012.  For compatibiliy,
+ * we will do what the hardware did first, what's right if it fails (as it will).
+ */
 
 #define BOOT_START      0377000                         /* start */
 #define BOOT_LEN (sizeof (boot_rom_dec) / sizeof (d10))
 
 static const d10 boot_rom_dec[] = {
-    0515040000001,                          /* boot:hrlzi 1,1       ; uba # */
-    0201000140001,                          /*      movei 0,140001  ; vld,fst,pg 1 */
-    0713001000000+(IOBA_UBMAP+1 & RMASK),   /*      wrio 0,763001(1); set ubmap */
-    0435040000000+(IOBA_RP & RMASK),        /*      iori 1,776700   ; rh addr */
-    0202040000000+FE_RHBASE,                /*      movem 1,FE_RHBASE */
-    0201000000040,                          /*      movei 0,40      ; ctrl reset */
-    0713001000010,                          /*      wrio 0,10(1)    ; ->RPCS2 */
-    0201000000021,                          /*      movei 0,21      ; preset */
-    0713001000000,                          /*      wrio 0,0(1)     ; ->RPCS1 */
-    0201100000001,                          /*      movei 2,1       ; blk #1 */
-    0265740377032,                          /*      jsp 17,rdbl     ; read */
-    0204140001000,                          /*      movs 3,1000     ; id word */
-    0306140505755,                          /*      cain 3,sixbit /HOM/ */
-    0254000377023,                          /*      jrst .+6        ; match */
-    0201100000010,                          /*      movei 2,10      ; blk #10 */
-    0265740377032,                          /*      jsp 17,rdbl     ; read */
-    0204140001000,                          /*      movs 3,1000     ; id word */
-    0302140505755,                          /*      caie 3,sixbit /HOM/ */
-    0254200377022,                          /*      halt .          ; inv home */
-    0336100001103,                          /*      skipn 2,1103    ; pg of ptrs */
-    0254200377024,                          /*      halt .          ; inv ptr */
-    0265740377032,                          /*      jsp 17,rdbl     ; read */
-    0336100001004,                          /*      skipn 2,1004    ; mon boot */
-    0254200377027,                          /*      halt .          ; inv ptr */
-    0265740377032,                          /*      jsp 17,rdbl     ; read */
-    0254000001000,                          /*      jrst 1000       ; start */
-    0201140176000,                          /* rdbl:movei 3,176000  ; wd cnt */
-    0201200004000,                          /*      movei 4,4000    ; addr */
-    0200240000000+FE_UNIT,                  /*      move 5,FE_UNIT  ; unit */
-    0200300000002,                          /*      move 6,2 */
-    0242300777750,                          /*      lsh 6,-24.      ; cyl */
-    0713141000002,                          /*      wrio 3,2(1)     ; ->RPWC */
-    0713201000004,                          /*      wrio 4,4(1)     ; ->RPBA */
-    0713101000006,                          /*      wrio 2,6(1)     ; ->RPDA */
-    0713241000010,                          /*      wrio 5,10(1)    ; ->RPCS2 */
-    0713301000034,                          /*      wrio 6,34(1)    ; ->RPDC */
-    0201000000071,                          /*      movei 0,71      ; read+go */
-    0713001000000,                          /*      wrio 0,0(1)     ; ->RPCS1 */
-    0712341000000,                          /*      rdio 7,0(1)     ; read csr */
-    0606340000200,                          /*      trnn 7,200      ; test rdy */
-    0254000377046,                          /*      jrst .-2        ; loop */
-    0602340100000,                          /*      trne 7,100000   ; test err */
-    0254200377052,                          /*      halt */
-    0254017000000,                          /*      jrst 0(17)      ; return */
+    INT64_C(0510040000000)+FE_RHBASE,       /* boot:hllz 1,FE_RHBASE   ; uba # */
+    INT64_C(0201000140001),                 /*      movei 0,140001  ; vld,fst,pg 1 */
+    INT64_C(0713001000000)+((IOBA_UBMAP+1) & RMASK),   /*      wrio 0,763001(1); set ubmap */
+    INT64_C(0200040000000)+FE_RHBASE,       /*      move 1,FE_RHBASE */
+    INT64_C(0201000000040),                 /*      movei 0,40      ; ctrl reset */
+    INT64_C(0713001000010),                 /*      wrio 0,10(1)    ; ->RPCS2 */
+    INT64_C(0200240000000)+FE_UNIT,         /*      move 5,FE_UNIT  ; unit */
+    INT64_C(0713241000010),                 /*      wrio 5,10(1)    ; select ->RPCS2 */
+
+    INT64_C(0712001000012),                 /*10    rdio 0,12(1)    ; RPDS */
+    INT64_C(0640000010600),                 /*      trc  0,10600    ; MOL + DPR + RDY */
+    INT64_C(0642000010600),                 /*      trce 0,10600    ; */
+    INT64_C(0254000377010),                 /*      jrst .-3        ; wait */
+    INT64_C(0201000000377),                 /*      movei 0,377     ; All units */
+    INT64_C(0713001000016),                 /*      wrio 0,16(1)    ; Clear on-line attns */
+    INT64_C(0201000000021),                 /*      movei 0,21      ; preset */
+    INT64_C(0713001000000),                 /*      wrio 0,0(1)     ; ->RPCS1 */
+
+    INT64_C(0201100000001),                 /*20    movei 2,1       ; blk #1 */
+    INT64_C(0265740377041),                 /*      jsp 17,rdbl     ; read */
+    INT64_C(0204140001000),                 /*      movs 3,1000     ; id word */
+    INT64_C(0306140505755),                 /*      cain 3,sixbit /HOM/ */
+    INT64_C(0254000377032),                 /*      jrst pg         ; match */
+    INT64_C(0201100000010),                 /*      movei 2,10      ; blk #10 */
+    INT64_C(0265740377041),                 /*      jsp 17,rdbl     ; read */
+    INT64_C(0204140001000),                 /*      movs 3,1000     ; id word */
+
+    INT64_C(0302140505755),                 /*30    caie 3,sixbit /HOM/ */
+    INT64_C(0254000377061),                 /*      jrst alt2        ; inv home */
+    INT64_C(0336100001103),                 /* pg:  skipn 2,1103    ; pg of ptrs */
+    INT64_C(0254200377033),                 /*      halt .          ; inv ptr */
+    INT64_C(0265740377041),                 /*      jsp 17,rdbl     ; read */
+    INT64_C(0336100001004),                 /*      skipn 2,1004    ; mon boot */
+    INT64_C(0254200377036),                 /*      halt .          ; inv ptr */
+    INT64_C(0265740377041),                 /*      jsp 17,rdbl     ; read */
+
+    INT64_C(0254000001000),                 /*40    jrst 1000       ; start */
+    INT64_C(0201140176000),                 /* rdbl:movei 3,176000  ; wd cnt 1P = -512*2 */
+    INT64_C(0201200004000),                 /*      movei 4,4000    ; 11 addr => M[1000] */
+    INT64_C(0200300000002),                 /*      move 6,2 */
+    INT64_C(0242300777750),                 /*      lsh 6,-24.      ; cyl */
+    INT64_C(0713141000002),                 /*      wrio 3,2(1)     ; ->RPWC */
+    INT64_C(0713201000004),                 /*      wrio 4,4(1)     ; ->RPBA */
+    INT64_C(0713101000006),                 /*      wrio 2,6(1)     ; ->RPDA */
+
+    INT64_C(0713301000034),                 /*50    wrio 6,34(1)    ; ->RPDC */
+    INT64_C(0201000000071),                 /*      movei 0,71      ; read+go */
+    INT64_C(0713001000000),                 /*      wrio 0,0(1)     ; ->RPCS1 */
+    INT64_C(0712341000000),                 /*      rdio 7,0(1)     ; read csr */
+    INT64_C(0606340000200),                 /*      trnn 7,200      ; test rdy */
+    INT64_C(0254000377053),                 /*      jrst .-2        ; loop */
+    INT64_C(0602340100000),                 /*      trne 7,100000   ; test err */
+    INT64_C(0254200377057),                 /*      halt . */
+
+    INT64_C(0254017000000),                 /*60    jrst 0(17)      ; return */
+    INT64_C(0201100000012),                 /*alt2: movei 2,10.     ; blk #10. */
+    INT64_C(0265740377041),                 /*      jsp 17,rdbl     ; read */
+    INT64_C(0204140001000),                 /*      movs 3,1000     ; id word */
+    INT64_C(0302140505755),                 /*      caie 3,sixbit /HOM/ */
+    INT64_C(0254200377065),                 /*      halt .          ; inv home */
+    INT64_C(0254000377032),                 /*      jrst pg         ; Read ptrs */
     };
 
 static const d10 boot_rom_its[] = {
-    0515040000001,                          /* boot:hrlzi 1,1       ; uba # */
-    0201000140001,                          /*      movei 0,140001  ; vld,fst,pg 1 */
-    0715000000000+(IOBA_UBMAP+1 & RMASK),   /*      iowrq 0,763001  ; set ubmap */
-    0435040000000+(IOBA_RP & RMASK),        /*      iori 1,776700   ; rh addr */
-    0202040000000+FE_RHBASE,                /*      movem 1,FE_RHBASE */
-    0201000000040,                          /*      movei 0,40      ; ctrl reset */
-    0715001000010,                          /*      iowrq 0,10(1)   ; ->RPCS2 */
-    0201000000021,                          /*      movei 0,21      ; preset */
-    0715001000000,                          /*      iowrq 0,0(1)    ; ->RPCS1 */
-    0201100000001,                          /*      movei 2,1       ; blk #1 */
-    0265740377032,                          /*      jsp 17,rdbl     ; read */
-    0204140001000,                          /*      movs 3,1000     ; id word */
-    0306140505755,                          /*      cain 3,sixbit /HOM/ */
-    0254000377023,                          /*      jrst .+6        ; match */
-    0201100000010,                          /*      movei 2,10      ; blk #10 */
-    0265740377032,                          /*      jsp 17,rdbl     ; read */
-    0204140001000,                          /*      movs 3,1000     ; id word */
-    0302140505755,                          /*      caie 3,sixbit /HOM/ */
-    0254200377022,                          /*      halt .          ; inv home */
-    0336100001103,                          /*      skipn 2,1103    ; pg of ptrs */
-    0254200377024,                          /*      halt .          ; inv ptr */
-    0265740377032,                          /*      jsp 17,rdbl     ; read */
-    0336100001004,                          /*      skipn 2,1004    ; mon boot */
-    0254200377027,                          /*      halt .          ; inv ptr */
-    0265740377032,                          /*      jsp 17,rdbl     ; read */
-    0254000001000,                          /*      jrst 1000       ; start */
-    0201140176000,                          /* rdbl:movei 3,176000  ; wd cnt */
-    0201200004000,                          /*      movei 4,4000    ; addr */
-    0200240000000+FE_UNIT,                  /*      move 5,FE_UNIT  ; unit */
-    0200300000002,                          /*      move 6,2 */
-    0242300777750,                          /*      lsh 6,-24.      ; cyl */
-    0715141000002,                          /*      iowrq 3,2(1)    ; ->RPWC */
-    0715201000004,                          /*      iowrq 4,4(1)    ; ->RPBA */
-    0715101000006,                          /*      iowrq 2,6(1)    ; ->RPDA */
-    0715241000010,                          /*      iowrq 5,10(1)   ; ->RPCS2 */
-    0715301000034,                          /*      iowrq 6,34(1)   ; ->RPDC */
-    0201000000071,                          /*      movei 0,71      ; read+go */
-    0715001000000,                          /*      iowrq 0,0(1)    ; ->RPCS1 */
-    0711341000000,                          /*      iordq 7,0(1)    ; read csr */
-    0606340000200,                          /*      trnn 7,200      ; test rdy */
-    0254000377046,                          /*      jrst .-2        ; loop */
-    0602340100000,                          /*      trne 7,100000   ; test err */
-    0254200377052,                          /*      halt */
-    0254017000000,                          /*      jrst 0(17)      ; return */
+    INT64_C(0510040000001)+FE_RHBASE,       /* boot:hllzi 1,FE_RHBASE ; uba # */
+    INT64_C(0201000140001),                 /*      movei 0,140001  ; vld,fst,pg 1 */
+    INT64_C(0715000000000)+((IOBA_UBMAP+1) & RMASK),   /*      iowrq 0,763001  ; set ubmap */
+    INT64_C(0200040000000)+FE_RHBASE,       /*      move 1,FE_RHBASE */
+    INT64_C(0201000000040),                 /*      movei 0,40      ; ctrl reset */
+    INT64_C(0715001000010),                 /*      iowrq 0,10(1)   ; ->RPCS2 */
+    INT64_C(0200240000000)+FE_UNIT,         /*      move 5,FE_UNIT  ; unit */
+    INT64_C(0715241000010),                 /*      iowrq 5,10(1)   ; ->RPCS2 */
+
+    INT64_C(0711001000012),                 /*10    iordq 0,12(1)   ; RPDS */
+    INT64_C(0640000010600),                 /*      trc  0,10600    ; MOL + DPR + RDY */
+    INT64_C(0642000010600),                 /*      trce 0,10600    ; */
+    INT64_C(0254000377010),                 /*      jrst .-3        ; wait */
+    INT64_C(0201000000377),                 /*      movei 0,377     ; All units */
+    INT64_C(0715001000016),                 /*      iowrq 0,16(1)   ; Clear on-line attns */
+    INT64_C(0201000000021),                 /*      movei 0,21      ; preset */
+    INT64_C(0715001000000),                 /*      iowrq 0,0(1)    ; ->RPCS1 */
+
+    INT64_C(0201100000001),                 /*20    movei 2,1       ; blk #1 */
+    INT64_C(0265740377041),                 /*      jsp 17,rdbl     ; read */
+    INT64_C(0204140001000),                 /*      movs 3,1000     ; id word */
+    INT64_C(0306140505755),                 /*      cain 3,sixbit /HOM/ */
+    INT64_C(0254000377032),                 /*      jrst pg         ; match */
+    INT64_C(0201100000010),                 /*      movei 2,10      ; blk #10 */
+    INT64_C(0265740377041),                 /*      jsp 17,rdbl     ; read */
+    INT64_C(0204140001000),                 /*      movs 3,1000     ; id word */
+
+    INT64_C(0302140505755),                 /*30    caie 3,sixbit /HOM/ */
+    INT64_C(0254000377061),                 /*      jrst alt2       ; inv home */
+    INT64_C(0336100001103),                 /* pg:  skipn 2,1103    ; pg of ptrs */
+    INT64_C(0254200377033),                 /*      halt .          ; inv ptr */
+    INT64_C(0265740377041),                 /*      jsp 17,rdbl     ; read */
+    INT64_C(0336100001004),                 /*      skipn 2,1004    ; mon boot */
+    INT64_C(0254200377036),                 /*      halt .          ; inv ptr */
+    INT64_C(0265740377041),                 /*      jsp 17,rdbl     ; read */
+
+    INT64_C(0254000001000),                 /*40    jrst 1000       ; start */
+    INT64_C(0201140176000),                 /* rdbl:movei 3,176000  ; wd cnt 1P = -512 *2 */
+    INT64_C(0201200004000),                 /*      movei 4,4000    ; addr */
+    INT64_C(0200300000002),                 /*      move 6,2 */
+    INT64_C(0242300777750),                 /*      lsh 6,-24.      ; cyl */
+    INT64_C(0715141000002),                 /*      iowrq 3,2(1)    ; ->RPWC */
+    INT64_C(0715201000004),                 /*      iowrq 4,4(1)    ; ->RPBA */
+    INT64_C(0715101000006),                 /*      iowrq 2,6(1)    ; ->RPDA */
+
+    INT64_C(0715301000034),                 /*50    iowrq 6,34(1)   ; ->RPDC */
+    INT64_C(0201000000071),                 /*      movei 0,71      ; read+go */
+    INT64_C(0715001000000),                 /*      iowrq 0,0(1)    ; ->RPCS1 */
+    INT64_C(0711341000000),                 /*      iordq 7,0(1)    ; read csr */
+    INT64_C(0606340000200),                 /*      trnn 7,200      ; test rdy */
+    INT64_C(0254000377053),                 /*      jrst .-2        ; loop */
+    INT64_C(0602340100000),                 /*      trne 7,100000   ; test err */
+    INT64_C(0254200377057),                 /*      halt */
+
+    INT64_C(0254017000000),                 /*60    jrst 0(17)      ; return */
+    INT64_C(0201100000012),                 /* alt2:movei 2,10.     ; blk #10. */
+    INT64_C(0265740377041),                 /*      jsp 17,rdbl     ; read */
+    INT64_C(0204140001000),                 /*      movs 3,1000     ; id word */
+    INT64_C(0302140505755),                 /*      caie 3,sixbit /HOM/ */
+    INT64_C(0254200377065),                 /*      halt .          ; inv home */
+    INT64_C(0254000377032),                 /*      jrst pg         ; Read ptrs */
     };
 
 t_stat rp_boot (int32 unitno, DEVICE *dptr)
 {
-int32 i;
+size_t i;
 extern a10 saved_PC;
+UNIT *uptr;
+
+unitno &= CS2_M_UNIT;
+uptr = rp_dev.units + unitno;
+if (!(uptr->flags & UNIT_ATT))
+    return SCPE_NOATT;
+
+M[FE_RHBASE] = fe_bootrh = rp_dib.ba;
+M[FE_UNIT] = fe_bootunit = unitno;
+
+assert (sizeof(boot_rom_dec) == sizeof(boot_rom_its));
+
+M[FE_KEEPA] = (M[FE_KEEPA] & ~INT64_C(0xFF)) | ((sim_switches & SWMASK ('A'))? 010 : 0);
 
-M[FE_UNIT] = unitno & CS2_M_UNIT;
 for (i = 0; i < BOOT_LEN; i++)
     M[BOOT_START + i] = Q_ITS? boot_rom_its[i]: boot_rom_dec[i];
 saved_PC = BOOT_START;
