@@ -1,6 +1,6 @@
 /* pdp10_tu.c - PDP-10 RH11/TM03/TU45 magnetic tape simulator
 
-   Copyright (c) 1993-2008, Robert M Supnik
+   Copyright (c) 1993-2018, Robert M Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -25,7 +25,11 @@
 
    tu           RH11/TM03/TU45 magtape
 
-   29-Apr-07    RMS     Fixed bug in setting FCE on TMK (found by Naoki Hamada)
+   12-Jan-18    RMS     Fixed missing () in logical test (Mark Pizzolato)
+   29-Dec-17    RMS     Read tape mark must set Massbus EXC (TRE)
+   28-Mar-17    RMS     Documented switch fall through case (COVERITY)
+   17-Mar-13    RMS     Fixed bug in read/write check reverse (Dave Bryan)
+   29-Apr-07    RMS     Fixed bug in setting FCE on TMK (Naoki Hamada)
    16-Feb-06    RMS     Added tape capacity checking
    16-Aug-05    RMS     Fixed C++ declaration and cast problems
    07-Jul-05    RMS     Removed extraneous externs
@@ -42,8 +46,8 @@
    28-Mar-03    RMS     Added multiformat support
    28-Feb-03    RMS     Revised for magtape library
    27-Jan-03    RMS     Changed to dynamically allocate buffer
-   21-Nov-02    RMS     Fixed bug in bootstrap (reported by Michael Thompson)
-                        Fixed bug in read (reported by Harris Newman)
+   21-Nov-02    RMS     Fixed bug in bootstrap (Michael Thompson)
+                        Fixed bug in read (Harris Newman)
    29-Sep-02    RMS     Added variable vector support
                         New data structures
    28-Aug-02    RMS     Added end of medium support
@@ -92,6 +96,7 @@
 
 #include "pdp10_defs.h"
 #include "sim_tape.h"
+#include <assert.h>
 
 #define TU_NUMFM        1                               /* #formatters */
 #define TU_NUMDR        8                               /* #drives */
@@ -99,6 +104,9 @@
 #define UDENS           u4                              /* unit density */
 #define  UD_UNK         0                               /* unknown */
 #define MT_MAXFR        (1 << 16)                       /* max data buf */
+#define TU_STATEFLAGS   u5                              /* Simulator state flags */
+#define TUS_ATTPENDING  0000001                         /* Attach pending */
+#define SPINUPDLY       100*1000                        /* 100 msec */
 
 /* MTCS1 - 172440 - control/status 1 */
 
@@ -297,8 +305,6 @@ extern int32 int_req;
 extern int32 ubmap[UBANUM][UMAP_MEMSIZE];               /* Unibus map */
 extern int32 ubcs[UBANUM];
 extern UNIT cpu_unit;
-extern int32 sim_switches;
-extern FILE *sim_deb;
 
 int32 tucs1 = 0;                                        /* control/status 1 */
 int32 tuwc = 0;                                         /* word count */
@@ -415,7 +421,7 @@ DEVICE tu_dev = {
     TU_NUMDR, 10, 31, 1, 8, 8,
     NULL, NULL, &tu_reset,
     &tu_boot, &tu_attach, &tu_detach,
-    &tu_dib, DEV_UBUS | DEV_DEBUG
+    &tu_dib, DEV_UBUS | DEV_DEBUG | DEV_TAPE
     };
 
 /* I/O dispatch routine, I/O addresses 17772440 - 17772472 */
@@ -667,13 +673,18 @@ switch (fnc) {                                          /* case on function */
         tuer = 0;                                       /* clear errors */
         tutc = tutc & ~TC_FCS;                          /* clear fc status */
         tufs = tufs & ~(FS_SAT | FS_SSC | FS_ID | FS_ERR);
-        sim_cancel (uptr);                              /* reset drive */
-        uptr->USTAT = 0;
+        if (!(uptr->TU_STATEFLAGS & TUS_ATTPENDING))
+            sim_cancel (uptr);                          /* stop motion, not on-line delay */
+        uptr->USTAT = 0;                                /* fall through */
     case FNC_NOP:
         tucs1 = tucs1 & ~CS1_GO;                        /* no operation */
         return;
 
     case FNC_RIP:                                       /* read-in preset */
+        if ((tufs & FS_MOL) == 0) {                     /* unattached? */
+            set_tuer (ER_UNS);
+            break;
+            }
         tutc = TC_RIP;                                  /* density = 800 */
         sim_tape_rewind (&tu_unit[0]);                  /* rewind unit 0 */
         tu_unit[0].USTAT = 0;
@@ -682,7 +693,7 @@ switch (fnc) {                                          /* case on function */
         return;
 
     case FNC_UNLOAD:                                    /* unload */
-        if ((uptr->flags & UNIT_ATT) == 0) {            /* unattached? */
+        if ((tufs & FS_MOL) == 0) {                     /* unattached? */
             set_tuer (ER_UNS);
             break;
             }
@@ -694,7 +705,7 @@ switch (fnc) {                                          /* case on function */
         return; 
 
     case FNC_REWIND:
-        if ((uptr->flags & UNIT_ATT) == 0) {            /* unattached? */
+        if ((tufs & FS_MOL) == 0) {                     /* unattached? */
             set_tuer (ER_UNS);
             break;
             }
@@ -705,7 +716,7 @@ switch (fnc) {                                          /* case on function */
         return;
 
     case FNC_SPACEF:
-        if ((uptr->flags & UNIT_ATT) == 0) {            /* unattached? */
+        if ((tufs & FS_MOL) == 0) {                     /* unattached? */
             set_tuer (ER_UNS);
             break;
             }
@@ -717,7 +728,7 @@ switch (fnc) {                                          /* case on function */
         goto GO_XFER;
 
     case FNC_SPACER:
-        if ((uptr->flags & UNIT_ATT) == 0) {            /* unattached? */
+        if ((tufs & FS_MOL) == 0) {                     /* unattached? */
             set_tuer (ER_UNS);
             break;
             }
@@ -730,7 +741,7 @@ switch (fnc) {                                          /* case on function */
 
     case FNC_WREOF:                                     /* write tape mark */
     case FNC_ERASE:                                     /* erase */
-        if ((uptr->flags & UNIT_ATT) == 0) {            /* unattached? */
+        if ((tufs & FS_MOL) == 0) {                     /* unattached? */
             set_tuer (ER_UNS);
             break;
             }
@@ -764,7 +775,7 @@ switch (fnc) {                                          /* case on function */
     case FNC_WCHKF:                                     /* wchk = read */
     case FNC_READF:                                     /* read */
     DATA_XFER:
-        if ((uptr->flags & UNIT_ATT) == 0) {            /* unattached? */
+        if ((tufs & FS_MOL) == 0) {                     /* unattached? */
             set_tuer (ER_UNS);
             break;
             }
@@ -803,12 +814,24 @@ return;
 t_stat tu_svc (UNIT *uptr)
 {
 int32 fnc, fmt, i, j, k, wc10, ba10;
-int32 ba, fc, wc, drv, mpa10, vpn;
+int32 ba, fc, wc, drv, mpa10 = 0, vpn;
 d10 val, v[4];
 t_mtrlnt tbc;
 t_stat st, r = SCPE_OK;
 
 drv = (int32) (uptr - tu_dev.units);                    /* get drive # */
+
+/* Set MOL for a delayed attach */
+
+if (uptr->TU_STATEFLAGS & TUS_ATTPENDING) {
+    uptr->TU_STATEFLAGS &= ~TUS_ATTPENDING;             /* Allow transition to on-line */
+    tufs = tufs | FS_ATA | FS_SSC;                      /* set attention */
+    if ((GET_FMTR (tucs2) == 0) && (GET_DRV (tutc) == drv)) /* selected drive? */
+        tufs = tufs | FS_SAT;                           /* set slave attn */
+    update_tucs (CS1_SC, drv);                          /* update status */
+    return SCPE_OK;
+}
+
 if (uptr->USTAT & FS_REW) {                             /* rewind or unload? */
     sim_tape_rewind (uptr);                             /* rewind tape */
     uptr->USTAT = 0;                                    /* clear status */
@@ -833,12 +856,12 @@ switch (fnc) {                                          /* case on function */
     case FNC_SPACEF:                                    /* space forward */
         do {
             tufc = (tufc + 1) & 0177777;                /* incr fc */
-            if (st = sim_tape_sprecf (uptr, &tbc)) {    /* space rec fwd, err? */
+            if ((st = sim_tape_sprecf (uptr, &tbc))) {  /* space rec fwd, err? */
                 r = tu_map_err (uptr, st, 0);           /* map error */
                 break;
                 }
             } while ((tufc != 0) && !sim_tape_eot (uptr));
-        if (tufc)
+        if (tufc != 0)
             set_tuer (ER_FCE);
         else tutc = tutc & ~TC_FCS;
         tufs = tufs | FS_ATA;
@@ -847,19 +870,19 @@ switch (fnc) {                                          /* case on function */
     case FNC_SPACER:                                    /* space reverse */
         do {
             tufc = (tufc + 1) & 0177777;                /* incr wc */
-            if (st = sim_tape_sprecr (uptr, &tbc)) {    /* space rec rev, err? */
+            if ((st = sim_tape_sprecr (uptr, &tbc))) {  /* space rec rev, err? */
                 r = tu_map_err (uptr, st, 0);           /* map error */
                 break;
                 }
             } while (tufc != 0);
-        if (tufc)
+        if (tufc != 0)
             set_tuer (ER_FCE);
         else tutc = tutc & ~TC_FCS;
         tufs = tufs | FS_ATA;
         break;
 
     case FNC_WREOF:                                     /* write end of file */
-        if (st = sim_tape_wrtmk (uptr))                 /* write tmk, err? */
+        if ((st = sim_tape_wrtmk (uptr)))               /* write tmk, err? */
             r = tu_map_err (uptr, st, 0);               /* map error */
         tufs = tufs | FS_ATA;
         break;
@@ -889,9 +912,7 @@ switch (fnc) {                                          /* case on function */
         if ((uptr->UDENS == TC_1600) && sim_tape_bot (uptr))
             tufs = tufs | FS_ID;                        /* PE BOT? ID burst */
         TXFR (ba, wc, 0);                               /* validate transfer */
-        if (st = sim_tape_rdrecf (uptr, xbuf, &tbc, MT_MAXFR)) { /* read fwd */
-            if (st == MTSE_TMK)                         /* TMK also sets FCE */
-                set_tuer (ER_FCE);
+        if ((st = sim_tape_rdrecf (uptr, xbuf, &tbc, MT_MAXFR))) {/* read fwd */
             r = tu_map_err (uptr, st, 1);               /* map error */
             break;                                      /* done */
             }
@@ -936,7 +957,7 @@ switch (fnc) {                                          /* case on function */
             }                                           /* end for */
         if (j < fc)                                     /* short record? */
             fc = j;
-        if (st = sim_tape_wrrecf (uptr, xbuf, fc))      /* write rec, err? */
+        if ((st = sim_tape_wrrecf (uptr, xbuf, fc)))    /* write rec, err? */
             r = tu_map_err (uptr, st, 1);               /* map error */
         else {
             tufc = (tufc + fc) & 0177777;
@@ -951,9 +972,7 @@ switch (fnc) {                                          /* case on function */
     case FNC_WCHKR:                                     /* wcheck = read */
         tufc = 0;                                       /* clear frame count */
         TXFR (ba, wc, 1);                               /* validate xfer rev */
-        if (st = sim_tape_rdrecr (uptr, xbuf + 4, &tbc, MT_MAXFR)) { /* read rev */
-            if (st == MTSE_TMK)                         /* TMK also sets FCE */
-                set_tuer (ER_FCE);
+        if ((st = sim_tape_rdrecr (uptr, xbuf + 4, &tbc, MT_MAXFR))) {/* read rev */
             r = tu_map_err (uptr, st, 1);               /* map error */
             break;                                      /* done */
             }
@@ -964,7 +983,7 @@ switch (fnc) {                                          /* case on function */
                 MAPM (ba10 - i, mpa10, UMAP_RRV);
                 }
             val = ((fmt == TC_10C)? (((d10) xbuf [--j]) & 017): 0);
-            for (k = 0; k < 4; i++)
+            for (k = 0; k < 4; k++)
                 v[k] = xbuf[--j];
             val = val | (v[0] << 4) | (v[1] << 12) | (v[2] << 20) | (v[3] << 28);
             if (fnc == FNC_READR)                       /* read? store */
@@ -1015,12 +1034,15 @@ return;
 
 void update_tucs (int32 flag, int32 drv)
 {
-int32 act = sim_is_active (&tu_unit[drv]);
+int32 act = sim_activate_time (&tu_unit[drv]);
 
 if ((flag & ~tucs1) & CS1_DONE)                         /* DONE 0 to 1? */
     tuiff = (tucs1 & CS1_IE)? 1: 0;                     /* CSTB INTR <- IE */
 if (GET_FMTR (tucs2) == 0) {                            /* formatter present? */
     tufs = (tufs & ~FS_DYN) | FS_FPR;
+    if (tu_unit[drv].TU_STATEFLAGS & TUS_ATTPENDING)    /* Delayed on-line timer running? */
+        act = 0;                                        /* Not a tape motion op */
+    else {
     if (tu_unit[drv].flags & UNIT_ATT) {
         tufs = tufs | FS_MOL | tu_unit[drv].USTAT;
         if (tu_unit[drv].UDENS == TC_1600)
@@ -1034,6 +1056,7 @@ if (GET_FMTR (tucs2) == 0) {                            /* formatter present? */
                 tufs = tufs | FS_EOT;
             }
         }
+        }
     if (tuer)
         tufs = tufs | FS_ERR;
     }
@@ -1062,7 +1085,9 @@ tuiff = 0;                                              /* clear CSTB INTR */
 return VEC_TU;                                          /* acknowledge */
 }
 
-/* Map tape error status */
+/* Map tape error status
+
+   Note that tape mark on a data transfer sets FCE and Massbus EXC */
 
 t_stat tu_map_err (UNIT *uptr, t_stat st, t_bool qdt)
 {
@@ -1078,6 +1103,10 @@ switch (st) {
 
     case MTSE_TMK:                                      /* end of file */
         tufs = tufs | FS_TMK;
+        if (qdt) {                                      /* data transfer? */
+            set_tuer (ER_FCE);                          /* set FCE */
+            tucs1 = tucs1 | CS1_TRE;
+            }
         break;
 
     case MTSE_IOERR:                                    /* IO error */
@@ -1141,7 +1170,11 @@ int_req = int_req & ~INT_TU;                            /* clear interrupt */
 for (u = 0; u < TU_NUMDR; u++) {                        /* loop thru units */
     uptr = tu_dev.units + u;
     sim_tape_reset (uptr);                              /* clear pos flag */
-    sim_cancel (uptr);                                  /* cancel activity */
+    if (!(uptr->TU_STATEFLAGS & TUS_ATTPENDING))        /* delayed on-line must survive massbus clear */
+        sim_cancel (uptr);                              /* cancel activity */
+    else if (!sim_is_active(uptr) )
+        sim_activate (uptr, SPINUPDLY);
+
     uptr->USTAT = 0;
     }
 if (xbuf == NULL)
@@ -1163,10 +1196,20 @@ if (r != SCPE_OK)
     return r;
 uptr->USTAT = 0;                                        /* clear unit status */
 uptr->UDENS = UD_UNK;                                   /* unknown density */
+/* Delay setting MOL since we may have just detached a previous file.
+ * In that case, the OS must see MOL clear, so that it will know that the
+ * drive was off-line.  This ensures that the OS will detect a tape change.
+ * 100 msec should suffice - though a real operator would take longer!
+ * Here, we ensure that the off-line transition from detach causes an attention
+ * interrupt.  The on-line transition will happen later.
+ */
 tufs = tufs | FS_ATA | FS_SSC;                          /* set attention */
 if ((GET_FMTR (tucs2) == 0) && (GET_DRV (tutc) == drv)) /* selected drive? */
     tufs = tufs | FS_SAT;                               /* set slave attn */
+uptr->TU_STATEFLAGS |= TUS_ATTPENDING;
 update_tucs (CS1_SC, drv);                              /* update status */
+sim_cancel(uptr);
+sim_activate (uptr,SPINUPDLY);
 return r;
 }
 
@@ -1181,104 +1224,173 @@ if (!(uptr->flags & UNIT_ATT))                          /* attached? */
 if (sim_is_active (uptr)) {                             /* unit active? */
     sim_cancel (uptr);                                  /* cancel operation */
     tuer = tuer | ER_UNS;                               /* set formatter error */
-    if ((uptr->USTAT & FS_REW) == 0)                    /* data transfer? */
+    if (uptr->TU_STATEFLAGS & TUS_ATTPENDING)
+        uptr->TU_STATEFLAGS &= ~TUS_ATTPENDING;
+    else if ((uptr->USTAT & FS_REW) == 0)               /* data transfer? */
         tucs1 = tucs1 | CS1_DONE | CS1_TRE;             /* set done, err */
     }
 uptr->USTAT = 0;                                        /* clear status flags */
 tufs = tufs | FS_ATA | FS_SSC;                          /* set attention */
+if ((GET_FMTR (tucs2) == 0) && (GET_DRV (tutc) == drv)) /* selected drive? */
+    tufs = tufs | FS_SAT;                               /* set slave attn */
+uptr->flags &= ~UNIT_ATT;                               /* Ensure MOL is cleared */
 update_tucs (CS1_SC, drv);                              /* update status */
+uptr->flags |= UNIT_ATT;
 return sim_tape_detach (uptr);
 }
 
 /* Device bootstrap */
+/* Note that the dec and ITS boot code is word for word identical,
+ * except for the IO instructions.  The ITS instructions encode the
+ * UBA number.  No attempt is made to allow UBA selection under ITS,
+ * though it should work with the DEC rom.
+ * The sequence is:
+ * controller clear - to clear controller errors
+ * formatter select - to gain access to the formatter registers. (since only
+ * one formatter is supported, and it's assumed to be zero, this isn't strictly
+ * necessary.  But maybe someday...)
+ * wait for MOL to appear.
+ * Drive clear - to clear any errors in the transport, including attention from on-line.
+ * Space forward one file - this is the KS CPU microcode, which the simulator doesn't
+ * use.
+ * Read the preboot (next level bootstrap) from the tape into page 1.
+ * Each operation produces erors - for one, the frame count is not exact.
+ * They are cleared, and the expected ones ignored.  If no unexpected
+ * errors are encountered, control is transferred to the preboot.
+ */
 
 #define BOOT_START      0377000                         /* start */
 #define BOOT_LEN (sizeof (boot_rom_dec) / sizeof (d10))
 
 static const d10 boot_rom_dec[] = {
-    0515040000003,                          /* boot:hrlzi 1,3       ; uba # */
-    0201000040001,                          /*      movei 0,40001   ; vld,pg 1 */
-    0713001000000+(IOBA_UBMAP+1 & RMASK),   /*      wrio 0,763001(1); set ubmap */
-    0435040000000+(IOBA_TU & RMASK),        /*      iori 1,772440   ; rh addr */
-    0202040000000+FE_RHBASE,                /*      movem 1,FE_RHBASE */
-    0201000000040,                          /*      movei 0,40      ; ctrl reset */
-    0713001000010,                          /*      wrio 0,10(1)    ; ->MTFS */
-    0201100000031,                          /*      movei 2,31      ; space f */
-    0265740377014,                          /*      jsp 17,tpop     ; skip ucode */
-    0201100000071,                          /*      movei 2,71      ; read f */
-    0265740377014,                          /*      jsp 17,tpop     ; read boot */
-    0254000001000,                          /*      jrst 1000       ; start */
-    0200000000000+FE_MTFMT,                 /* tpop:move 0,FE_MTFMT ; den,fmt,slv */
-    0713001000032,                          /*      wrio 0,32(1)    ; ->MTTC */
-    0201000000011,                          /*      movei 0,11      ; clr+go */
-    0713001000000,                          /*      wrio 0,0(1)     ; ->MTCS1 */
-    0201140176000,                          /*      movei 3,176000  ; wd cnt */
-    0201200004000,                          /*      movei 4,4000    ; addr */
-    0200240000000+FE_MTFMT,                 /*      move 5,FE_MTFMT ; unit */
-    0201300000000,                          /*      movei 6,0       ; fmtr */
-    0713141000002,                          /*      wrio 3,2(1)     ; ->MTWC */
-    0713201000004,                          /*      wrio 4,4(1)     ; ->MTBA */
-    0713301000006,                          /*      wrio 6,6(1)     ; ->MTFC */
-    0713301000010,                          /*      wrio 6,10(1)    ; ->MTFS */
-    0713241000032,                          /*      wrio 5,32(1)    ; ->MTTC */
-    0713101000000,                          /*      wrio 2,0(1)     ; ->MTCS1 */
-    0712341000012,                          /*      rdio 7,12(1)    ; read FS */
-    0606340000200,                          /*      trnn 7,200      ; test rdy */
-    0254000377032,                          /*      jrst .-2        ; loop */
-    0606340040000,                          /*      trnn 7,40000    ; test err */
-    0254017000000,                          /*      jrst 0(17)      ; return */
-    0712341000014,                          /*      rdio 7,14(1)    ; read err */
-    0302340001000,                          /*      caie 7,1000     ; fce? */
-    0254200377052,                          /*      halt */
-    0254017000000,                          /*      jrst 0(17)      ; return */
+    INT64_C(0510040000000)+FE_RHBASE,       /* boot:hllz  1,FE_RHBASE ; uba # */
+    INT64_C(0201000040001),                 /*      movei 0,40001   ; vld,pg 1 */
+    INT64_C(0713001000000)+((IOBA_UBMAP+1) & RMASK),   /*      wrio 0,763001(1); set ubmap */
+    INT64_C(0200040000000)+FE_RHBASE,       /*      move 1,FE_RHBASE */
+    INT64_C(0201000000040),                 /*      movei 0,40      ; ctrl reset */
+    INT64_C(0713001000010),                 /*      wrio 0,10(1)    ; ->MTFS */
+    INT64_C(0200300000000)+FE_UNIT,         /*      move 6,FE_UNIT  ; fmtr */
+    INT64_C(0713301000010),                 /*      wrio 6,10(1)    ; ->MTCS2 */
+
+    INT64_C(0200240000000)+FE_MTFMT,        /*10    move 5,FE_MTFMT ; slave, dens, fmt */
+    INT64_C(0713241000032),                 /*      wrio 5,32(1)    ; ->MTTC */
+    INT64_C(0712001000012),                 /*      rdio 0,12(1)    ; MTFS */
+    INT64_C(0640000010600),                 /*      trc  0,10600    ; MOL + DPR + RDY */
+    INT64_C(0642000010600),                 /*      trce 0,10600    ; */
+    INT64_C(0254000377012),                 /*      jrst .-3        ; wait */
+    INT64_C(0201000000011),                 /*      movei 0,11      ; clr+go */
+    INT64_C(0713001000000),                 /*      wrio 0,0(1)     ; ->MTCS1 */
+
+    INT64_C(0201000000377),                 /*20    movei 0,1       ; Formatter */
+    INT64_C(0242006000000),                 /*      lsh 0,(6)       ; attn bit */
+    INT64_C(0713001000016),                 /*      wrio 0,16(1)    ; Clear on-line attn */
+    INT64_C(0201100000031),                 /*      movei 2,31      ; space f */
+    INT64_C(0265740377030),                 /*      jsp 17,tpop     ; skip ucode */
+    INT64_C(0201100000071),                 /*      movei 2,71      ; read f */
+    INT64_C(0265740377030),                 /*      jsp 17,tpop     ; read boot */
+    INT64_C(0254000001000),                 /*      jrst 1000       ; start */
+
+                                            /*30 */
+    INT64_C(0713241000032),                 /* tpop:wrio 5,32(1)    ; ->MTTC */
+    INT64_C(0201000000011),                 /*      movei 0,11      ; clr+go */
+    INT64_C(0713001000000),                 /*      wrio 0,0(1)     ; ->MTCS1 */
+    INT64_C(0201140176000),                 /*      movei 3,176000  ; wd cnt */
+    INT64_C(0713141000002),                 /*      wrio 3,2(1)     ; ->MTWC */
+    INT64_C(0201200004000),                 /*      movei 4,4000    ; addr */
+    INT64_C(0713201000004),                 /*      wrio 4,4(1)     ; ->MTBA */
+    INT64_C(0400400000000),                 /*      setz 10,        ; max fc */
+
+    INT64_C(0713401000006),                 /*40    wrio 10,6(1)    ; ->MTFC */
+    INT64_C(0713301000010),                 /*      wrio 6,10(1)    ; ->MTCS2 reset errs */
+    INT64_C(0713241000032),                 /*      wrio 5,32(1)    ; ->MTTC reset errs */
+    INT64_C(0713101000000),                 /*      wrio 2,0(1)     ; OP ->MTCS1 */
+    INT64_C(0712341000012),                 /*      rdio 7,12(1)    ; read FS */
+    INT64_C(0606340000200),                 /*      trnn 7,200      ; test rdy */
+    INT64_C(0254000377044),                 /*      jrst .-2        ; loop */
+    INT64_C(0606340040000),                 /*      trnn 7,40000    ; test err */
+
+    INT64_C(0254017000000),                 /*50    jrst 0(17)      ; return */
+    INT64_C(0712341000014),                 /*      rdio 7,14(1)    ; read err */
+    INT64_C(0302340001000),                 /*      caie 7,1000     ; fce? */
+    INT64_C(0254200377053),                 /*      halt . */
+    INT64_C(0254017000000),                 /*      jrst 0(17)      ; return */
     };
 
 static const d10 boot_rom_its[] = {
-    0515040000003,                          /* boot:hrlzi 1,3       ; uba # - not used */
-    0201000040001,                          /*      movei 0,40001   ; vld,pg 1 */
-    0714000000000+(IOBA_UBMAP+1 & RMASK),   /*      iowri 0,763001  ; set ubmap */
-    0435040000000+(IOBA_TU & RMASK),        /*      iori 1,772440   ; rh addr */
-    0202040000000+FE_RHBASE,                /*      movem 1,FE_RHBASE */
-    0201000000040,                          /*      movei 0,40      ; ctrl reset */
-    0714001000010,                          /*      iowri 0,10(1)   ; ->MTFS */
-    0201100000031,                          /*      movei 2,31      ; space f */
-    0265740377014,                          /*      jsp 17,tpop     ; skip ucode */
-    0201100000071,                          /*      movei 2,71      ; read f */
-    0265740377014,                          /*      jsp 17,tpop     ; read boot */
-    0254000001000,                          /*      jrst 1000       ; start */
-    0200000000000+FE_MTFMT,                 /* tpop:move 0,FE_MTFMT ; den,fmt,slv */
-    0714001000032,                          /*      iowri 0,32(1)   ; ->MTTC */
-    0201000000011,                          /*      movei 0,11      ; clr+go */
-    0714001000000,                          /*      iowri 0,0(1)    ; ->MTCS1 */
-    0201140176000,                          /*      movei 3,176000  ; wd cnt */
-    0201200004000,                          /*      movei 4,4000    ; addr */
-    0200240000000+FE_MTFMT,                 /*      move 5,FE_MTFMT ; unit */
-    0201300000000,                          /*      movei 6,0       ; fmtr */
-    0714141000002,                          /*      iowri 3,2(1)    ; ->MTWC */
-    0714201000004,                          /*      iowri 4,4(1)    ; ->MTBA */
-    0714301000006,                          /*      iowri 6,6(1)    ; ->MTFC */
-    0714301000010,                          /*      iowri 6,10(1)   ; ->MTFS */
-    0714241000032,                          /*      iowri 5,32(1)   ; ->MTTC */
-    0714101000000,                          /*      iowri 2,0(1)    ; ->MTCS1 */
-    0710341000012,                          /*      iordi 7,12(1)   ; read FS */
-    0606340000200,                          /*      trnn 7,200      ; test rdy */
-    0254000377032,                          /*      jrst .-2        ; loop */
-    0606340040000,                          /*      trnn 7,40000    ; test err */
-    0254017000000,                          /*      jrst 0(17)      ; return */
-    0710341000014,                          /*      iordi 7,14(1)   ; read err */
-    0302340001000,                          /*      caie 7,1000     ; fce? */
-    0254200377052,                          /*      halt */
-    0254017000000,                          /*      jrst 0(17)      ; return */
+    INT64_C(0510040000000)+FE_RHBASE,       /* boot:hllz 1,FE_RHBASE ; uba # - not used */
+    INT64_C(0201000040001),                 /*      movei 0,40001   ; vld,pg 1 */
+    INT64_C(0714000000000)+((IOBA_UBMAP+1) & RMASK),   /*      iowri 0,763001  ; set ubmap */
+    INT64_C(0200040000000)+FE_RHBASE,       /*      move 1,FE_RHBASE */
+    INT64_C(0201000000040),                 /*      movei 0,40      ; ctrl reset */
+    INT64_C(0714001000010),                 /*      iowri 0,10(1)   ; ->MTFS */
+    INT64_C(0200300000000)+FE_UNIT,         /*      move 6,FE_UNIT  ; fmtr */
+    INT64_C(0714301000010),                 /*      iowri 6,10(1)   ; ->MTFS */
+
+    INT64_C(0200240000000)+FE_MTFMT,        /*20    move 5,FE_MTFMT ; slave, dens, fmt */
+    INT64_C(0714241000032),                 /*      iowri 5,32(1)   ; ->MTTC */
+    INT64_C(0710001000012),                 /*      iordi 0,12(1)   ; read FS */
+    INT64_C(0640000010600),                 /*      trc  0,10600    ; MOL + DPR + RDY */
+    INT64_C(0642000010600),                 /*      trce 0,10600    ; */
+    INT64_C(0254000377012),                 /*      jrst .-3        ; wait */
+    INT64_C(0201000000011),                 /*      movei 0,11      ; clr+go */
+    INT64_C(0714001000000),                 /*      iowri 0,0(1)    ; ->MTCS1 */
+
+    INT64_C(0201000000377),                 /*30    movei 0,1       ; Formatter */
+    INT64_C(0242006000000),                 /*      lsh 0,(6)       ; attn bit */
+    INT64_C(0714001000016),                 /*      iowri 0,16(1)    ; Clear on-line attn */
+    INT64_C(0201100000031),                 /*      movei 2,31      ; space f */
+    INT64_C(0265740377030),                 /*      jsp 17,tpop     ; skip ucode */
+    INT64_C(0201100000071),                 /*      movei 2,71      ; read f */
+    INT64_C(0265740377030),                 /*      jsp 17,tpop     ; read boot */
+    INT64_C(0254000001000),                 /*      jrst 1000       ; start */
+
+                                            /*30 */
+    INT64_C(0714241000032),                 /* tpop:iowri 5,32(1)   ; ->MTTC */
+    INT64_C(0201000000011),                 /*      movei 0,11      ; clr+go */
+    INT64_C(0714001000000),                 /*      iowri 0,0(1)    ; ->MTCS1 */
+    INT64_C(0201140176000),                 /*      movei 3,176000  ; wd cnt */
+    INT64_C(0714141000002),                 /*      iowri 3,2(1)    ; ->MTWC */
+    INT64_C(0201200004000),                 /*      movei 4,4000    ; addr */
+    INT64_C(0714201000004),                 /*      iowri 4,4(1)    ; ->MTBA */
+    INT64_C(0400400000000),                 /*      setz 10,        ; max fc */
+
+    INT64_C(0714401000006),                 /*40    iowri 10,6(1)    ; ->MTFC */
+    INT64_C(0714301000010),                 /*      iowri 6,10(1)   ; ->MTFS */
+    INT64_C(0714241000032),                 /*      iowri 5,32(1)   ; ->MTTC */
+    INT64_C(0714101000000),                 /*      iowri 2,0(1)    ; ->MTCS1 */
+    INT64_C(0710341000012),                 /*      iordi 7,12(1)   ; read FS */
+    INT64_C(0606340000200),                 /*      trnn 7,200      ; test rdy */
+    INT64_C(0254000377044),                 /*      jrst .-2        ; loop */
+    INT64_C(0606340040000),                 /*      trnn 7,40000    ; test err */
+
+    INT64_C(0254017000000),                 /*50    jrst 0(17)      ; return */
+    INT64_C(0710341000014),                 /*      iordi 7,14(1)   ; read err */
+    INT64_C(0302340001000),                 /*      caie 7,1000     ; fce? */
+    INT64_C(0254200377053),                 /*      halt . */
+    INT64_C(0254017000000),                 /*      jrst 0(17)      ; return */
     };
 
 t_stat tu_boot (int32 unitno, DEVICE *dptr)
 {
-int32 i;
+size_t i;
 extern a10 saved_PC;
+UNIT *uptr;
+
+unitno &= TC_M_UNIT;
+uptr = tu_unit + unitno;
+if (!(uptr->flags & UNIT_ATT))
+    return SCPE_NOATT;
+
+M[FE_RHBASE] = tu_dib.ba;
+M[FE_UNIT] = 0;                             /* Only one formatter in this implementation */
+
+assert (sizeof(boot_rom_dec) == sizeof(boot_rom_its));
 
-M[FE_UNIT] = 0;
 M[FE_MTFMT] = (unitno & TC_M_UNIT) | (TC_1600 << TC_V_DEN) | (TC_10C << TC_V_FMT);
 tu_unit[unitno].pos = 0;
+
+M[FE_KEEPA] = (M[FE_KEEPA] & ~INT64_C(0xFF)) | ((sim_switches & SWMASK ('A'))? 010 : 0);
+
 for (i = 0; i < BOOT_LEN; i++)
     M[BOOT_START + i] = Q_ITS? boot_rom_its[i]: boot_rom_dec[i];
 saved_PC = BOOT_START;
