@@ -1,6 +1,6 @@
 /* pdp11_cpumod.c: PDP-11 CPU model-specific features
 
-   Copyright (c) 2004-2008, Robert M Supnik
+   Copyright (c) 2004-2016, Robert M Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -25,14 +25,17 @@
 
    system       PDP-11 model-specific registers
 
+   04-Mar-16    RMS     Fixed maximum memory sizes to exclude IO page
+   14-Mar-16    RMS     Modified to keep cpu_memsize in sync with MEMSIZE
+   06-Jun-13    RMS     Fixed change model to set memory size last
    20-May-08    RMS     Added JCSR default for KDJ11B, KDJ11E
    22-Apr-08    RMS     Fixed write behavior of 11/70 MBRK, LOSIZE, HISIZE
-                        (found by Walter Mueller)
+                        (Walter Mueller)
    29-Apr-07    RMS     Don't run bus setup routine during RESTORE
    30-Aug-05    RMS     Added additional 11/60 registers
    16-Aug-05    RMS     Fixed C++ declaration and cast problems
-   15-Feb-05    RMS     Fixed bug in SHOW MODEL (from Sergey Okhapkin)
-   19-Jan-05    RMS     Added variable SYSID, MBRK write (from Tim Chapman)
+   15-Feb-05    RMS     Fixed bug in SHOW MODEL (Sergey Okhapkin)
+   19-Jan-05    RMS     Added variable SYSID, MBRK write (Tim Chapman)
 
    This module includes CPU- and system-specific registers, such as the Unibus
    map and control registers on 22b Unibus systems, the board registers for the
@@ -84,13 +87,11 @@ static int32 clk_tps_map[4] = { 60, 60, 50, 800 };
 
 extern uint16 *M;
 extern int32 R[8];
-extern DEVICE cpu_dev, *sim_devices[];
+extern DEVICE cpu_dev;
 extern UNIT cpu_unit;
-extern FILE *sim_log;
 extern int32 STKLIM, PIRQ;
 extern uint32 cpu_model, cpu_type, cpu_opt;
 extern int32 clk_fie, clk_fnxm, clk_tps, clk_default;
-extern int32 sim_switches;
 
 t_stat CPU24_rd (int32 *data, int32 addr, int32 access);
 t_stat CPU24_wr (int32 data, int32 addr, int32 access);
@@ -248,7 +249,8 @@ CNFTAB cnf_tab[] = {
 static const char *opt_name[] = {
     "Unibus", "Qbus", "EIS", "NOEIS", "FIS", "NOFIS",
     "FPP", "NOFPP", "CIS", "NOCIS", "MMU", "NOMMU",
-    "RH11", "RH70",     "PARITY", "NOPARITY", "Unibus map", "No map", NULL
+    "RH11", "RH70", "PARITY", "NOPARITY", "Unibus map", "No map",
+    "BEVENT enabled", "BEVENT disabled", NULL
     };
 
 static const char *jcsr_val[4] = {
@@ -1075,7 +1077,7 @@ t_stat r;
 for (i = 0; cnf_tab[i].dib != NULL; i++) {              /* loop thru config tab */
     if (((cnf_tab[i].cpum == 0) || (cpu_type & cnf_tab[i].cpum)) &&
         ((cnf_tab[i].optm == 0) || (cpu_opt & cnf_tab[i].optm))) {
-        if (r = build_ubus_tab (&cpu_dev, cnf_tab[i].dib)) /* add to dispatch tab */
+        if ((r = build_ubus_tab (&cpu_dev, cnf_tab[i].dib)))/* add to dispatch tab */
              return r;
         }
     }
@@ -1092,14 +1094,14 @@ if (val >= MOD_MAX)
     return SCPE_IERR;
 if (val == (int32) cpu_model)
     return SCPE_OK;
-if (MEMSIZE > cpu_tab[val].maxm)
-    cpu_set_size (uptr, cpu_tab[val].maxm, NULL, NULL);
-if (MEMSIZE > cpu_tab[val].maxm)
-    return SCPE_INCOMP;
 cpu_model = val;
 cpu_type = 1u << cpu_model;
 cpu_opt = cpu_tab[cpu_model].std;
 cpu_set_bus (cpu_opt);
+if (MEMSIZE > cpu_tab[val].maxm)
+    cpu_set_size (uptr, cpu_tab[val].maxm, NULL, NULL);
+if (MEMSIZE > cpu_tab[val].maxm)
+    return SCPE_INCOMP;
 reset_all (0);                                          /* reset world */
 return SCPE_OK;
 }
@@ -1149,9 +1151,11 @@ uint32 i, clim;
 uint16 *nM;
 
 if ((val <= 0) ||
-    (val > (int32) cpu_tab[cpu_model].maxm) ||
+    (val > ((int32) cpu_tab[cpu_model].maxm)) ||
     ((val & 07777) != 0))
     return SCPE_ARG;
+if (val > ((int32) (cpu_tab[cpu_model].maxm - IOPAGESIZE)))
+    val = (int32) (cpu_tab[cpu_model].maxm - IOPAGESIZE);
 for (i = val; i < MEMSIZE; i = i + 2)
     mc = mc | M[i >> 1];
 if ((mc != 0) && !get_yn ("Really truncate memory [N]?", FALSE))
@@ -1159,7 +1163,7 @@ if ((mc != 0) && !get_yn ("Really truncate memory [N]?", FALSE))
 nM = (uint16 *) calloc (val >> 1, sizeof (uint16));
 if (nM == NULL)
     return SCPE_MEM;
-clim = (((t_addr) val) < MEMSIZE)? val: MEMSIZE;
+clim = (((t_addr) val) < MEMSIZE)? (uint32)val: MEMSIZE;
 for (i = 0; i < clim; i = i + 2)
     nM[i >> 1] = M[i >> 1];
 free (M);
@@ -1186,9 +1190,7 @@ for (i = 0; (dptr = sim_devices[i]) != NULL; i++) {
     if ((dptr->flags & DEV_DISABLE) &&                  /* disable-able? */
         !(dptr->flags & DEV_DIS) &&                     /* enabled? */
         ((dptr->flags & mask) == 0)) {                  /* not allowed? */
-        printf ("Disabling %s\n", sim_dname (dptr));
-        if (sim_log)
-            fprintf (sim_log, "Disabling %s\n", sim_dname (dptr));
+        sim_printf ("Disabling %s\n", sim_dname (dptr));
         dptr->flags = dptr->flags | DEV_DIS;
         }
     }
