@@ -1,6 +1,6 @@
 /* pdp11_cpumod.h: PDP-11 CPU model definitions
 
-   Copyright (c) 2004-2008, Robert M Supnik
+   Copyright (c) 2004-2015, Robert M Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -23,15 +23,16 @@
    used in advertising or otherwise to promote the sale, use or other dealings
    in this Software without prior written authorization from Robert M Supnik.
 
+   30-Dec-15    RMS     Added 11/03, 11/23 BEVENT disable
    22-Apr-08    RMS     Added 11/70 MBRK register
    30-Aug-05    RMS     Added additional 11/60 registers
 */
 
-#ifndef _PDP11_CPUMOD_H_
-#define _PDP11_CPUMOD_H_        0
+#ifndef PDP11_CPUMOD_H_
+#define PDP11_CPUMOD_H_        0
 
-#define SOP_1103        (BUS_Q)
-#define OPT_1103        (OPT_EIS|OPT_FIS)
+#define SOP_1103        (BUS_Q|OPT_BVT)
+#define OPT_1103        (OPT_EIS|OPT_FIS|OPT_BVT)
 #define PSW_1103        0000377
 
 #define SOP_1104        (BUS_U)
@@ -46,8 +47,8 @@
 #define OPT_1120        0
 #define PSW_1120        0000377
 
-#define SOP_1123        (BUS_Q|OPT_EIS|OPT_FPP|OPT_MMU)
-#define OPT_1123        (OPT_FPP|OPT_CIS)
+#define SOP_1123        (BUS_Q|OPT_EIS|OPT_FPP|OPT_MMU|OPT_BVT)
+#define OPT_1123        (OPT_FPP|OPT_CIS|OPT_BVT)
 #define PSW_F           0170777
 #define PAR_F           0177777
 #define PDR_F           0077516
