@@ -1,6 +1,6 @@
 /* pdp11_defs.h: PDP-11 simulator definitions
 
-   Copyright (c) 1993-2008, Robert M Supnik
+   Copyright (c) 1993-2017, Robert M Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -26,16 +26,25 @@
    The author gratefully acknowledges the help of Max Burnet, Megan Gentry,
    and John Wilson in resolving questions about the PDP-11
 
+   10-Feb-17    RMS     Fixed RJS11 register block length (Mark Hill)
+   19-Jan-17    RMS     Moved CR11 to BR6, leaving CD11 at BR4 (Mark Pizzolato)
+   10-Mar-16    RMS     Added UC15 support
+   30-Dec-15    RMS     Added NOBVT option
+   23-Oct-13    RMS     Added cpu_set_boot prototype
+   02-Sep-13    RMS     Added third Massbus adapter and RS drive
+   12-Dec-12    RMS     Fixed base address for RQB, RQC, RQD
+   11-Dec-11    RMS     Fixed priority of PIRQ vs IO; added INT_INTERNALn
+   22-May-10    RMS     Added check for 64b definitions
    19-Nov-08    RMS     Moved I/O support routines to I/O library
    16-May-08    RMS     Added KE11A, DC11 support
-   02-Feb-08    RMS     Fixed DMA memory address limit test (found by John Dundas)
-   25-Jan-08    RMS     Added RC11, KG11A support (from John Dundas)
+   02-Feb-08    RMS     Fixed DMA memory address limit test (John Dundas)
+   25-Jan-08    RMS     Added RC11, KG11A support (John Dundas)
    16-Dec-06    RMS     Added TA11 support
    29-Oct-06    RMS     Added clock coscheduling
    06-Jul-06    RMS     Added multiple KL11/DL11 support
    26-Jun-06    RMS     Added RF11 support
-   24-May-06    RMS     Added 11/44 DR support (from CIS diagnostic)
-   17-May-06    RMS     Added CR11/CD11 support (from John Dundas)
+   24-May-06    RMS     Added 11/44 DR support (CIS diagnostic)
+   17-May-06    RMS     Added CR11/CD11 support (John Dundas)
    30-Sep-04    RMS     Added Massbus support
                         Removed Map_Addr prototype
                         Removed map argument from Unibus routines
@@ -45,7 +54,7 @@
    22-Dec-03    RMS     Added second DEUNA/DELUA support
    18-Oct-03    RMS     Added DECtape off reel message
    19-May-03    RMS     Revised for new conditional compilation
-   05-Apr-03    RMS     Fixed bug in MMR1 update (found by Tim Stark)
+   05-Apr-03    RMS     Fixed bug in MMR1 update (Tim Stark)
    28-Feb-03    RMS     Added TM logging support
    19-Jan-03    RMS     Changed mode definitions for Apple Dev Kit conflict
    11-Nov-02    RMS     Changed log definitions to be VAX compatible
@@ -70,8 +79,8 @@
    10-Feb-01    RMS     Added DECtape support
 */
 
-#ifndef _PDP11_DEFS_H
-#define _PDP11_DEFS_H   0
+#ifndef PDP11_DEFS_H
+#define PDP11_DEFS_H   0
 
 #ifndef VM_PDP11
 #define VM_PDP11        0
@@ -80,6 +89,10 @@
 #include "sim_defs.h"                                   /* simulator defns */
 #include <setjmp.h>
 
+#if defined(USE_INT64) || defined(USE_ADDR64)
+#error "PDP-11 does not support 64b values!"
+#endif
+
 /* Architectural constants */
 
 #define STKL_R          0340                            /* stack limit */
@@ -87,7 +100,6 @@
 #define VASIZE          0200000                         /* 2**16 */
 #define VAMASK          (VASIZE - 1)                    /* 2**16 - 1 */
 #define MEMSIZE64K      0200000                         /* 2**16 */
-#define INIMEMSIZE      001000000                       /* 2**18 */
 #define UNIMEMSIZE      001000000                       /* 2**18 */
 #define UNIMASK         (UNIMEMSIZE - 1)                /* 2**18 - 1 */
 #define IOPAGEBASE      017760000                       /* 2**22 - 2**13 */
@@ -96,7 +108,6 @@
 #define MAXMEMSIZE      020000000                       /* 2**22 */
 #define PAMASK          (MAXMEMSIZE - 1)                /* 2**22 - 1 */
 #define MEMSIZE         (cpu_unit.capac)
-#define ADDR_IS_MEM(x)  (((t_addr) (x)) < cpu_memsize)  /* use only in sim! */
 #define DMASK           0177777
 
 /* CPU models */
@@ -165,6 +176,7 @@
 #define OPT_RH11        (1u << 6)                       /* RH11 */
 #define OPT_PAR         (1u << 7)                       /* parity */
 #define OPT_UBM         (1u << 8)                       /* UBM */
+#define OPT_BVT         (1u << 9)                       /* BEVENT */
 
 #define CPUT(x)         ((cpu_type & (x)) != 0)
 #define CPUO(x)         ((cpu_opt & (x)) != 0)
@@ -518,16 +530,22 @@ typedef struct pdp_dib DIB;
 #define IOLN_XUB        010
 #define IOBA_RQB        (IOPAGEBASE + 000334 + (020 * (DZ_MUXES / 2)))
 #define IOLN_RQB        004
-#define IOBA_RQC        (IOPAGEBASE + IOBA_RQB + IOLN_RQB)
+#define IOBA_RQC        (IOBA_RQB + IOLN_RQB)
 #define IOLN_RQC        004
-#define IOBA_RQD        (IOPAGEBASE + IOBA_RQC + IOLN_RQC)
+#define IOBA_RQD        (IOBA_RQC + IOLN_RQC)
 #define IOLN_RQD        004
 #define IOBA_VH         (IOPAGEBASE + 000440)           /* DHQ11 */
 #define IOLN_VH         020
+#define IOBA_UCA        (IOPAGEBASE + 007770)           /* UC15 DR11 #1 */
+#define IOLN_UCA        006
+#define IOBA_UCB        (IOPAGEBASE + 007760)           /* UC15 DR11 #2 */
+#define IOLN_UCB        006
 #define IOBA_UBM        (IOPAGEBASE + 010200)           /* Unibus map */
 #define IOLN_UBM        (UBM_LNT_LW * sizeof (int32))
-#define	IOBA_KG         (IOPAGEBASE + 010700)           /* KG11-A */
-#define	IOLN_KG         006
+#define IOBA_RS         (IOPAGEBASE + 012040)           /* RHC: RS03/RS04 */
+#define IOLN_RS         034
+#define IOBA_KG         (IOPAGEBASE + 010700)           /* KG11-A */
+#define IOLN_KG         006
 #define IOBA_RQ         (IOPAGEBASE + 012150)           /* RQDX3 */
 #define IOLN_RQ         004
 #define IOBA_SUP        (IOPAGEBASE + 012200)           /* supervisor APR's */
@@ -540,7 +558,7 @@ typedef struct pdp_dib DIB;
 #define IOLN_KIPAR      020
 #define IOBA_KDPAR      (IOPAGEBASE + 012360)
 #define IOLN_KDPAR      020
-#define IOBA_TU         (IOPAGEBASE + 012440)           /* TU */
+#define IOBA_TU         (IOPAGEBASE + 012440)           /* RHB: TU */
 #define IOLN_TU         040
 #define IOBA_MMR3       (IOPAGEBASE + 012516)           /* MMR3 */
 #define IOLN_MMR3       002
@@ -564,7 +582,7 @@ typedef struct pdp_dib DIB;
 #define IOLN_XU         010
 #define IOBA_DL         (IOPAGEBASE + 016500)           /* extra KL11/DL11 */
 #define IOLN_DL         (DLX_LINES * 010)
-#define IOBA_RP         (IOPAGEBASE + 016700)           /* RP/RM */
+#define IOBA_RP         (IOPAGEBASE + 016700)           /* RHA: RP/RM */
 #define IOLN_RP         054
 #define IOBA_CR         (IOPAGEBASE + 017160)           /* CD/CR/CM */
 #define IOLN_CR         010
@@ -623,61 +641,71 @@ typedef struct pdp_dib DIB;
 #define IOBA_PSW        (IOPAGEBASE + 017776)           /* PSW */
 #define IOLN_PSW        002
 
-/* Interrupt assignments; within each level, priority is right to left */
+/* Interrupt assignments; within each level, priority is right to left
+   PIRQn has the highest priority with a level and is always bit <0>
+   On level 6, the clock is second highest priority */
 
 #define IPL_HLVL        8                               /* # int levels */
+#define IPL_HMIN        4                               /* lowest IO int level */
 
 #define INT_V_PIR7      0                               /* BR7 */
+#define INT_V_UCA       1
 
-#define INT_V_CLK       0                               /* BR6 */
-#define INT_V_PCLK      1
-#define INT_V_DTA       2
-#define INT_V_TA        3
-#define INT_V_PIR6      4
+#define INT_V_PIR6      0                               /* BR6 */
+#define INT_V_CLK       1
+#define INT_V_PCLK      2
+#define INT_V_DTA       3
+#define INT_V_TA        4
+#define INT_V_CR        5                               /* CR11 */
 
-#define INT_V_RK        0                               /* BR5 */
-#define INT_V_RL        1
-#define INT_V_RX        2
-#define INT_V_TM        3
-#define INT_V_RP        4
-#define INT_V_TS        5
-#define INT_V_HK        6
-#define INT_V_RQ        7
-#define INT_V_DZRX      8
-#define INT_V_DZTX      9
-#define INT_V_TQ        10
-#define INT_V_RY        11
-#define INT_V_XQ        12
-#define INT_V_XU        13
-#define INT_V_TU        14
-#define INT_V_RF        15
-#define INT_V_RC        16
-#define INT_V_PIR5      17
+#define INT_V_PIR5      0                               /* BR5 */
+#define INT_V_RK        1
+#define INT_V_RL        2
+#define INT_V_RX        3
+#define INT_V_TM        4
+#define INT_V_RP        5
+#define INT_V_TS        6
+#define INT_V_HK        7
+#define INT_V_RQ        8
+#define INT_V_DZRX      9
+#define INT_V_DZTX      10
+#define INT_V_TQ        11
+#define INT_V_RY        12
+#define INT_V_XQ        13
+#define INT_V_XU        14
+#define INT_V_TU        15
+#define INT_V_RF        16
+#define INT_V_RC        17
+#define INT_V_RS        18
+#define INT_V_UCB       19
 
-#define INT_V_TTI       0                               /* BR4 */
-#define INT_V_TTO       1
-#define INT_V_PTR       2
-#define INT_V_PTP       3
-#define INT_V_LPT       4
-#define INT_V_VHRX      5
-#define INT_V_VHTX      6  
-#define INT_V_CR        7
-#define INT_V_DLI       8
-#define INT_V_DLO       9
-#define INT_V_DCI       10
-#define INT_V_DCO       11
-#define INT_V_PIR4      12
+#define INT_V_PIR4      0                               /* BR4 */
+#define INT_V_TTI       1
+#define INT_V_TTO       2
+#define INT_V_PTR       3
+#define INT_V_PTP       4
+#define INT_V_LPT       5
+#define INT_V_VHRX      6
+#define INT_V_VHTX      7  
+#define INT_V_CD        8                               /* CD11 */
+#define INT_V_DLI       9
+#define INT_V_DLO       10
+#define INT_V_DCI       11
+#define INT_V_DCO       12
 
 #define INT_V_PIR3      0                               /* BR3 */
 #define INT_V_PIR2      0                               /* BR2 */
 #define INT_V_PIR1      0                               /* BR1 */
 
 #define INT_PIR7        (1u << INT_V_PIR7)
+#define INT_UCB         (1u << INT_V_UCB)
+#define INT_PIR6        (1u << INT_V_PIR6)
 #define INT_CLK         (1u << INT_V_CLK)
 #define INT_PCLK        (1u << INT_V_PCLK)
 #define INT_DTA         (1u << INT_V_DTA)
 #define INT_TA          (1u << INT_V_TA)
-#define INT_PIR6        (1u << INT_V_PIR6)
+#define INT_CR          (1u << INT_V_CR)
+#define INT_PIR5        (1u << INT_V_PIR5)
 #define INT_RK          (1u << INT_V_RK)
 #define INT_RL          (1u << INT_V_RL)
 #define INT_RX          (1u << INT_V_RX)
@@ -695,28 +723,39 @@ typedef struct pdp_dib DIB;
 #define INT_TU          (1u << INT_V_TU)
 #define INT_RF          (1u << INT_V_RF)
 #define INT_RC          (1u << INT_V_RC)
-#define INT_PIR5        (1u << INT_V_PIR5)
-#define INT_PTR         (1u << INT_V_PTR)
-#define INT_PTP         (1u << INT_V_PTP)
+#define INT_RS          (1u << INT_V_RS)
+#define INT_UCA         (1u << INT_V_UCA)
+#define INT_PIR4        (1u << INT_V_PIR4)
 #define INT_TTI         (1u << INT_V_TTI)
 #define INT_TTO         (1u << INT_V_TTO)
+#define INT_PTR         (1u << INT_V_PTR)
+#define INT_PTP         (1u << INT_V_PTP)
 #define INT_LPT         (1u << INT_V_LPT)
 #define INT_VHRX        (1u << INT_V_VHRX)
 #define INT_VHTX        (1u << INT_V_VHTX)
-#define INT_CR          (1u << INT_V_CR)
+#define INT_CD          (1u << INT_V_CD)
 #define INT_DLI         (1u << INT_V_DLI)
 #define INT_DLO         (1u << INT_V_DLO)
 #define INT_DCI         (1u << INT_V_DCI)
 #define INT_DCO         (1u << INT_V_DCO)
-#define INT_PIR4        (1u << INT_V_PIR4)
 #define INT_PIR3        (1u << INT_V_PIR3)
 #define INT_PIR2        (1u << INT_V_PIR2)
 #define INT_PIR1        (1u << INT_V_PIR1)
 
-#define IPL_CLK         6                               /* int pri levels */
+#define INT_INTERNAL7   (INT_PIR7)
+#define INT_INTERNAL6   (INT_PIR6 | INT_CLK)
+#define INT_INTERNAL5   (INT_PIR5)
+#define INT_INTERNAL4   (INT_PIR4)
+#define INT_INTERNAL3   (INT_PIR3)
+#define INT_INTERNAL2   (INT_PIR2)
+#define INT_INTERNAL1   (INT_PIR1)
+
+#define IPL_UCB         7                               /* int pri levels */
+#define IPL_CLK         6
 #define IPL_PCLK        6
 #define IPL_DTA         6
 #define IPL_TA          6
+#define IPL_CR          6
 #define IPL_RK          5
 #define IPL_RL          5
 #define IPL_RX          5
@@ -734,6 +773,8 @@ typedef struct pdp_dib DIB;
 #define IPL_TU          5
 #define IPL_RF          5
 #define IPL_RC          5
+#define IPL_RS          5
+#define IPL_UCA         5
 #define IPL_PTR         4
 #define IPL_PTP         4
 #define IPL_TTI         4
@@ -741,7 +782,7 @@ typedef struct pdp_dib DIB;
 #define IPL_LPT         4
 #define IPL_VHRX        4
 #define IPL_VHTX        4
-#define IPL_CR          4
+#define IPL_CD          4
 #define IPL_DLI         4
 #define IPL_DLO         4
 #define IPL_DCI         4
@@ -771,6 +812,7 @@ typedef struct pdp_dib DIB;
 #define VEC_RL          0160
 #define VEC_LPT         0200
 #define VEC_RF          0204
+#define VEC_RS          0204
 #define VEC_HK          0210
 #define VEC_RC          0210
 #define VEC_RK          0220
@@ -791,7 +833,10 @@ typedef struct pdp_dib DIB;
 #define VEC_DZRX        0300
 #define VEC_DZTX        0304
 #define VEC_VHRX        0310
+#define VEC_CD          0230
 #define VEC_VHTX        0314
+#define VEC_UCA         0300
+#define VEC_UCB         0310
 
 /* Interrupt macros */
 
@@ -802,9 +847,10 @@ typedef struct pdp_dib DIB;
 
 /* Massbus definitions */
 
-#define MBA_NUM         2                               /* number of MBA's */
+#define MBA_NUM         3                               /* number of MBA's */
 #define MBA_RP          0                               /* MBA for RP */
 #define MBA_TU          1                               /* MBA for TU */
+#define MBA_RS          2                               /* MBA for RS */
 #define MBA_RMASK       037                             /* max 32 reg */
 #define MBE_NXD         1                               /* nx drive */
 #define MBE_NXR         2                               /* nx reg */
@@ -839,6 +885,43 @@ t_stat mba_show_num (FILE *st, UNIT *uptr, int32 val, void *desc);
 
 int32 clk_cosched (int32 wait);
 
+void cpu_set_boot (int32 pc);
+
 #include "pdp11_io_lib.h"
 
+#if defined (UC15)                                      /* UC15 */
+#define INIMODEL        MOD_1105
+#define INIOPTNS        SOP_1105
+#define INIMEMSIZE      00040000                       /* 16KB */
+#define ADDR_IS_MEM(x)  (((uint32) (x)) < uc15_memsize)
+
+#define RdMemW(pa)      uc15_RdMemW (pa)
+#define RdMemB(pa)      uc15_RdMemB (pa)
+#define WrMemW(pa,d)    uc15_WrMemW (pa, d)
+#define WrMemB(pa, d)   uc15_WrMemB (pa, d)
+
+uint32 uc15_memsize;
+int32 uc15_RdMemW (int32 pa);
+int32 uc15_RdMemB (int32 pa);
+void uc15_WrMemW (int32 pa, int32 d);
+void uc15_WrMemB (int32 pa, int32 d);
+int32 Map_Read18 (uint32 ba, int32 bc, uint32 *buf);
+int32 Map_Write18 (uint32 ba, int32 bc, uint32 *buf);
+
+#else                                                   /* PDP-11 */
+
+#define INIMODEL        MOD_1173
+#define INIOPTNS        SOP_1173
+#define INIMEMSIZE      001000000                       /* 2**18 */
+#define ADDR_IS_MEM(x)  (((t_addr) (x)) < MEMSIZE)
+
+#define RdMemW(pa)      (M[(pa) >> 1])
+#define RdMemB(pa)      ((((pa) & 1)? M[(pa) >> 1] >> 8: M[(pa) >> 1]) & 0377)
+#define WrMemW(pa,d)    M[(pa) >> 1] = (d)
+#define WrMemB(pa,d)    M[(pa) >> 1] = ((pa) & 1)? \
+                            ((M[(pa) >> 1] & 0377) | (((d) & 0377) << 8)): \
+                            ((M[(pa) >> 1] & ~0377) | ((d) & 0377))
+
+#endif
+
 #endif
