@@ -1,6 +1,6 @@
 /* pdp11_kg.c - Communications Arithmetic Option KG11-A
 
-   Copyright (c) 2007-2008, John A. Dundas III
+   Copyright (c) 2007-2010, John A. Dundas III
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -25,6 +25,7 @@
 
    kg           KG11-A Communications Arithmetic Option (M7251)
 
+   03-Jan-10    JAD     Eliminate gcc warnings
    08-Jan-08    JAD     First public release integrated with SIMH V3.7-3.
    09-Dec-07    JAD     SIMH-style debugging.
                         Finished validating against real hardware.
@@ -94,7 +95,6 @@
 #endif
 #include "pdp11_defs.h"
 
-extern FILE *sim_deb;
 extern REG cpu_reg[];
 extern int32 R[];
 
@@ -178,7 +178,7 @@ static t_stat set_units (UNIT *, int32, char *, void *);
 
 /* 16-bit rotate right */
 
-#define ROR(n,v)        (((v >> n) & DMASK) | (v << (16 - n)) & DMASK)
+#define ROR(n,v)        (((v >> n) & DMASK) | ((v << (16 - n)) & DMASK))
 
 /* 8-bit rotate right */
 
@@ -458,7 +458,7 @@ static void do_poly (int unit, t_bool step)
 
 static t_stat set_units (UNIT *u, int32 val, char *s, void *desc)
 {
-    int32       i, units;
+    uint32      i, units;
     t_stat      stat;
 
     if (s == NULL)
@@ -466,6 +466,10 @@ static t_stat set_units (UNIT *u, int32 val, char *s, void *desc)
     units = get_uint (s, 10, KG_UNITS, &stat);
     if (stat != SCPE_OK)
         return (stat);
+    if (units == 0)
+        return SCPE_ARG;
+    if (units == kg_dev.numunits)
+        return SCPE_OK;
     for (i = 0; i < KG_UNITS; i++) {
         if (i < units)
             kg_unit[i].flags &= ~UNIT_DIS;
@@ -473,5 +477,6 @@ static t_stat set_units (UNIT *u, int32 val, char *s, void *desc)
             kg_unit[i].flags |= UNIT_DIS;
     }
     kg_dev.numunits = units;
+    kg_reset (&kg_dev);
     return (SCPE_OK);
 }
