@@ -1,6 +1,6 @@
 /* pdp11_stddev.c: PDP-11 standard I/O devices simulator
 
-   Copyright (c) 1993-2008, Robert M Supnik
+   Copyright (c) 1993-2016, Robert M Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -26,13 +26,17 @@
    tti,tto      DL11 terminal input/output
    clk          KW11L (and other) line frequency clock
 
+   25-Sep-16    RMS     Added Dave Gesswein's fix to prevent data loss
+   02-Jan-16    RMS     Changed TTO default to 7B
+   30-Dec-15    RMS     Added NOBEVENT support
+   18-Apr-12    RMS     Modified to use clock coscheduling
    20-May-08    RMS     Standardized clock delay at 1mips
    18-Jun-07    RMS     Added UNIT_IDLE flag to console input, clock
-   29-Oct-06	RMS     Synced keyboard and clock
+   29-Oct-06    RMS     Synced keyboard and clock
                         Added clock coscheduling support
    05-Jul-06    RMS     Added UC only support for early DOS/RSTS
    22-Nov-05    RMS     Revised for new terminal processing routines
-   22-Sep-05    RMS     Fixed declarations (from Sterling Garwood)
+   22-Sep-05    RMS     Fixed declarations (Sterling Garwood)
    07-Jul-05    RMS     Removed extraneous externs
    11-Oct-04    RMS     Added clock model dependencies
    28-May-04    RMS     Removed SET TTI CTRL-C
@@ -46,7 +50,7 @@
                         Split DL11 dibs
    30-May-02    RMS     Widened POS to 32b
    26-Jan-02    RMS     Revised for multiple timers
-   09-Jan-02    RMS     Fixed bugs in KW11L (found by John Dundas)
+   09-Jan-02    RMS     Fixed bugs in KW11L (John Dundas)
    06-Jan-02    RMS     Split I/O address routines, revised enable/disable support
    29-Nov-01    RMS     Added read only unit support
    09-Nov-01    RMS     Added RQDX3 support
@@ -70,7 +74,7 @@
 #define CLK_DELAY       16667
 
 extern int32 int_req[IPL_HLVL];
-extern uint32 cpu_type;
+extern uint32 cpu_type, cpu_opt;
 
 int32 tti_csr = 0;                                      /* control/status */
 int32 tto_csr = 0;                                      /* control/status */
@@ -111,7 +115,7 @@ DIB tti_dib = {
     1, IVCL (TTI), VEC_TTI, { NULL }
     };
 
-UNIT tti_unit = { UDATA (&tti_svc, UNIT_IDLE, 0), 0 };
+UNIT tti_unit = { UDATA (&tti_svc, UNIT_IDLE, 0), KBD_POLL_WAIT };
 
 REG tti_reg[] = {
     { ORDATA (BUF, tti_unit.buf, 8) },
@@ -121,7 +125,7 @@ REG tti_reg[] = {
     { FLDATA (DONE, tti_csr, CSR_V_DONE) },
     { FLDATA (IE, tti_csr, CSR_V_IE) },
     { DRDATA (POS, tti_unit.pos, T_ADDR_W), PV_LEFT },
-    { DRDATA (TIME, tti_unit.wait, 24), PV_LEFT },
+    { DRDATA (TIME, tti_unit.wait, 24), PV_LEFT+REG_NZ },
     { NULL }
     };
 
@@ -157,7 +161,7 @@ DIB tto_dib = {
     1, IVCL (TTO), VEC_TTO, { NULL }
     };
 
-UNIT tto_unit = { UDATA (&tto_svc, TT_MODE_7P, 0), SERIAL_OUT_WAIT };
+UNIT tto_unit = { UDATA (&tto_svc, TT_MODE_7B, 0), SERIAL_OUT_WAIT };
 
 REG tto_reg[] = {
     { ORDATA (BUF, tto_unit.buf, 8) },
@@ -254,6 +258,7 @@ switch ((PA >> 1) & 01) {                               /* decode PA<1> */
         tti_csr = tti_csr & ~CSR_DONE;
         CLR_INT (TTI);
         *data = tti_unit.buf & 0377;
+        sim_activate_abs (&tti_unit, tti_unit.wait);    /* check soon for more input */
         return SCPE_OK;
         }                                               /* end switch PA */
 
@@ -287,7 +292,7 @@ t_stat tti_svc (UNIT *uptr)
 {
 int32 c;
 
-sim_activate (uptr, KBD_WAIT (uptr->wait, tmr_poll));   /* continue poll */
+sim_activate (uptr, clk_cosched (tmr_poll));            /* continue poll */
 if ((c = sim_poll_kbd ()) < SCPE_KFLAG)                 /* no char or error? */
     return c;
 if (c & SCPE_BREAK)                                     /* break? */
@@ -307,7 +312,7 @@ t_stat tti_reset (DEVICE *dptr)
 tti_unit.buf = 0;
 tti_csr = 0;
 CLR_INT (TTI);
-sim_activate_abs (&tti_unit, KBD_WAIT (tti_unit.wait, tmr_poll));
+sim_activate (&tti_unit, tmr_poll);
 return SCPE_OK;
 }
 
@@ -470,12 +475,15 @@ t_stat clk_reset (DEVICE *dptr)
 {
 if (CPUT (HAS_LTCR))                                    /* reg there? */
     clk_fie = clk_fnxm = 0;
-else clk_fie = clk_fnxm = 1;                            /* no, BEVENT */
+else {
+    clk_fnxm = 1;                                       /* no LTCR, set nxm */
+    clk_fie = CPUO (OPT_BVT);                           /* ie = 1 unless no BEVENT */
+    }
 clk_tps = clk_default;                                  /* set default tps */
 clk_csr = CSR_DONE;                                     /* set done */
 CLR_INT (CLK);
 sim_rtcn_init (clk_unit.wait, TMR_CLK);                 /* init line clock */
-sim_activate_abs (&clk_unit, clk_unit.wait);            /* activate unit */
+sim_activate (&clk_unit, clk_unit.wait);                /* activate unit */
 tmr_poll = clk_unit.wait;                               /* set timer poll */
 tmxr_poll = clk_unit.wait;                              /* set mux poll */
 return SCPE_OK;
