@@ -1,6 +1,6 @@
 /* pdp11_sys.c: PDP-11 simulator interface
 
-   Copyright (c) 1993-2008, Robert M Supnik
+   Copyright (c) 1993-2020, Robert M Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -23,6 +23,11 @@
    used in advertising or otherwise to promote the sale, use or other dealings
    in this Software without prior written authorization from Robert M Supnik.
 
+   17-Feb-20    RMS     Disabled VH11
+   23-May-18    RMS     Changed UC15 simulator name
+   14-Mar-16    RMS     Added UC15 support
+   02-Sep-13    RMS     Added third Massbus, RS03/RS04
+   29-Apr-12    RMS     Fixed compiler warning (Mark Pizzolato)
    19-Nov-08    RMS     Moved I/O support routines to I/O library
    15-May-08    RMS     Added KE11-A, DC11 support
                         Renamed DL11
@@ -30,13 +35,13 @@
    25-Jan-08    RMS     Added RC11, KG11A support from John Dundas
    10-Sep-07    RMS     Cleaned up binary loader
    20-Dec-06    RMS     Added TA11 support
-   12-Nov-06    RMS     Fixed operand order in EIS instructions (found by W.F.J. Mueller)
+   12-Nov-06    RMS     Fixed operand order in EIS instructions (W.F.J. Mueller)
    14-Jul-06    RMS     Reordered device list
    06-Jul-06    RMS     Added multiple KL11/DL11 support
    26-Jun-06    RMS     Added RF11 support
-   17-May-06    RMS     Added CR11/CD11 support (from John Dundas)
+   17-May-06    RMS     Added CR11/CD11 support (John Dundas)
    16-Aug-05    RMS     Fixed C++ declaration and cast problems
-   22-Jul-05    RMS     Fixed missing , in initializer (from Doug Gwyn)
+   22-Jul-05    RMS     Fixed missing , in initializer (Doug Gwyn)
    22-Dec-03    RMS     Added second DEUNA/DELUA support
    18-Oct-03    RMS     Added DECtape off reel message
    06-May-03    RMS     Added support for second DEQNA/DELQA
@@ -91,6 +96,7 @@ extern DEVICE rx_dev;
 extern DEVICE ry_dev;
 extern DEVICE mba_dev[];
 extern DEVICE rp_dev;
+extern DEVICE rs_dev;
 extern DEVICE rq_dev, rqb_dev, rqc_dev, rqd_dev;
 extern DEVICE tm_dev;
 extern DEVICE tq_dev;
@@ -101,6 +107,7 @@ extern DEVICE xq_dev, xqb_dev;
 extern DEVICE xu_dev, xub_dev;
 extern DEVICE ke_dev;
 extern DEVICE kg_dev;
+extern DEVICE uca_dev, ucb_dev;
 extern UNIT cpu_unit;
 extern REG cpu_reg[];
 extern uint16 *M;
@@ -116,7 +123,11 @@ extern int32 saved_PC;
    sim_load             binary loader
 */
 
+#if !defined (UC15)
 char sim_name[] = "PDP-11";
+#else
+char sim_name[] = "UC-15";
+#endif
 
 REG *sim_PC = &cpu_reg[0];
 
@@ -125,8 +136,10 @@ int32 sim_emax = 4;
 DEVICE *sim_devices[] = {
     &cpu_dev,
     &sys_dev,
+#if !defined (UC15)
     &mba_dev[0],
     &mba_dev[1],
+    &mba_dev[2],
     &clk_dev,
     &pclk_dev,
     &ptr_dev,
@@ -140,7 +153,7 @@ DEVICE *sim_devices[] = {
     &dci_dev,
     &dco_dev,
     &dz_dev,
-    &vh_dev,
+//    &vh_dev,
     &rc_dev,
     &rf_dev,
     &rk_dev,
@@ -149,6 +162,7 @@ DEVICE *sim_devices[] = {
     &rx_dev,
     &ry_dev,
     &rp_dev,
+    &rs_dev,
     &rq_dev,
     &rqb_dev,
     &rqc_dev,
@@ -163,8 +177,18 @@ DEVICE *sim_devices[] = {
     &xqb_dev,
     &xu_dev,
     &xub_dev,
-    &ke_dev,
     &kg_dev,
+    &ke_dev,
+#else
+    &clk_dev,
+    &tti_dev,
+    &tto_dev,
+    &cr_dev,
+    &lpt_dev,
+    &rk_dev,
+    &uca_dev,
+    &ucb_dev,
+#endif
     NULL
     };
 
@@ -244,11 +268,9 @@ do {                                                    /* block loop */
         if ((d = getc (fileref)) == EOF)                /* data char */
             return SCPE_FMT;
         csum = csum + d;                                /* add into csum */
-        if (org >= MEMSIZE)                             /* invalid addr? */
+        if (!ADDR_IS_MEM (org))                         /* invalid addr? */
             return SCPE_NXM;
-        M[org >> 1] = (org & 1)?                        /* store data */
-            (M[org >> 1] & 0377) | (d << 8):
-            (M[org >> 1] & 0177400) | d;
+        WrMemB (org, ((uint16) d));
         org = (org + 1) & 0177777;                      /* inc origin */
         }
     if ((d = getc (fileref)) == EOF)                    /* get csum */
@@ -796,7 +818,7 @@ return tptr;
 t_stat get_spec (char *cptr, t_addr addr, int32 n1, int32 *sptr, t_value *dptr,
     int32 cflag, int32 iflag)
 {
-int32 reg, indir, pflag, disp;
+int32 reg, indir, pflag, disp = 0;
 
 indir = 0;                                              /* no indirect */
 pflag = 0;
@@ -992,7 +1014,7 @@ switch (j) {                                            /* case on class */
                 disp = (disp - addr) & 0177777;
             else return SCPE_ARG;
             }
-        if ((disp & 1) || (disp > 0400) && (disp < 0177402))
+        if ((disp & 1) || ((disp > 0400) && (disp < 0177402)))
             return SCPE_ARG;
         val[0] = val[0] | (((disp - 2) >> 1) & 0377);
         break;
@@ -1070,7 +1092,8 @@ switch (j) {                                            /* case on class */
         for (cptr = get_glyph (cptr, gbuf, 0); gbuf[0] != 0;
             cptr = get_glyph (cptr, gbuf, 0)) {
             for (i = 0; (opcode[i] != NULL) &&
-                (strcmp (opcode[i], gbuf) != 0) ; i++) ;
+                (strcmp (opcode[i], gbuf) != 0) ; i++)
+                ;
              if ((((opc_val[i] >> I_V_CL) & I_M_CL) != j) ||
                 (opcode[i] == NULL))
                 return SCPE_ARG;
