@@ -1,6 +1,6 @@
 /* pdp11_ts.c: TS11/TSV05 magnetic tape simulator
 
-   Copyright (c) 1993-2008, Robert M Supnik
+   Copyright (c) 1993-2014, Robert M Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -25,6 +25,11 @@
 
    ts           TS11/TSV05 magtape
 
+   27-Oct-14    RMS     Fixed bug in read forward with byte swap
+   23-Oct-13    RMS     Revised for new boot setup routine
+   19-Mar-12    RMS     Fixed declaration of cpu_opt (Mark Pizzolato)
+   22-May-10    RMS     Fixed t_addr printouts for 64b big-endian systems
+                        (Mark Pizzolato)
    16-Feb-06    RMS     Added tape capacity checking
    31-Oct-05    RMS     Fixed address width for large files
    16-Aug-05    RMS     Fixed C++ declaration and cast problems
@@ -57,7 +62,7 @@
    19-Sep-01    RMS     Fixed bug in bootstrap
    15-Sep-01    RMS     Fixed bug in NXM test
    07-Sep-01    RMS     Revised device disable and interrupt mechanism
-   13-Jul-01    RMS     Fixed bug in space reverse (found by Peter Schorn)
+   13-Jul-01    RMS     Fixed bug in space reverse (Peter Schorn)
 
    Magnetic tapes are represented as a series of variable 8b records
    of the form:
@@ -94,7 +99,7 @@
 #else                                                   /* PDP-11 version */
 #include "pdp11_defs.h"
 #define TS_DIS          DEV_DIS                         /* off by default */
-extern int32 cpu_opt;
+extern uint32 cpu_opt;
 #endif
 
 #include "sim_tape.h"
@@ -265,7 +270,6 @@ extern int32 cpu_opt;
 
 extern int32 int_req[IPL_HLVL];
 extern UNIT cpu_unit;
-extern FILE *sim_deb;
 
 uint8 *tsxb = NULL;                                     /* xfer buffer */
 int32 tssr = 0;                                         /* status register */
@@ -281,7 +285,6 @@ int32 ts_bcmd = 0;                                      /* boot cmd */
 int32 ts_time = 10;                                     /* record latency */
 static uint16 cpy_buf[MAX_PLNT];                        /* copy buffer */
 
-DEVICE ts_dev;
 t_stat ts_rd (int32 *data, int32 PA, int32 access);
 t_stat ts_wr (int32 data, int32 PA, int32 access);
 t_stat ts_svc (UNIT *uptr);
@@ -487,7 +490,7 @@ do {
     fc = (fc - 1) & DMASK;                              /* decr wc */
     if (upd)
         msgrfc = fc;
-    if (st = sim_tape_sprecf (uptr, &tbc))              /* space rec fwd, err? */
+    if ((st = sim_tape_sprecf (uptr, &tbc)))            /* space rec fwd, err? */
         return ts_map_status (st);                      /* map status */
     msgxs0 = msgxs0 | XS0_MOT;                          /* tape has moved */
     } while (fc != 0);
@@ -530,7 +533,7 @@ do {
     fc = (fc - 1) & DMASK;                              /* decr wc */
     if (upd)
         msgrfc = fc;
-    if (st = sim_tape_sprecr (uptr, &tbc))              /* space rec rev, err? */
+    if ((st = sim_tape_sprecr (uptr, &tbc)))            /* space rec rev, err? */
         return ts_map_status (st);                      /* map status */
     msgxs0 = msgxs0 | XS0_MOT;                          /* tape has moved */
     } while (fc != 0);
@@ -580,7 +583,7 @@ msgxs0 = msgxs0 | XS0_MOT;                              /* tape has moved */
 if (cmdhdr & CMD_SWP) {                                 /* swapped? */
     for (i = 0; i < wbc; i++) {                         /* copy buffer */
         wa = tsba ^ 1;                                  /* apply OPP */
-        if (Map_WriteB (tsba, 1, &tsxb[i])) {           /* store byte, nxm? */
+        if (Map_WriteB (wa, 1, &tsxb[i])) {             /* store byte, nxm? */
             tssr = ts_updtssr (tssr | TSSR_NXM);        /* set error */
             return (XTC (XS0_RLS, TC4));
             }
@@ -616,7 +619,7 @@ if (st != MTSE_OK)                                      /* error? */
     return ts_map_status (st);
 if (fc == 0)                                            /* byte count */
     fc = 0200000;
-tsba = (cmdadh << 16) | cmdadl + fc;                    /* buf addr */
+tsba = ((cmdadh << 16) | cmdadl) + fc;                  /* buf addr */
 wbc = (tbc > fc)? fc: tbc;                              /* cap buf size */
 msgxs0 = msgxs0 | XS0_MOT;                              /* tape has moved */
 for (i = wbc; i > 0; i--) {                             /* copy buffer */
@@ -663,7 +666,7 @@ else {
         return TC5;
         }
     }
-if (st = sim_tape_wrrecf (uptr, tsxb, fc))              /* write rec, err? */
+if ((st = sim_tape_wrrecf (uptr, tsxb, fc)))            /* write rec, err? */
     return ts_map_status (st);                          /* return status */
 msgxs0 = msgxs0 | XS0_MOT;                              /* tape has moved */
 msgrfc = 0;
@@ -676,7 +679,7 @@ int32 ts_wtmk (UNIT *uptr)
 {
 t_stat st;
 
-if (st = sim_tape_wrtmk (uptr))                         /* write tmk, err? */
+if ((st = sim_tape_wrtmk (uptr)))                       /* write tmk, err? */
     return ts_map_status (st);                          /* return status */
 msgxs0 = msgxs0 | XS0_MOT;                              /* tape has moved */
 if (sim_tape_eot (&ts_unit))                            /* EOT on write? */
@@ -733,9 +736,12 @@ if (!(cmdhdr & CMD_ACK)) {                              /* no acknowledge? */
     }
 fnc = GET_FNC (cmdhdr);                                 /* get fnc+mode */
 mod = GET_MOD (cmdhdr);
-if (DEBUG_PRS (ts_dev))
-    fprintf (sim_deb, ">>TS: cmd=%s, mod=%o, buf=%o, lnt=%d, pos=%d\n",
-        fnc_name[fnc], mod, cmdadl, cmdlnt, ts_unit.pos);
+if (DEBUG_PRS (ts_dev)) {
+    fprintf (sim_deb, ">>TS: cmd=%s, mod=%o, buf=%o, lnt=%d, pos=",
+        fnc_name[fnc], mod, cmdadl, cmdlnt);
+    fprint_val (sim_deb, ts_unit.pos, 10, T_ADDR_W, PV_LEFT);
+    fprintf (sim_deb, "\n");
+    }
 if ((fnc != FNC_WCHR) && (tssr & TSSR_NBA)) {           /* ~wr chr & nba? */
     ts_endcmd (TC3, 0, 0);                              /* error */
     return SCPE_OK;
@@ -1016,9 +1022,12 @@ tssr = ts_updtssr (tssr | tc | TSSR_SSR | (tc? TSSR_SC: 0));
 if (cmdhdr & CMD_IE)
     SET_INT (TS);
 ts_ownm = 0; ts_ownc = 0;
-if (DEBUG_PRS (ts_dev))
-    fprintf (sim_deb, ">>TS: sta=%o, tc=%o, rfc=%d, pos=%d\n",
-        msgxs0, GET_TC (tssr), msgrfc, ts_unit.pos);
+if (DEBUG_PRS (ts_dev)) {
+    fprintf (sim_deb, ">>TS: sta=%o, tc=%o, rfc=%d, pos=",
+        msgxs0, GET_TC (tssr), msgrfc);
+    fprint_val (sim_deb, ts_unit.pos, 10, T_ADDR_W, PV_LEFT);
+    fprintf (sim_deb, "\n");
+    }
 return;
 }
 
@@ -1141,8 +1150,7 @@ static const uint16 boot_rom[] = {
 
 t_stat ts_boot (int32 unitno, DEVICE *dptr)
 {
-int32 i;
-extern int32 saved_PC;
+size_t i;
 extern uint16 *M;
 
 sim_tape_rewind (&ts_unit);
@@ -1150,7 +1158,7 @@ for (i = 0; i < BOOT_LEN; i++)
     M[(BOOT_START >> 1) + i] = boot_rom[i];
 M[BOOT_CSR0 >> 1] = ts_dib.ba & DMASK;
 M[BOOT_CSR1 >> 1] = (ts_dib.ba & DMASK) + 02;
-saved_PC = BOOT_START;
+cpu_set_boot (BOOT_START);
 return SCPE_OK;
 }
  
