@@ -1,7 +1,7 @@
 /* pdp11_xu.c: DEUNA/DELUA ethernet controller simulator
   ------------------------------------------------------------------------------
 
-   Copyright (c) 2003-2007, David T. Hittner
+   Copyright (c) 2003-2018, David T. Hittner
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -30,7 +30,7 @@
     Digital DELUA Users Guide, Part# EK-DELUA-UG-002
     Digital DEUNA Users Guide, Part# EK-DEUNA-UG-001
   These manuals can be found online at:
-    http://www.spies.com/~aek/pdf/dec/unibus
+    http://www.bitsavers.org/pdf/dec/unibus
 
   Testing performed:
    1) Receives/Transmits single packet under custom RSX driver
@@ -41,7 +41,13 @@
         DECNET - SET HOST in/out, COPY in/out
         TCP/IP - PING in/out; SET HOST/TELNET in/out, COPY/FTP in/out
         Clustering - Successfully clustered with AlphaVMS 8.2
-   4) Runs VAX EVDWA diagnostic tests 1-10; tests 11-19 (M68000/ROM/RAM) fail
+   4) VMS 4.7 on VAX780 summary:
+       (Jan/2011: Win7 x64 host; MS VC++ 2008; SIMH v3.8-2 rc1 base; WinPcap 4.1)
+        LAT    - SET HOST/LAT in (no outbound exists)
+        DECNET - SET HOST in/out, DIR in/out, COPY in/out
+        TCP/IP - no kit available to test
+        Clustering - not tested
+   5) Runs VAX EVDWA diagnostic tests 1-10; tests 11-19 (M68000/ROM/RAM) fail
 
   Known issues:
    1) Most auxiliary commands are not implemented yet.
@@ -50,12 +56,21 @@
       probably need to be converted to Map_ReadW and Map_WriteW calls.
    4) Some jerkiness seen during interactive I/O with remote systems;
       this is probably attributable to changed polling times from when
-	  the poll duration was standardized for idling support.
+      the poll duration was standardized for idling support.
 
   ------------------------------------------------------------------------------
 
   Modification history:
 
+  28-May-18 RMS   Changed to avoid nested comment warnings (Mark Pizzolato)
+  12-Jan-11  DTH  Added SHOW XU FILTERS modifier
+  11-Jan-11  DTH  Corrected SELFTEST command, enabling use by VMS 3.7, VMS 4.7, and Ultrix 1.1
+  09-Dec-10  MP   Added address conflict check during attach.
+  06-Dec-10  MP   Added loopback processing support
+  30-Nov-10  MP   Fixed the fact that no broadcast packets were received by the DEUNA
+  15-Aug-08  MP   Fixed transmitted packets to have the correct source MAC address.
+                  Fixed incorrect address filter setting calling eth_filter().
+  23-Jan-08  MP   Added debugging support to display packet headers and packet data
   18-Jun-07  RMS  Added UNIT_IDLE flag
   03-May-07  DTH  Added missing FC_RMAL command; cleared multicast on write
   29-Oct-06  RMS  Synced poll and clock
@@ -91,6 +106,7 @@ extern FILE *sim_log;
 t_stat xu_rd(int32* data, int32 PA, int32 access);
 t_stat xu_wr(int32  data, int32 PA, int32 access);
 t_stat xu_svc(UNIT * uptr);
+t_stat xu_tmrsvc(UNIT * uptr);
 t_stat xu_reset (DEVICE * dptr);
 t_stat xu_attach (UNIT * uptr, char * cptr);
 t_stat xu_detach (UNIT * uptr);
@@ -112,12 +128,14 @@ void xu_clrint (CTLR* xu);
 void xu_process_receive(CTLR* xu);
 void xu_dump_rxring(CTLR* xu);
 void xu_dump_txring(CTLR* xu);
+t_stat xu_show_filters (FILE* st, UNIT* uptr, int32 val, void* desc);
 
 DIB xua_dib = { IOBA_XU, IOLN_XU, &xu_rd, &xu_wr,
 1, IVCL (XU), VEC_XU, {&xu_int} };
 
 UNIT xua_unit[] = {
- { UDATA (&xu_svc, UNIT_IDLE|UNIT_ATTABLE|UNIT_DISABLE, 0) }      /* receive timer */
+ { UDATA (&xu_svc, UNIT_IDLE|UNIT_ATTABLE|UNIT_DISABLE, 0) },     /* receive timer */
+ { UDATA (&xu_tmrsvc, UNIT_IDLE|UNIT_DIS, 0) }
 };
 
 struct xu_device    xua = {
@@ -129,44 +147,51 @@ struct xu_device    xua = {
 
 MTAB xu_mod[] = {
 #if defined (VM_PDP11)
-	{ MTAB_XTD|MTAB_VDV, 004, "ADDRESS", "ADDRESS",
-		&set_addr, &show_addr, NULL },
+  { MTAB_XTD|MTAB_VDV, 004, "ADDRESS", "ADDRESS",
+    &set_addr, &show_addr, NULL },
+  { MTAB_XTD | MTAB_VDV, 0, NULL, "AUTOCONFIGURE",
+    &set_addr_flt, NULL, NULL },
+  { MTAB_XTD|MTAB_VDV, 0, "VECTOR", NULL,
+    &set_vec, &show_vec, NULL },
 #else
-	{ MTAB_XTD|MTAB_VDV, 004, "ADDRESS", NULL,
-		NULL, &show_addr, NULL },
+  { MTAB_XTD|MTAB_VDV, 004, "ADDRESS", NULL,
+    NULL, &show_addr, NULL },
+  { MTAB_XTD|MTAB_VDV, 0, "VECTOR", NULL,
+    NULL, &show_vec, NULL },
 #endif
-	{ MTAB_XTD|MTAB_VDV, 0, "VECTOR", NULL,
-		NULL, &show_vec, NULL },
   { MTAB_XTD | MTAB_VDV, 0, "MAC", "MAC=xx:xx:xx:xx:xx:xx",
     &xu_setmac, &xu_showmac, NULL },
   { MTAB_XTD | MTAB_VDV | MTAB_NMO, 0, "ETH", "ETH",
     NULL, &eth_show, NULL },
   { MTAB_XTD | MTAB_VDV | MTAB_NMO, 0, "STATS", "STATS",
     &xu_set_stats, &xu_show_stats, NULL },
+  { MTAB_XTD | MTAB_VDV | MTAB_NMO, 0, "FILTERS", "FILTERS",
+    NULL, &xu_show_filters, NULL },
   { MTAB_XTD | MTAB_VDV, 0, "TYPE", "TYPE={DEUNA|DELUA}",
     &xu_set_type, &xu_show_type, NULL },
   { 0 },
 };
 
 REG xua_reg[] = {
-	{ NULL }  };
+    { NULL }  };
 
 DEBTAB xu_debug[] = {
   {"TRACE",  DBG_TRC},
   {"WARN",   DBG_WRN},
   {"REG",    DBG_REG},
   {"PACKET", DBG_PCK},
+  {"DATA",   DBG_DAT},
   {"ETH",    DBG_ETH},
   {0}
 };
 
 
 DEVICE xu_dev = {
-	"XU", xua_unit, xua_reg, xu_mod,
-	1, XU_RDX, 8, 1, XU_RDX, 8,
-	&xu_ex, &xu_dep, &xu_reset,
-	NULL, &xu_attach, &xu_detach,
-	&xua_dib, DEV_FLTA | DEV_DISABLE | DEV_DIS | DEV_UBUS | DEV_DEBUG,
+    "XU", xua_unit, xua_reg, xu_mod,
+    2, XU_RDX, 8, 1, XU_RDX, 8,
+    &xu_ex, &xu_dep, &xu_reset,
+    NULL, &xu_attach, &xu_detach,
+    &xua_dib, DEV_FLTA | DEV_DISABLE | DEV_DIS | DEV_UBUS | DEV_DEBUG,
   0, xu_debug
   };
 
@@ -175,7 +200,7 @@ DEVICE xu_dev = {
 #if defined(IOBA_XUB)
 
 DIB xub_dib = { IOBA_XUB, IOLN_XUB, &xu_rd, &xu_wr,
-		1, IVCL (XU), 0, { &xu_int } };
+        1, IVCL (XU), 0, { &xu_int } };
 
 UNIT xub_unit[] = {
  { UDATA (&xu_svc, UNIT_IDLE|UNIT_ATTABLE|UNIT_DISABLE, 0) }      /* receive timer */
@@ -189,7 +214,7 @@ struct xu_device    xub = {
   };
 
 REG xub_reg[] = {
-	{ NULL }  };
+    { NULL }  };
 
 DEVICE xub_dev = {
   "XUB", xub_unit, xub_reg, xu_mod,
@@ -294,20 +319,39 @@ t_stat xu_set_stats (UNIT* uptr, int32 val, char* cptr, void* desc)
 
 t_stat xu_show_stats (FILE* st, UNIT* uptr, int32 val, void* desc)
 {
-  char* fmt = "  %-24s%d\n";
+  char* fmt = "  %-26s%d\n";
   CTLR* xu = xu_unit2ctlr(uptr);
   struct xu_stats* stats = &xu->var->stats;
 
   fprintf(st, "Ethernet statistics:\n");
-  fprintf(st, fmt, "Seconds since cleared:",  stats->secs); 
-  fprintf(st, fmt, "Recv frames:",            stats->frecv);
-  fprintf(st, fmt, "Recv dbytes:",            stats->rbytes);
-  fprintf(st, fmt, "Xmit frames:",            stats->ftrans);
-  fprintf(st, fmt, "Xmit dbytes:",            stats->tbytes);
-  fprintf(st, fmt, "Recv frames(multicast):", stats->mfrecv);
-  fprintf(st, fmt, "Recv dbytes(multicast):", stats->mrbytes);
-  fprintf(st, fmt, "Xmit frames(multicast):", stats->mftrans);
-  fprintf(st, fmt, "Xmit dbytes(multicast):", stats->mtbytes);
+  fprintf(st, fmt, "Seconds since cleared:",   stats->secs); 
+  fprintf(st, fmt, "Recv frames:",             stats->frecv);
+  fprintf(st, fmt, "Recv dbytes:",             stats->rbytes);
+  fprintf(st, fmt, "Xmit frames:",             stats->ftrans);
+  fprintf(st, fmt, "Xmit dbytes:",             stats->tbytes);
+  fprintf(st, fmt, "Recv frames(multicast):",  stats->mfrecv);
+  fprintf(st, fmt, "Recv dbytes(multicast):",  stats->mrbytes);
+  fprintf(st, fmt, "Xmit frames(multicast):",  stats->mftrans);
+  fprintf(st, fmt, "Xmit dbytes(multicast):",  stats->mtbytes);
+  fprintf(st, fmt, "Loopback forward Frames:", stats->loopf);
+  return SCPE_OK;
+}
+
+t_stat xu_show_filters (FILE* st, UNIT* uptr, int32 val, void* desc)
+{
+  CTLR* xu = xu_unit2ctlr(uptr);
+  char  buffer[20];
+  int i;
+
+  fprintf(st, "Filters:\n");
+  for (i=0; i<XU_FILTER_MAX; i++) {
+    eth_mac_fmt((ETH_MAC*)xu->var->setup.macs[i], buffer);
+    fprintf(st, "  [%2d]: %s\n", i, buffer);
+  }
+  if (xu->var->setup.multicast)
+    fprintf(st, "All Multicast Receive Mode\n");
+  if (xu->var->setup.promiscuous)
+    fprintf(st, "Promiscuous Receive Mode\n");
   return SCPE_OK;
 }
 
@@ -359,13 +403,74 @@ void bit_stat16(uint16* stat, uint16 bits)
   *stat |= bits;
 }
 
+t_stat xu_process_loopback(CTLR* xu, ETH_PACK* pack)
+{
+  ETH_PACK  response;
+  ETH_MAC   physical_address;
+  t_stat    status;
+  int offset   = 16 + (pack->msg[14] | (pack->msg[15] << 8));
+  int function = pack->msg[offset] | (pack->msg[offset+1] << 8);
+
+  sim_debug(DBG_TRC, xu->dev, "xu_process_loopback()\n");
+
+  if (function != 2 /*forward*/)
+    return SCPE_NOFNC;
+
+  /* create forward response packet */
+  memcpy (&response, pack, sizeof(ETH_PACK));
+  memcpy (physical_address, xu->var->setup.macs[0], sizeof(ETH_MAC));
+
+  /* The only packets we should be responding to are ones which 
+     we received due to them being directed to our physical MAC address, 
+     OR the Broadcast address OR to a Multicast address we're listening to 
+     (we may receive others if we're in promiscuous mode, but shouldn't 
+     respond to them) */
+  if ((0 == (pack->msg[0]&1)) &&           /* Multicast or Broadcast */
+      (0 != memcmp(physical_address, pack->msg, sizeof(ETH_MAC))))
+      return SCPE_NOFNC;
+
+
+  memcpy (&response.msg[0], &response.msg[offset+2], sizeof(ETH_MAC));
+  memcpy (&response.msg[6], physical_address, sizeof(ETH_MAC));
+  offset += 8;
+  response.msg[14] = offset & 0xFF;
+  response.msg[15] = (offset >> 8) & 0xFF;
+
+  /* send response packet */
+  status = eth_write(xu->var->etherface, &response, NULL);
+  ++xu->var->stats.loopf;
+
+  if (DBG_PCK & xu->dev->dctrl)
+      eth_packet_trace_ex(xu->var->etherface, response.msg, response.len, ((function == 1) ? "xu-loopbackreply" : "xu-loopbackforward"), DBG_DAT & xu->dev->dctrl, DBG_PCK);
+
+  return status;
+}
+
 t_stat xu_process_local (CTLR* xu, ETH_PACK* pack)
 {
-  return SCPE_NOFNC; /* not implemented yet */
+  /* returns SCPE_OK if local processing occurred,
+     otherwise returns SCPE_NOFNC or some other code */
+  int protocol;
+
+  sim_debug(DBG_TRC, xu->dev, "xu_process_local()\n");
+
+  protocol = pack->msg[12] | (pack->msg[13] << 8);
+  switch (protocol) {
+    case 0x0090:  /* ethernet loopback */
+      return xu_process_loopback(xu, pack);
+      break;
+    case 0x0260:  /* MOP remote console */
+      return SCPE_NOFNC; /* not implemented yet */
+      break;
+  }
+  return SCPE_NOFNC;
 }
 
 void xu_read_callback(CTLR* xu, int status)
 {
+  if (DBG_PCK & xu->dev->dctrl)
+      eth_packet_trace_ex(xu->var->etherface, xu->var->read_buffer.msg, xu->var->read_buffer.len, "xu-recvd", DBG_DAT & xu->dev->dctrl, DBG_PCK);
+
   /* process any packets locally that can be */
   status = xu_process_local (xu, &xu->var->read_buffer);
 
@@ -442,6 +547,8 @@ t_stat xu_system_id (CTLR* xu, const ETH_MAC dest, uint16 receipt_id)
   /* write system id */
   system_id.len = 60;
   status = eth_write(xu->var->etherface, &system_id, NULL);
+  if (DBG_PCK & xu->dev->dctrl)
+    eth_packet_trace_ex(xu->var->etherface, system_id.msg, system_id.len, "xu-systemid", DBG_DAT & xu->dev->dctrl, DBG_PCK);
 
   return status;
 }
@@ -449,10 +556,7 @@ t_stat xu_system_id (CTLR* xu, const ETH_MAC dest, uint16 receipt_id)
 t_stat xu_svc(UNIT* uptr)
 {
   int queue_size;
-  t_stat status;
   CTLR* xu = xu_unit2ctlr(uptr);
-  const ETH_MAC mop_multicast = {0xAB, 0x00, 0x00, 0x02, 0x00, 0x00};
-  const int one_second = clk_tps * tmr_poll;
 
   /* First pump any queued packets into the system */
   if ((xu->var->ReadQ.count > 0) && ((xu->var->pcsr1 & PCSR1_STATE) == STATE_RUNNING))
@@ -464,37 +568,46 @@ t_stat xu_svc(UNIT* uptr)
     {
     queue_size = xu->var->ReadQ.count;
     /* read a packet from the ethernet - processing is via the callback */
-    status = eth_read (xu->var->etherface, &xu->var->read_buffer, xu->var->rcallback);
+    eth_read (xu->var->etherface, &xu->var->read_buffer, xu->var->rcallback);
   } while (queue_size != xu->var->ReadQ.count);
 
   /* Now pump any still queued packets into the system */
   if ((xu->var->ReadQ.count > 0) && ((xu->var->pcsr1 & PCSR1_STATE) == STATE_RUNNING))
     xu_process_receive(xu);
 
-  /* send identity packet when timer expires */
-  if (--xu->var->idtmr <= 0) {
-    if ((xu->var->mode & MODE_DMNT) == 0)           /* if maint msg is not disabled */
-      status = xu_system_id(xu, mop_multicast, 0);  /*   then send ID packet */
-    xu->var->idtmr = XU_ID_TIMER_VAL * one_second;  /* reset timer */
-  }
-
-  /* has one second timer expired? if so, update stats and reset timer */
-  if (++xu->var->sectmr >= XU_SERVICE_INTERVAL) {
-    upd_stat16 (&xu->var->stats.secs, 1);
-    xu->var->sectmr = 0;
-  }
-
   /* resubmit service timer if controller not halted */
   switch (xu->var->pcsr1 & PCSR1_STATE) {
     case STATE_READY:
     case STATE_RUNNING:
-      sim_activate(&xu->unit[0], tmxr_poll);
+      sim_activate(&xu->unit[0], clk_cosched(tmxr_poll));
       break;
   };
 
   return SCPE_OK;
 }
 
+t_stat xu_tmrsvc(UNIT* uptr)
+{
+  CTLR* xu = xu_unit2ctlr(uptr);
+  const ETH_MAC mop_multicast = {0xAB, 0x00, 0x00, 0x02, 0x00, 0x00};
+  const int one_second = clk_tps * tmr_poll;
+
+  /* send identity packet when timer expires */
+  if (--xu->var->idtmr <= 0) {
+    if ((xu->var->mode & MODE_DMNT) == 0)           /* if maint msg is not disabled */
+      xu_system_id(xu, mop_multicast, 0);           /*   then send ID packet */
+    xu->var->idtmr = XU_ID_TIMER_VAL;               /* reset timer */
+  }
+
+  /* update stats */
+  upd_stat16 (&xu->var->stats.secs, 1);
+
+  /* resubmit service timer */
+  sim_activate(uptr, one_second);
+
+  return SCPE_OK;
+}
+
 void xu_write_callback (CTLR* xu, int status)
 {
   xu->var->write_buffer.status = status;
@@ -523,7 +636,7 @@ void xu_setclrint(CTLR* xu, int32 bits)
 
 t_stat xu_sw_reset (CTLR* xu)
 {
-  t_stat status;
+  int i;
 
   sim_debug(DBG_TRC, xu->dev, "xu_sw_reset()\n");
 
@@ -559,15 +672,21 @@ t_stat xu_sw_reset (CTLR* xu)
 
   /* reset ethernet interface */
   memcpy (xu->var->setup.macs[0], xu->var->mac, sizeof(ETH_MAC));
-  xu->var->setup.mac_count = 1;
+  for (i=0; i<6; i++)
+    xu->var->setup.macs[1][i] = 0xff; /* Broadcast Address */
+  xu->var->setup.mac_count = 2;
   if (xu->var->etherface)
-    status = eth_filter (xu->var->etherface, xu->var->setup.mac_count,
-                         &xu->var->mac, xu->var->setup.multicast,
-                         xu->var->setup.promiscuous);
+    eth_filter (xu->var->etherface, xu->var->setup.mac_count,
+                xu->var->setup.macs, xu->var->setup.multicast,
+                xu->var->setup.promiscuous);
 
   /* activate device if not disabled */
   if ((xu->dev->flags & DEV_DIS) == 0) {
     sim_activate_abs(&xu->unit[0], clk_cosched (tmxr_poll));
+
+    /* start service timer */
+    if (xu->var->etherface)
+      sim_activate_abs(&xu->unit[1], tmr_poll * clk_tps);
   }
 
   /* clear load_server address */
@@ -591,7 +710,7 @@ t_stat xu_reset(DEVICE* dptr)
   /* software reset controller */
   xu_sw_reset(xu);
 
-  return SCPE_OK;
+  return auto_config (0, 0);                              /* run autoconfig */
 }
 
 
@@ -601,7 +720,7 @@ int32 xu_command(CTLR* xu)
   uint32 udbb;
   int fnc, mtlen, i, j;
   uint16 value, pltlen;
-  t_stat status, rstatus, wstatus, wstatus2, wstatus3;
+  t_stat rstatus, wstatus, wstatus2, wstatus3;
   struct xu_stats* stats = &xu->var->stats;
   uint16* udb = xu->var->udb;
   uint16* mac_w = (uint16*) xu->var->mac;
@@ -649,19 +768,19 @@ int32 xu_command(CTLR* xu)
     case FC_NOOP:
       break;
 
-    case FC_RDPA:			/* read default physical address */
+    case FC_RDPA:            /* read default physical address */
       wstatus = Map_WriteB(xu->var->pcbb + 2, 6, xu->var->mac);
       if (wstatus)
         return PCSR0_PCEI + 1;
       break;
 
-    case FC_RPA:			/* read current physical address */
+    case FC_RPA:            /* read current physical address */
       wstatus = Map_WriteB(xu->var->pcbb + 2, 6, (uint8*)&xu->var->setup.macs[0]);
       if (wstatus)
         return PCSR0_PCEI + 1;
       break;
 
-    case FC_WPA:			/* write current physical address */
+    case FC_WPA:            /* write current physical address */
       rstatus = Map_ReadB(xu->var->pcbb + 2, 6, (uint8*)&xu->var->setup.macs[0]);
       if (xu->var->pcb[1] & 1)
         return PCSR0_PCEI;
@@ -670,33 +789,33 @@ int32 xu_command(CTLR* xu)
     case FC_RMAL:   /* read multicast address list */
       mtlen = (xu->var->pcb[2] & 0xFF00) >> 8;
       udbb = xu->var->pcb[1] | ((xu->var->pcb[2] & 03) << 16);
-      wstatus = Map_WriteB(udbb, mtlen * 3, (uint8*) &xu->var->setup.macs[1]);
-	  break;
+      wstatus = Map_WriteB(udbb, mtlen * 3, (uint8*) &xu->var->setup.macs[2]);
+      break;
 
     case FC_WMAL:   /* write multicast address list */
       mtlen = (xu->var->pcb[2] & 0xFF00) >> 8;
-sim_debug(DBG_TRC, xu->dev, "FC_WAL: mtlen=%d\n", mtlen);
+      sim_debug(DBG_TRC, xu->dev, "FC_WAL: mtlen=%d\n", mtlen);
       if (mtlen > 10)
         return PCSR0_PCEI;
       udbb = xu->var->pcb[1] | ((xu->var->pcb[2] & 03) << 16);
-	  /* clear existing multicast list */
-	  for (i=1; i<XU_FILTER_MAX; i++) {
-		  for (j=0; j<6; j++)
-			xu->var->setup.macs[i][j] = 0;
-	  }
-	  /* get multicast list from host */
-      rstatus = Map_ReadB(udbb, mtlen * 6, (uint8*) &xu->var->setup.macs[1]);
+      /* clear existing multicast list */
+      for (i=2; i<XU_FILTER_MAX; i++) {
+        for (j=0; j<6; j++)
+          xu->var->setup.macs[i][j] = 0;
+      }
+      /* get multicast list from host */
+      rstatus = Map_ReadB(udbb, mtlen * 6, (uint8*) &xu->var->setup.macs[2]);
       if (rstatus == 0) {
-        xu->var->setup.mac_count = mtlen + 1;
-        status = eth_filter (xu->var->etherface, xu->var->setup.mac_count,
-                             xu->var->setup.macs, xu->var->setup.multicast,
-                             xu->var->setup.promiscuous);
+        xu->var->setup.mac_count = mtlen + 2;
+        eth_filter (xu->var->etherface, xu->var->setup.mac_count,
+                    xu->var->setup.macs, xu->var->setup.multicast,
+                    xu->var->setup.promiscuous);
       } else {
         xu->var->pcsr0 |= PCSR0_PCEI;
       }
       break;
 
-    case FC_RRF:			/* read ring format */
+    case FC_RRF:            /* read ring format */
       if ((xu->var->pcb[1] & 1) || (xu->var->pcb[2] & 0374)) 
         return PCSR0_PCEI;
       xu->var->udb[0] = xu->var->tdrb & 0177776;
@@ -713,7 +832,7 @@ sim_debug(DBG_TRC, xu->dev, "FC_WAL: mtlen=%d\n", mtlen);
         return PCSR0_PCEI+1;
       break;
 
-    case FC_WRF:			/* write ring format */
+    case FC_WRF:            /* write ring format */
       if ((xu->var->pcb[1] & 1) || (xu->var->pcb[2] & 0374)) 
         return PCSR0_PCEI;
       if ((xu->var->pcsr1 & PCSR1_STATE) == STATE_RUNNING)
@@ -797,17 +916,17 @@ sim_debug(DBG_TRC, xu->dev, "FC_WAL: mtlen=%d\n", mtlen);
         memset(stats, 0, sizeof(struct xu_stats));
       break;
 
-    case FC_RMODE:			/* read mode register */
+    case FC_RMODE:            /* read mode register */
       value = xu->var->mode;
       wstatus = Map_WriteW(xu->var->pcbb+2, 2, &value);
       if (wstatus)
         return PCSR0_PCEI + 1;
       break;
 
-    case FC_WMODE:			/* write mode register */
+    case FC_WMODE:            /* write mode register */
       value = xu->var->mode;
       xu->var->mode = xu->var->pcb[1];
-sim_debug(DBG_TRC, xu->dev, "FC_WMODE: mode=%04x\n", xu->var->mode);
+      sim_debug(DBG_TRC, xu->dev, "FC_WMODE: mode=%04x\n", xu->var->mode);
 
       /* set promiscuous and multicast flags */
       xu->var->setup.promiscuous = (xu->var->mode & MODE_PROM) ? 1 : 0;
@@ -815,13 +934,13 @@ sim_debug(DBG_TRC, xu->dev, "FC_WMODE: mode=%04x\n", xu->var->mode);
 
       /* if promiscuous or multicast flags changed, change filter */
       if ((value ^ xu->var->mode) & (MODE_PROM | MODE_ENAL))
-        status = eth_filter (xu->var->etherface, xu->var->setup.mac_count,
-                            &xu->var->mac, xu->var->setup.multicast,
-                             xu->var->setup.promiscuous);
+        eth_filter (xu->var->etherface, xu->var->setup.mac_count,
+                    xu->var->setup.macs, xu->var->setup.multicast,
+                    xu->var->setup.promiscuous);
       break;
 
-    case FC_RSTAT:			/* read extended status */
-    case FC_RCSTAT:			/* read and clear extended status */
+    case FC_RSTAT:            /* read extended status */
+    case FC_RCSTAT:            /* read and clear extended status */
       value = xu->var->stat;
       wstatus = Map_WriteW(xu->var->pcbb+2, 2, &value);
       value = 10;
@@ -832,7 +951,7 @@ sim_debug(DBG_TRC, xu->dev, "FC_WMODE: mode=%04x\n", xu->var->mode);
         return PCSR0_PCEI + 1;
 
       if (fnc == FC_RCSTAT)
-        xu->var->stat &= 0377;	/* clear high byte */
+        xu->var->stat &= 0377;    /* clear high byte */
       break;
 
     case FC_RSID: /* read system id parameters */
@@ -901,7 +1020,7 @@ sim_debug(DBG_TRC, xu->dev, "FC_WMODE: mode=%04x\n", xu->var->mode);
         return PCSR0_PCEI + 1;
       break;
 
-    default:			/* Unknown (unimplemented) command. */
+    default:            /* Unknown (unimplemented) command. */
       printf("%s: unknown ancilliary command 0%o requested !\n", xu->dev->name, fnc);
       return PCSR0_PCEI;
       break;
@@ -923,7 +1042,7 @@ void xu_process_receive(CTLR* xu)
 
   sim_debug(DBG_TRC, xu->dev, "xu_process_receive(), buffers: %d\n", xu->var->rrlen);
 
-/* xu_dump_rxring(xu); /* debug receive ring */
+// xu_dump_rxring(xu); /* debug receive ring */
 
   /* process only when in the running state, and host buffers are available */
   if ((state != STATE_RUNNING) || no_buffers)
@@ -982,7 +1101,7 @@ void xu_process_receive(CTLR* xu)
       if (item->packet.len < ETH_MIN_PACKET) {
         int len = item->packet.len;
         memset (&item->packet.msg[len], 0, ETH_MIN_PACKET - len);
-	      item->packet.len = ETH_MIN_PACKET;
+          item->packet.len = ETH_MIN_PACKET;
       }
     }
 
@@ -1095,7 +1214,7 @@ void xu_process_transmit(CTLR* xu)
   t_stat rstatus, wstatus;
 
   sim_debug(DBG_TRC, xu->dev, "xu_process_transmit()\n");
-/* xu_dump_txring(xu); /* debug receive ring */
+// xu_dump_txring(xu); /* debug receive ring */
 
   for (;;) {
 
@@ -1151,6 +1270,11 @@ void xu_process_transmit(CTLR* xu)
          runt = 1;
       }
 
+      /* As described in the DEUNA User Guide (Section 4.7), the DEUNA is responsible 
+         for inserting the appropriate source MAC address in the outgoing packet header, 
+         so we do that now. */
+      memcpy(xu->var->write_buffer.msg+6, (uint8*)&xu->var->setup.macs[0], sizeof(ETH_MAC));
+
       /* are we in internal loopback mode ? */
       if ((xu->var->mode & MODE_LOOP) && (xu->var->mode & MODE_INTL)) {
         /* just put packet in  receive buffer */
@@ -1160,6 +1284,9 @@ void xu_process_transmit(CTLR* xu)
         wstatus = eth_write(xu->var->etherface, &xu->var->write_buffer, xu->var->wcallback);
         if (wstatus)
           xu->var->pcsr0 |= PCSR0_PCEI;
+        else
+          if (DBG_PCK & xu->dev->dctrl)
+            eth_packet_trace_ex(xu->var->etherface, xu->var->write_buffer.msg, xu->var->write_buffer.len, "xu-write", DBG_DAT & xu->dev->dctrl, DBG_PCK);
       }
 
       /* update transmit status in transmit buffer */
@@ -1224,7 +1351,6 @@ void xu_port_command (CTLR* xu)
   char* msg;
   int command = xu->var->pcsr0 & PCSR0_PCMD;
   int state = xu->var->pcsr1 & PCSR1_STATE;
-  int bits;
   static char* commands[] = {
       "NO-OP",
       "GET PCBB",
@@ -1246,28 +1372,35 @@ void xu_port_command (CTLR* xu)
 
   sim_debug(DBG_TRC, xu->dev, "xu_port_command(), Command = %s [0%o]\n", commands[command], command);
   switch (command) {  /* cases in order of most used to least used */
-    case CMD_PDMD:			/* POLLING DEMAND */
+    case CMD_PDMD:            /* POLLING DEMAND */
       /* process transmit buffers, receive buffers are done in the service timer */
       xu_process_transmit(xu);
       xu->var->pcsr0 |= PCSR0_DNI;
       break;
 
-    case CMD_GETCMD:		/* GET COMMAND */
-      bits = xu_command(xu);
+    case CMD_GETCMD:        /* GET COMMAND */
+      xu_command(xu);
       xu->var->pcsr0 |= PCSR0_DNI;
       break;
 
-    case CMD_GETPCBB:		/* GET PCB-BASE */
+    case CMD_GETPCBB:        /* GET PCB-BASE */
       xu->var->pcbb = (xu->var->pcsr3 << 16) | xu->var->pcsr2;
       xu->var->pcsr0 |= PCSR0_DNI;
       break;
 
-    case CMD_SELFTEST:		/* SELFTEST */
-      xu_sw_reset(xu);
+    case CMD_SELFTEST:        /* SELFTEST */
+      /*
+        SELFTEST is a <=15-second self diagnostic test, setting various
+        error flags and the DONE (DNI) flag when complete. For simulation
+        purposes, signal completion immediately with no errors. This
+        inexact behavior could be incompatible with any guest machine
+        diagnostics that are expecting to be able to monitor the
+        controller's progress through the diagnostic testing.
+      */
       xu->var->pcsr0 |= PCSR0_DNI;
       break;
 
-    case CMD_START:			/* START */
+    case CMD_START:            /* START */
       if (state == STATE_READY) {
         xu->var->pcsr1 &= ~PCSR1_STATE;
         xu->var->pcsr1 |= STATE_RUNNING; 
@@ -1291,7 +1424,7 @@ void xu_port_command (CTLR* xu)
         xu->var->pcsr0 |= PCSR0_PCEI;
       break;
 
-    case CMD_STOP:			/* STOP */
+    case CMD_STOP:            /* STOP */
       if (state == STATE_RUNNING) {
         xu->var->pcsr1 &= ~PCSR1_STATE;
         xu->var->pcsr1 |= STATE_READY;
@@ -1348,7 +1481,7 @@ t_stat xu_rd(int32 *data, int32 PA, int32 access)
       *data = xu->var->pcsr3;
       break;
   }
-  sim_debug(DBG_TRC, xu->dev, "xu_rd(), PCSR%d, data=%04x\n", reg, *data);
+  sim_debug(DBG_REG, xu->dev, "xu_rd(), PCSR%d, data=%04x\n", reg, *data);
   if (PA & 1)
     sim_debug(DBG_WRN, xu->dev, "xu_rd(), Unexpected Odd address access of PCSR%d\n", reg);
   return SCPE_OK;
@@ -1375,10 +1508,10 @@ t_stat xu_wr(int32 data, int32 PA, int32 access)
       strcpy(desc, "Unknown");
       break;
   }
-  sim_debug(DBG_TRC, xu->dev, "xu_wr(), PCSR%d, data=%08x, PA=%08x, access=%d[%s]\n", reg, data, PA, access, desc);
+  sim_debug(DBG_REG, xu->dev, "xu_wr(), PCSR%d, data=%08x, PA=%08x, access=%d[%s]\n", reg, data, PA, access, desc);
   switch (reg) {
     case 00:
-	/* Clear write-one-to-clear interrupt bits */
+    /* Clear write-one-to-clear interrupt bits */
       if (access == WRITEB) {
         data &= 0377;
         if (PA & 1) {
@@ -1425,11 +1558,11 @@ t_stat xu_wr(int32 data, int32 PA, int32 access)
       break;
 
     case 02:
-      xu->var->pcsr2 = data & 0177776;	/* store word, but not MBZ LSB */
+      xu->var->pcsr2 = data & 0177776;    /* store word, but not MBZ LSB */
       break;
 
     case 03:
-      xu->var->pcsr3 = data & 0000003;	/* store significant bits */
+      xu->var->pcsr3 = data & 0000003;    /* store significant bits */
       break;
   }
   return SCPE_OK;
@@ -1449,7 +1582,10 @@ t_stat xu_attach(UNIT* uptr, char* cptr)
   strcpy(tptr, cptr);
 
   xu->var->etherface = (ETH_DEV *) malloc(sizeof(ETH_DEV));
-  if (!xu->var->etherface) return SCPE_MEM;
+  if (!xu->var->etherface) {
+    free(tptr);
+    return SCPE_MEM;
+    }
 
   status = eth_open(xu->var->etherface, cptr, xu->dev, DBG_ETH);
   if (status != SCPE_OK) {
@@ -1458,6 +1594,17 @@ t_stat xu_attach(UNIT* uptr, char* cptr)
     xu->var->etherface = 0;
     return status;
   }
+  if (SCPE_OK != eth_check_address_conflict (xu->var->etherface, &xu->var->mac)) {
+    char buf[32];
+
+    eth_mac_fmt(&xu->var->mac, buf);     /* format ethernet mac address */
+    printf("%s: MAC Address Conflict on LAN for address %s\n", xu->dev->name, buf);
+    if (sim_log) fprintf (sim_log, "%s: MAC Address Conflict on LAN for address %s\n", xu->dev->name, buf);
+    eth_close(xu->var->etherface);
+    free(tptr);
+    xu->var->etherface = 0;
+    return SCPE_NOATT;
+  }
   uptr->filename = tptr;
   uptr->flags |= UNIT_ATT;
   eth_setcrc(xu->var->etherface, 1); /* enable CRC */
@@ -1472,12 +1619,11 @@ t_stat xu_attach(UNIT* uptr, char* cptr)
 
 t_stat xu_detach(UNIT* uptr)
 {
-  t_stat status;
   CTLR* xu = xu_unit2ctlr(uptr);
   sim_debug(DBG_TRC, xu->dev, "xu_detach()\n");
 
   if (uptr->flags & UNIT_ATT) {
-    status = eth_close (xu->var->etherface);
+    eth_close (xu->var->etherface);
     free(xu->var->etherface);
     xu->var->etherface = 0;
     free(uptr->filename);
@@ -1535,7 +1681,7 @@ void xu_dump_rxring (CTLR* xu)
   for (i=0; i<rrlen; i++) {
     uint16 rxhdr[4] = {0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF};
     uint32 ba = xu->var->rdrb + (xu->var->relen * 2) * i;
-    t_stat rstatus = Map_ReadW (ba, 8, rxhdr);	/* get rxring entry[i] */
+    t_stat rstatus = Map_ReadW (ba, 8, rxhdr);    /* get rxring entry[i] */
     int own = (rxhdr[2] & RXR_OWN) >> 15;
     int len = rxhdr[0];
     uint32 addr = rxhdr[1] + ((rxhdr[2] & 3) << 16);
@@ -1551,7 +1697,7 @@ void xu_dump_txring (CTLR* xu)
   for (i=0; i<trlen; i++) {
     uint16 txhdr[4];
     uint32 ba = xu->var->tdrb + (xu->var->telen * 2) * i;
-    t_stat tstatus = Map_ReadW (ba, 8, txhdr);	/* get rxring entry[i] */
+    t_stat tstatus = Map_ReadW (ba, 8, txhdr);    /* get rxring entry[i] */
     int own = (txhdr[2] & RXR_OWN) >> 15;
     int len = txhdr[0];
     uint32 addr = txhdr[1] + ((txhdr[2] & 3) << 16);
