@@ -1,6 +1,6 @@
-/* pdp18b_drm.c: drum/fixed head disk simulator
+/* pdp18b_drm.c: drum head disk simulator
 
-   Copyright (c) 1993-2008, Robert M Supnik
+   Copyright (c) 1993-2016, Robert M Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -23,19 +23,24 @@
    used in advertising or otherwise to promote the sale, use or other dealings
    in this Software without prior written authorization from Robert M Supnik.
 
-   drm          (PDP-4,PDP-7) Type 24 serial drum
+   drm          (PDP-4,PDP-7) Type 24 serial drum; (PDP-9) RM09 drum
 
+   07-Mar-16    RMS     Revised for dynamically allocated memory
+   26-Feb-16    RMS     Added PDP-9 support; set default state to disabled
+   03-Sep-13    RMS     Added explicit void * cast
    14-Jan-04    RMS     Revised IO device call interface
    26-Oct-03    RMS     Cleaned up buffer copy code
    05-Dec-02    RMS     Updated from Type 24 documentation
    22-Nov-02    RMS     Added PDP-4 support
    05-Feb-02    RMS     Added DIB, device number support
-   03-Feb-02    RMS     Fixed bug in reset routine (found by Robert Alan Byer)
+   03-Feb-02    RMS     Fixed bug in reset routine (Robert Alan Byer)
    06-Jan-02    RMS     Revised enable/disable support
    25-Nov-01    RMS     Revised interrupt structure
    10-Jun-01    RMS     Cleaned up IOT decoding to reflect hardware
    26-Apr-01    RMS     Added device enable/disable support
    14-Apr-99    RMS     Changed t_addr to unsigned
+
+   Variable drum sizes are not supported.
 */
 
 #include "pdp18b_defs.h"
@@ -60,7 +65,7 @@
 #define GET_POS(x)      ((int) fmod (sim_gtime() / ((double) (x)), \
                         ((double) DRM_NUMWDT)))
 
-extern int32 M[];
+extern int32 *M;
 extern int32 int_hwre[API_HLVL+1];
 extern UNIT cpu_unit;
 
@@ -117,7 +122,7 @@ DEVICE drm_dev = {
     1, 8, 20, 1, 8, 18,
     NULL, NULL, &drm_reset,
     &drm_boot, NULL, NULL,
-    &drm_dib, DEV_DISABLE
+    &drm_dib, DEV_DISABLE + DEV_DIS
     };
 
 /* IOT routines */
@@ -181,7 +186,7 @@ t_stat drm_svc (UNIT *uptr)
 {
 int32 i;
 uint32 da;
-int32 *fbuf = uptr->filebuf;
+int32 *fbuf = (int32 *) uptr->filebuf;
 
 if ((uptr->flags & UNIT_BUF) == 0) {                    /* not buf? abort */
     drm_err = 1;                                        /* set error */
@@ -240,11 +245,11 @@ static const int32 boot_rom[] = {
     0706101,                        /* DRSF             ; wait for done */
     0602003,                        /* JMP .-1 */
     0600000                         /* JMP 0            ; enter boot */
-	};
+    };
 
 t_stat drm_boot (int32 unitno, DEVICE *dptr)
 {
-int32 i;
+size_t i;
 extern int32 PC;
 
 if (drm_dib.dev != DEV_DRM)                             /* non-std addr? */
