@@ -1,6 +1,6 @@
 /* pdp18b_dt.c: 18b DECtape simulator
 
-   Copyright (c) 1993-2006, Robert M Supnik
+   Copyright (c) 1993-2017, Robert M Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -27,8 +27,14 @@
                 (PDP-9) TC02/TU55 DECtape
                 (PDP-15) TC15/TU56 DECtape
 
-   23-Jun-06	RMS     Fixed switch conflict in ATTACH
-                    	Revised Type 550 header based on DECTOG formatter
+   15-Mar-17    RMS     Fixed dt_seterr to clear successor states
+   09-Mar-17    RMS     Fixed dt_seterr to handle nx unit select (COVERITY)
+   10-Mar-16    RMS     Added 3-cycle databreak set/show entries
+   07-Mar-16    RMS     Revised for dynamically allocated memory
+   13-Mar-15    RMS     Added APIVEC register
+   28-Mar-15    RMS     Revised to use sim_printf
+   23-Jun-06    RMS     Fixed switch conflict in ATTACH
+                        Revised Type 550 header based on DECTOG formatter
    13-Jun-06    RMS     Fixed checksum calculation bug in Type 550
    16-Aug-05    RMS     Fixed C++ declaration and cast problems
    25-Jan-04    RMS     Revised for device debug support
@@ -210,7 +216,7 @@
 #define DTA_RW          (0777700 & ~(DTA_CERF | DTA_CDTF))
 #define DTA_GETUNIT(x)  (((x) >> DTA_V_UNIT) & DTA_M_UNIT)
 #define DT_UPDINT       if ((dtsa & DTA_ENB) && (dtsb & (DTB_ERF | DTB_DTF))) \
-                        SET_INT (DTA); \
+                            SET_INT (DTA); \
                         else CLR_INT (DTA);
 
 #else                                                   /* Type 550 */
@@ -234,7 +240,7 @@
 #define DTA_RW          077
 #define DTA_GETUNIT(x)  map_unit[(((x) >> DTA_V_UNIT) & DTA_M_UNIT)]
 #define DT_UPDINT       if (dtsb & (DTB_DTF | DTB_BEF | DTB_ERF)) \
-                        SET_INT (DTA); \
+                            SET_INT (DTA); \
                         else CLR_INT (DTA);
 #endif
 
@@ -324,12 +330,10 @@
 
 #define ABS(x)          (((x) < 0)? (-(x)): (x))
 
-extern int32 M[];
+extern int32 *M;
 extern int32 int_hwre[API_HLVL+1];
+extern int32 api_vec[API_HLVL][32];
 extern UNIT cpu_unit;
-extern int32 sim_switches;
-extern int32 sim_is_running;
-extern FILE *sim_deb;
 
 int32 dtsa = 0;                                         /* status A */
 int32 dtsb = 0;                                         /* status B */
@@ -404,10 +408,6 @@ REG dt_reg[] = {
     { FLDATA (BEF, dtsb, DTB_V_BEF) },
 #endif
     { FLDATA (ERF, dtsb, DTB_V_ERF) },
-#if defined (TC02)                                      /* TC02/TC15 */
-    { ORDATA (WC, M[DT_WC], 18) },
-    { ORDATA (CA, M[DT_CA], 18) },
-#endif
     { DRDATA (LTIME, dt_ltime, 31), REG_NZ },
     { DRDATA (DCTIME, dt_dctime, 31), REG_NZ },
     { ORDATA (SUBSTATE, dt_substate, 2) },
@@ -420,6 +420,9 @@ REG dt_reg[] = {
               DT_NUMDR, REG_HRO) },
     { ORDATA (DEVNO, dt_dib.dev, 6), REG_HRO },
     { FLDATA (STOP_OFFR, dt_stopoffr, 0) },
+#if defined (TC02)
+    { ORDATA (APIVEC, api_vec[API_DTA][INT_V_DTA], 6), REG_HRO },
+#endif
     { NULL }
     };
 
@@ -429,6 +432,10 @@ MTAB dt_mod[] = {
     { UNIT_8FMT + UNIT_11FMT, 0, "18b", NULL, NULL },
     { UNIT_8FMT + UNIT_11FMT, UNIT_8FMT, "12b", NULL, NULL },
     { UNIT_8FMT + UNIT_11FMT, UNIT_11FMT, "16b", NULL, NULL },
+#if defined (TC02)
+    { MTAB_XTD|MTAB_VDV|MTAB_NMO, DT_WC, "WC", "WC", &set_3cyc_reg, &show_3cyc_reg, "WC" },
+    { MTAB_XTD|MTAB_VDV|MTAB_NMO, DT_CA, "CA", "CA", &set_3cyc_reg, &show_3cyc_reg, "CA" },
+#endif
     { MTAB_XTD|MTAB_VDV, 0, "DEVNO", "DEVNO", &set_devno, &show_devno },
     { 0 }
     };
@@ -459,14 +466,19 @@ int32 old_dtsa = dtsa, fnc;
 UNIT *uptr;
 
 if (((pulse & 060) == 040) && (pulse & 05)) {           /* select */
-    if (pulse & 01) dtsa = 0;                           /* DTCA */
-    if (pulse & 02) dat = dtsa;                         /* DTRA!... */
+    if (pulse & 01)                                     /* DTCA */
+        dtsa = 0;
+    if (pulse & 02)                                     /* DTRA!... */
+        dat = dtsa;
     if (pulse & 04) {                                   /* DTXA */
-        if ((dat & DTA_CERF) == 0) dtsb = dtsb & ~DTB_ALLERR;
-        if ((dat & DTA_CDTF) == 0) dtsb = dtsb & ~DTB_DTF;
+        if ((dat & DTA_CERF) == 0)
+            dtsb = dtsb & ~DTB_ALLERR;
+        if ((dat & DTA_CDTF) == 0)
+            dtsb = dtsb & ~DTB_DTF;
         dtsa = dtsa ^ (dat & DTA_RW);
         }
-    if ((old_dtsa ^ dtsa) & DTA_UNIT) dt_deselect (old_dtsa);
+    if ((old_dtsa ^ dtsa) & DTA_UNIT)
+        dt_deselect (old_dtsa);
     uptr = dt_dev.units + DTA_GETUNIT (dtsa);           /* get unit */
     fnc = DTA_GETFNC (dtsa);                            /* get fnc */
     if (((uptr->flags) & UNIT_DIS) ||                   /* disabled? */
@@ -478,10 +490,12 @@ if (((pulse & 060) == 040) && (pulse & 05)) {           /* select */
     DT_UPDINT;
     return dat;
     }
-if ((pulse & 067) == 042) return dtsa;                  /* DTRA */
+if ((pulse & 067) == 042)                               /* DTRA */
+    return dtsa;
 if ((pulse & 067) == 061)                               /* DTEF */
     return ((dtsb & DTB_ERF)? IOT_SKP + dat: dat);
-if ((pulse & 067) == 062) return dtsb;                  /* DTRB */
+if ((pulse & 067) == 062)                               /* DTRB */
+    return dtsb;
 if ((pulse & 067) == 063)                               /* DTEF!DTRB */
     return ((dtsb & DTB_ERF)? IOT_SKP + dtsb: dtsb);
 return dat;
@@ -519,21 +533,24 @@ int32 fnc, mot, unum;
 UNIT *uptr = NULL;
 
 unum = DTA_GETUNIT (dtsa);                              /* get unit no */
-if (unum >= 0) uptr = dt_dev.units + unum;              /* get unit */
+if (unum >= 0)                                          /* get unit */
+    uptr = dt_dev.units + unum;
 if ((pulse & 001) && (dtsb & DTB_BEF))                  /* MMBF */
     dat = dat | IOT_SKP;
 if (pulse & 002) {                                      /* MMRS */
     dtsb = dtsb & ~(DTB_REV | DTB_GO);                  /* clr rev, go */
     if (uptr) {                                         /* valid unit? */
         mot = DTS_GETMOT (uptr->STATE);                 /* get motion */
-        if (mot & DTS_DIR) dtsb = dtsb | DTB_REV;       /* rev? set */
+        if (mot & DTS_DIR)                              /* rev? set */
+            dtsb = dtsb | DTB_REV;
         if ((mot >= DTS_ACCF) || (uptr->STATE & 0777700))
             dtsb = dtsb | DTB_GO;                       /* accel? go */
         }
     dat = (dat & ~DMASK) | dtsb;
     }
 if ((pulse & 044) == 044) {                             /* MMSE */
-    if ((dtsa ^ dat) & DTA_UNIT) dt_deselect (dtsa);    /* new unit? */
+    if ((dtsa ^ dat) & DTA_UNIT)                        /* new unit? */
+        dt_deselect (dtsa);
     dtsa = (dtsa & ~DTA_UNIT) | (dat & DTA_UNIT);
     dtsb = dtsb & ~(DTB_DTF | DTB_BEF | DTB_ERF | DTB_ALLERR);
     }
@@ -548,7 +565,7 @@ else if ((pulse & 044) == 004) {                        /* MMLC */
         ((fnc == FNC_WALL) && (uptr->flags & UNIT_WLK)))
         dt_seterr (uptr, DTB_SEL);                      /* select err */
     else dt_newsa (dtsa);
-	}
+    }
 DT_UPDINT;
 return dat;
 }
@@ -562,7 +579,8 @@ int32 old_unit, old_mot;
 UNIT *uptr;
 
 old_unit = DTA_GETUNIT (oldf);                          /* get unit no */
-if (old_unit < 0) return;                               /* invalid? */
+if (old_unit < 0)                                       /* invalid? */
+    return;
 uptr = dt_dev.units + old_unit;                         /* get unit */
 old_mot = DTS_GETMOT (uptr->STATE);
 if (old_mot >= DTS_ATSF)                                /* at speed? */
@@ -600,7 +618,8 @@ int32 prev_mving, new_mving, prev_dir, new_dir;
 UNIT *uptr;
 
 new_unit = DTA_GETUNIT (newf);                          /* new unit */
-if (new_unit < 0) return;                               /* invalid? */
+if (new_unit < 0)                                       /* invalid? */
+    return;
 uptr = dt_dev.units + new_unit;
 if ((uptr->flags & UNIT_ATT) == 0) {                    /* new unit attached? */
     dt_seterr (uptr, DTB_SEL);                          /* no, error */
@@ -613,10 +632,12 @@ new_mving = (newf & DTA_STSTP) != 0;                    /* new moving? */
 new_dir = (newf & DTA_FWDRV) != 0;                      /* new dir? */
 new_fnc = DTA_GETFNC (newf);                            /* new function? */
 
-if ((prev_mving | new_mving) == 0) return;              /* stop to stop */
+if ((prev_mving | new_mving) == 0)                      /* stop to stop */
+    return;
 
 if (new_mving & ~prev_mving) {                          /* start? */
-    if (dt_setpos (uptr)) return;                       /* update pos */
+    if (dt_setpos (uptr))                               /* update pos */
+        return;
     sim_cancel (uptr);                                  /* stop current */
     sim_activate (uptr, dt_dctime - (dt_dctime >> 2));  /* sched accel */
     DTS_SETSTA (DTS_ACCF | new_dir, 0);                 /* state = accel */
@@ -626,7 +647,8 @@ if (new_mving & ~prev_mving) {                          /* start? */
 
 if (prev_mving & ~new_mving) {                          /* stop? */
     if ((prev_mot & ~DTS_DIR) != DTS_DECF) {            /* !already stopping? */
-        if (dt_setpos (uptr)) return;                   /* update pos */
+        if (dt_setpos (uptr))                           /* update pos */
+            return;
         sim_cancel (uptr);                              /* stop current */
         sim_activate (uptr, dt_dctime);                 /* schedule decel */
         }
@@ -636,7 +658,8 @@ if (prev_mving & ~new_mving) {                          /* stop? */
 
 if (prev_dir ^ new_dir) {                               /* dir chg? */
     if ((prev_mot & ~DTS_DIR) != DTS_DECF) {            /* !already stopping? */
-        if (dt_setpos (uptr)) return;                   /* update pos */
+        if (dt_setpos (uptr))                           /* update pos */
+            return;
         sim_cancel (uptr);                              /* stop current */
         sim_activate (uptr, dt_dctime);                 /* schedule decel */
         }
@@ -647,7 +670,8 @@ if (prev_dir ^ new_dir) {                               /* dir chg? */
     }
 
 if (prev_mot < DTS_ACCF) {                              /* not accel/at speed? */
-    if (dt_setpos (uptr)) return;                       /* update pos */
+    if (dt_setpos (uptr))                               /* update pos */
+        return;
     sim_cancel (uptr);                                  /* cancel cur */
     sim_activate (uptr, dt_dctime - (dt_dctime >> 2));  /* sched accel */
     DTS_SETSTA (DTS_ACCF | new_dir, 0);                 /* state = accel */
@@ -685,7 +709,8 @@ int32 relpos;
 uint32 oldpos;
 
 oldpos = uptr->pos;                                     /* save old pos */
-if (dt_setpos (uptr)) return;                           /* update pos */
+if (dt_setpos (uptr))                                   /* update pos */
+    return;
 uptr->STATE = newsta;                                   /* update state */
 fnc = DTS_GETFNC (uptr->STATE);                         /* set variables */
 dir = DTS_GETMOT (uptr->STATE) & DTS_DIR;
@@ -703,30 +728,35 @@ dt_substate = DTO_SOB;                                  /* substate = block star
 switch (fnc) {                                          /* case function */
 
     case DTS_OFR:                                       /* off reel */
-        if (dir) newpos = -1000;                        /* rev? < start */
+        if (dir)                                        /* rev? < start */
+            newpos = -1000;
         else newpos = DTU_FWDEZ (uptr) + DT_EZLIN + 1000;       /* fwd? > end */
         break;
 
     case FNC_MOVE:                                      /* move */
         dt_schedez (uptr, dir);                         /* sched end zone */
-        if (DEBUG_PRI (dt_dev, LOG_MS)) fprintf (sim_deb, ">>DT%d: moving %s\n",
-            unum, (dir? "backward": "forward"));
+        if (DEBUG_PRI (dt_dev, LOG_MS))
+            fprintf (sim_deb, ">>DT%d: moving %s\n",
+                     unum, (dir? "backward": "forward"));
         return;                                         /* done */
 
     case FNC_SRCH:                                      /* search */
-        if (dir) newpos = DT_BLK2LN ((DT_QFEZ (uptr)?
+        if (dir)
+            newpos = DT_BLK2LN ((DT_QFEZ (uptr)?
             DTU_TSIZE (uptr): blk), uptr) - DT_BLKLN - DT_WSIZE;
         else newpos = DT_BLK2LN ((DT_QREZ (uptr)?
             0: blk + 1), uptr) + DT_BLKLN + (DT_WSIZE - 1);
-        if (DEBUG_PRI (dt_dev, LOG_MS)) fprintf (sim_deb, ">>DT%d: searching %s\n",
-            unum, (dir? "backward": "forward"));
+        if (DEBUG_PRI (dt_dev, LOG_MS))
+            fprintf (sim_deb, ">>DT%d: searching %s\n", unum,
+                     (dir? "backward": "forward"));
         break;
 
     case FNC_WRIT:                                      /* write */
     case FNC_READ:                                      /* read */
 #if defined (TC02)                                      /* TC02/TC15 */
         if (DT_QEZ (uptr)) {                            /* in "ok" end zone? */
-            if (dir) newpos = DTU_FWDEZ (uptr) - DT_HTLIN - DT_WSIZE;
+            if (dir)
+                newpos = DTU_FWDEZ (uptr) - DT_HTLIN - DT_WSIZE;
             else newpos = DT_EZLIN + DT_HTLIN + (DT_WSIZE - 1);
             break;
             }
@@ -736,8 +766,9 @@ switch (fnc) {                                          /* case function */
             dt_seterr (uptr, DTB_SEL);
             return;
             }
-        if (dir) newpos = DT_BLK2LN (((relpos >= (DTU_LPERB (uptr) - DT_HTLIN))?
-            blk + 1: blk), uptr) - DT_HTLIN - DT_WSIZE;
+        if (dir)
+            newpos = DT_BLK2LN (((relpos >= (DTU_LPERB (uptr) - DT_HTLIN))?
+                blk + 1: blk), uptr) - DT_HTLIN - DT_WSIZE;
         else newpos = DT_BLK2LN (((relpos < DT_HTLIN)?
             blk: blk + 1), uptr) + DT_HTLIN + (DT_WSIZE - 1);
         break;
@@ -746,18 +777,20 @@ switch (fnc) {                                          /* case function */
     case FNC_RALL:                                      /* read all */
     case FNC_WALL:                                      /* write all */
         if (DT_QEZ (uptr)) {                            /* in "ok" end zone? */
-            if (dir) newpos = DTU_FWDEZ (uptr) - DT_WSIZE;
+            if (dir)
+                newpos = DTU_FWDEZ (uptr) - DT_WSIZE;
             else newpos = DT_EZLIN + (DT_WSIZE - 1);
             }
         else {
             newpos = ((uptr->pos) / DT_WSIZE) * DT_WSIZE;
-            if (!dir) newpos = newpos + (DT_WSIZE - 1);
+            if (!dir)
+                newpos = newpos + (DT_WSIZE - 1);
             }
         if (DEBUG_PRI (dt_dev, LOG_RA) ||
            (DEBUG_PRI (dt_dev, LOG_BL) && (blk == dt_logblk)))
             fprintf (sim_deb, ">>DT%d: read all block %d %s%s\n",
-                unum, blk, (dir? "backward": "forward"),
-                ((dtsa & DTA_MODE)? " continuous]": " "));
+                     unum, blk, (dir? "backward": "forward"),
+                    ((dtsa & DTA_MODE)? " continuous]": " "));
         break;
 
     default:
@@ -802,7 +835,8 @@ int32 unum, delta;
 
 new_time = sim_grtime ();                               /* current time */
 ut = new_time - uptr->LASTT;                            /* elapsed time */
-if (ut == 0) return FALSE;                              /* no time gone? exit */
+if (ut == 0)                                            /* no time gone? exit */
+    return FALSE;
 uptr->LASTT = new_time;                                 /* update last time */
 switch (mot & ~DTS_DIR) {                               /* case on motion */
 
@@ -827,17 +861,18 @@ switch (mot & ~DTS_DIR) {                               /* case on motion */
         break;
         }
 
-if (mot & DTS_DIR) uptr->pos = uptr->pos - delta;       /* update pos */
+if (mot & DTS_DIR)                                      /* update pos */
+    uptr->pos = uptr->pos - delta;
 else uptr->pos = uptr->pos + delta;
 if (((int32) uptr->pos < 0) ||
     ((int32) uptr->pos > (DTU_FWDEZ (uptr) + DT_EZLIN))) {
-	detach_unit (uptr);									/* off reel? */
-	uptr->STATE = uptr->pos = 0;
-	unum = (int32) (uptr - dt_dev.units);
-	if (unum == DTA_GETUNIT (dtsa))						/* if selected, */
-		dt_seterr (uptr, DTB_SEL);						/* error */
-	return TRUE;
-	}
+    detach_unit (uptr);                             /* off reel? */
+    uptr->STATE = uptr->pos = 0;
+    unum = (int32) (uptr - dt_dev.units);
+    if (unum == DTA_GETUNIT (dtsa))                     /* if selected, */
+        dt_seterr (uptr, DTB_SEL);                      /* error */
+    return TRUE;
+    }
 return FALSE;
 }
 
@@ -924,7 +959,8 @@ switch (fnc) {                                          /* at speed, check fnc *
         sim_activate (uptr, DTU_LPERB (uptr) * dt_ltime);/* sched next block */
         M[DT_WC] = (M[DT_WC] + 1) & DMASK;              /* inc WC */
         ma = M[DT_CA] & AMASK;                          /* get mem addr */
-        if (MEM_ADDR_OK (ma)) M[ma] = blk;              /* store block # */
+        if (MEM_ADDR_OK (ma))                           /* store block # */
+            M[ma] = blk;
         if (((dtsa & DTA_MODE) == 0) || (M[DT_WC] == 0))
                 dtsb = dtsb | DTB_DTF;                  /* set DTF */
         if (DEBUG_PRI (dt_dev, LOG_MS))
@@ -956,8 +992,8 @@ switch (fnc) {                                          /* at speed, check fnc *
             if (DEBUG_PRI (dt_dev, LOG_RW) ||
                (DEBUG_PRI (dt_dev, LOG_BL) && (blk == dt_logblk)))
                 fprintf (sim_deb, ">>DT%d: reading block %d %s%s\n",
-                    unum, blk, (dir? "backward": "forward"),
-                    ((dtsa & DTA_MODE)? " continuous": " "));
+                         unum, blk, (dir? "backward": "forward"),
+                         ((dtsa & DTA_MODE)? " continuous": " "));
             dt_substate = 0;                            /* fall through */
         case 0:                                         /* normal read */
             M[DT_WC] = (M[DT_WC] + 1) & DMASK;          /* incr WC, CA */
@@ -965,9 +1001,12 @@ switch (fnc) {                                          /* at speed, check fnc *
             ma = M[DT_CA] & AMASK;                      /* mem addr */
             ba = (blk * DTU_BSIZE (uptr)) + wrd;        /* buffer ptr */
             dtdb = fbuf[ba];                            /* get tape word */
-            if (dir) dtdb = dt_comobv (dtdb);           /* rev? comp obv */
-            if (MEM_ADDR_OK (ma)) M[ma] = dtdb;         /* mem addr legal? */
-            if (M[DT_WC] == 0) dt_substate = DTO_WCO;   /* wc ovf? */
+            if (dir)                                    /* rev? comp obv */
+                dtdb = dt_comobv (dtdb);
+            if (MEM_ADDR_OK (ma))                       /* mem addr legal? */
+                M[ma] = dtdb;
+            if (M[DT_WC] == 0)                          /* wc ovf? */
+                dt_substate = DTO_WCO;
         case DTO_WCO:                                   /* wc ovf, not sob */
             if (wrd != (dir? 0: DTU_BSIZE (uptr) - 1))  /* not last? */
                 sim_activate (uptr, DT_WSIZE * dt_ltime);
@@ -1007,12 +1046,12 @@ switch (fnc) {                                          /* at speed, check fnc *
             if (dtsb & DTB_DTF) {                       /* DTF set? */
                 dt_seterr (uptr, DTB_TIM);              /* timing error */
                 return SCPE_OK;
-				}
+                }
             if (DEBUG_PRI (dt_dev, LOG_RW) ||
                (DEBUG_PRI (dt_dev, LOG_BL) && (blk == dt_logblk)))
                 fprintf (sim_deb, ">>DT%d: writing block %d %s%s\n", unum, blk,
-                    (dir? "backward": "forward"),
-                    ((dtsa & DTA_MODE)? " continuous": " "));
+                         (dir? "backward": "forward"),
+                         ((dtsa & DTA_MODE)? " continuous": " "));
             dt_substate = 0;                            /* fall through */
         case 0:                                         /* normal write */
             M[DT_WC] = (M[DT_WC] + 1) & DMASK;          /* incr WC, CA */
@@ -1021,10 +1060,13 @@ switch (fnc) {                                          /* at speed, check fnc *
             ma = M[DT_CA] & AMASK;                      /* mem addr */
             ba = (blk * DTU_BSIZE (uptr)) + wrd;        /* buffer ptr */
             dtdb = dt_substate? 0: M[ma];               /* get word */
-            if (dir) dtdb = dt_comobv (dtdb);           /* rev? comp obv */
+            if (dir)                                    /* rev? comp obv */
+                dtdb = dt_comobv (dtdb);
             fbuf[ba] = dtdb;                            /* write word */
-            if (ba >= uptr->hwmark) uptr->hwmark = ba + 1;
-            if (M[DT_WC] == 0) dt_substate = DTO_WCO;
+            if (ba >= uptr->hwmark)
+                uptr->hwmark = ba + 1;
+            if (M[DT_WC] == 0)
+                dt_substate = DTO_WCO;
             if (wrd != (dir? 0: DTU_BSIZE (uptr) - 1))  /* not last? */
                 sim_activate (uptr, DT_WSIZE * dt_ltime);
             else {
@@ -1066,10 +1108,13 @@ switch (fnc) {                                          /* at speed, check fnc *
                 dtdb = fbuf[ba];                        /* get tape word */
                 }
             else dtdb = dt_gethdr (uptr, blk, relpos);  /* get hdr */
-            if (dir) dtdb = dt_comobv (dtdb);           /* rev? comp obv */
+            if (dir)                                    /* rev? comp obv */
+                dtdb = dt_comobv (dtdb);
             sim_activate (uptr, DT_WSIZE * dt_ltime);
-            if (MEM_ADDR_OK (ma)) M[ma] = dtdb;         /* mem addr legal? */
-            if (M[DT_WC] == 0) dt_substate = DTO_WCO;
+            if (MEM_ADDR_OK (ma))                       /* mem addr legal? */
+                M[ma] = dtdb;
+            if (M[DT_WC] == 0)
+                dt_substate = DTO_WCO;
             if (((dtsa & DTA_MODE) == 0) || (M[DT_WC] == 0))
                 dtsb = dtsb | DTB_DTF;                  /* set DTF */
             break;
@@ -1101,15 +1146,18 @@ switch (fnc) {                                          /* at speed, check fnc *
             if ((relpos >= DT_HTLIN) &&                 /* in data zone? */
                 (relpos < (DTU_LPERB (uptr) - DT_HTLIN))) {
                 dtdb = M[ma];                           /* get mem word */
-                if (dir) dtdb = dt_comobv (dtdb);
+                if (dir)
+                    dtdb = dt_comobv (dtdb);
                 wrd = DT_LIN2WD (uptr->pos, uptr);
                 ba = (blk * DTU_BSIZE (uptr)) + wrd;
                 fbuf[ba] = dtdb;                        /* write word */
-                if (ba >= uptr->hwmark) uptr->hwmark = ba + 1;
+                if (ba >= uptr->hwmark)
+                    uptr->hwmark = ba + 1;
                 }
-/*                                                      /* ignore hdr */
+                                                        /* ignore hdr */
             sim_activate (uptr, DT_WSIZE * dt_ltime);
-            if (M[DT_WC] == 0) dt_substate = DTO_WCO;
+            if (M[DT_WC] == 0)
+                dt_substate = DTO_WCO;
             if (((dtsa & DTA_MODE) == 0) || (M[DT_WC] == 0))
                 dtsb = dtsb | DTB_DTF;                  /* set DTF */
             break;
@@ -1161,13 +1209,15 @@ switch (fnc) {                                          /* at speed, check fnc *
 #endif
             if ((fnc == FNC_READ) &&                    /* read, skip if not */
                 (wrd != DT_CSMWD) &&                    /* fwd, rev cksum */
-                (wrd != ma)) break;
+                (wrd != ma))
+                break;
             dtdb = dt_gethdr (uptr, blk, relpos);
             if (wrd == (dir? DT_CSMWD: ma))             /* at end csum? */
                 dtsb = dtsb | DTB_BEF;                  /* end block */
             else dtsb = dtsb | DTB_DTF;                 /* else next word */
             }
-        if (dir) dtdb = dt_comobv (dtdb);
+        if (dir)
+            dtdb = dt_comobv (dtdb);
         break;
 
 /* Write and write all */
@@ -1183,9 +1233,11 @@ switch (fnc) {                                          /* at speed, check fnc *
             (relpos < (DTU_LPERB (uptr) - DT_HTLIN))) {
             wrd = DT_LIN2WD (uptr->pos, uptr);
             ba = (blk * DTU_BSIZE (uptr)) + wrd;
-            if (dir) fbuf[ba] = dt_comobv (dtdb);       /* get data word */
+            if (dir)                                    /* get data word */
+                fbuf[ba] = dt_comobv (dtdb);
             else fbuf[ba] = dtdb;
-            if (ba >= uptr->hwmark) uptr->hwmark = ba + 1;
+            if (ba >= uptr->hwmark)
+                uptr->hwmark = ba + 1;
             if (wrd == (dir? 0: DTU_BSIZE (uptr) - 1))
                 dtsb = dtsb | DTB_BEF;                  /* end block */
             else dtsb = dtsb | DTB_DTF;                 /* else next word */
@@ -1219,15 +1271,18 @@ return SCPE_OK;
 
 void dt_seterr (UNIT *uptr, int32 e)
 {
-int32 mot = DTS_GETMOT (uptr->STATE);
-
 dtsa = dtsa & ~DTA_STSTP;                               /* clear go */
 dtsb = dtsb | DTB_ERF | e;                              /* set error flag */
-if (mot >= DTS_ACCF) {                                  /* ~stopped or stopping? */
-    sim_cancel (uptr);                                  /* cancel activity */
-    if (dt_setpos (uptr)) return;                       /* update position */
-    sim_activate (uptr, dt_dctime);                     /* sched decel */
-    DTS_SETSTA (DTS_DECF | (mot & DTS_DIR), 0);         /* state = decel */
+if (uptr != NULL) {                                     /* valid select? */
+    int32 mot = DTS_GETMOT (uptr->STATE);               /* get motion */
+    if (mot >= DTS_ACCF) {                              /* ~stopped or stopping? */
+        sim_cancel (uptr);                              /* cancel activity */
+        if (dt_setpos (uptr))                           /* update position */
+            return;
+        sim_activate (uptr, dt_dctime);                 /* sched decel */
+        DTS_SETSTA (DTS_DECF | (mot & DTS_DIR), 0);     /* state = decel */
+        }
+    else DTS_SETSTA (mot, 0);                           /* clear 2nd, 3rd */
     }
 DT_UPDINT;
 return;
@@ -1239,7 +1294,8 @@ void dt_schedez (UNIT *uptr, int32 dir)
 {
 int32 newpos;
 
-if (dir) newpos = DT_EZLIN - DT_WSIZE;                  /* rev? rev ez */
+if (dir)                                                /* rev? rev ez */
+    newpos = DT_EZLIN - DT_WSIZE;
 else newpos = DTU_FWDEZ (uptr) + DT_WSIZE;              /* fwd? fwd ez */
 sim_activate (uptr, ABS (newpos - ((int32) uptr->pos)) * dt_ltime);
 return;
@@ -1276,7 +1332,8 @@ csum = 0777777;
 for (i = 0; i < DTU_BSIZE (uptr); i++) {                /* loop thru buf */
     wrd = fbuf[ba + i];                                 /* get word */
     csum = csum + wrd;                                  /* 1's comp add */
-    if (csum > DMASK) csum = (csum + 1) & DMASK;
+    if (csum > DMASK)
+        csum = (csum + 1) & DMASK;
     }
 return (csum ^ DMASK);                                  /* 1's comp res */
 #endif
@@ -1288,13 +1345,16 @@ int32 dt_gethdr (UNIT *uptr, int32 blk, int32 relpos)
 {
 int32 wrd = relpos / DT_WSIZE;
 
-if (wrd == DT_BLKWD) return blk;                        /* fwd blknum */
+if (wrd == DT_BLKWD)                                    /* fwd blknum */
+    return blk;
 #if defined (TC02)                                      /* TC02/TC15 */
-if (wrd == DT_CSMWD) return 077;                        /* rev csum */
+if (wrd == DT_CSMWD)                                    /* rev csum */
+    return 077;
 if (wrd == ((2 * DT_HTWRD) + DTU_BSIZE (uptr) - DT_CSMWD - 1))  /* fwd csum */
     return (dt_csum (uptr, blk) << 12);
 #else                                                   /* Type 550 */
-if (wrd == DT_CSMWD) return 0777777;                    /* rev csum */
+if (wrd == DT_CSMWD)                                    /* rev csum */
+    return 0777777;
 if (wrd == ((2 * DT_HTWRD) + DTU_BSIZE (uptr) - DT_CSMWD - 1))  /* fwd csum */
     return (dt_csum (uptr, blk));
 #endif
@@ -1315,7 +1375,8 @@ for (i = 0; i < DT_NUMDR; i++) {                        /* stop all drives */
     if (sim_is_running) {                               /* CAF? */
         prev_mot = DTS_GETMOT (uptr->STATE);            /* get motion */
         if ((prev_mot & ~DTS_DIR) > DTS_DECF) {         /* accel or spd? */
-            if (dt_setpos (uptr)) continue;             /* update pos */
+            if (dt_setpos (uptr))                       /* update pos */
+                continue;
             sim_cancel (uptr);
             sim_activate (uptr, dt_dctime);             /* sched decel */
             DTS_SETSTA (DTS_DECF | (prev_mot & DTS_DIR), 0);
@@ -1361,7 +1422,8 @@ int32 u = uptr - dt_dev.units;
 t_stat r;
 
 r = attach_unit (uptr, cptr);                           /* attach */
-if (r != SCPE_OK) return r;                             /* error? */
+if (r != SCPE_OK)                                       /* error? */
+    return r;
 if ((sim_switches & SIM_SW_REST) == 0) {                /* not from rest? */
     uptr->flags = uptr->flags & ~(UNIT_8FMT | UNIT_11FMT);      /* default 18b */
     if (sim_switches & SWMASK ('T'))                    /* att 12b? */
@@ -1383,16 +1445,20 @@ if (uptr->filebuf == NULL) {                            /* can't alloc? */
     return SCPE_MEM;
     }
 fbuf = (uint32 *) uptr->filebuf;                        /* file buffer */
-printf ("%s%d: ", sim_dname (&dt_dev), u);
-if (uptr->flags & UNIT_8FMT) printf ("12b format");
-else if (uptr->flags & UNIT_11FMT) printf ("16b format");
-else printf ("18b/36b format");
-printf (", buffering file in memory\n");
+sim_printf ("%s%d: ", sim_dname (&dt_dev), u);
+if (uptr->flags & UNIT_8FMT)
+    sim_printf ("12b format");
+else if (uptr->flags & UNIT_11FMT)
+    sim_printf ("16b format");
+else sim_printf ("18b/36b format");
+sim_printf (", buffering file in memory\n");
 if (uptr->flags & UNIT_8FMT) {                          /* 12b? */
     for (ba = 0; ba < uptr->capac; ) {                  /* loop thru file */
         k = fxread (pdp8b, sizeof (uint16), D8_NBSIZE, uptr->fileref);
-        if (k == 0) break;
-        for ( ; k < D8_NBSIZE; k++) pdp8b[k] = 0;
+        if (k == 0)
+            break;
+        for ( ; k < D8_NBSIZE; k++)
+            pdp8b[k] = 0;
         for (k = 0; k < D8_NBSIZE; k = k + 3) {         /* loop thru blk */
             fbuf[ba] = ((uint32) (pdp8b[k] & 07777) << 6) |
                 ((uint32) (pdp8b[k + 1] >> 6) & 077);
@@ -1402,12 +1468,14 @@ if (uptr->flags & UNIT_8FMT) {                          /* 12b? */
             }
         }                                               /* end file loop */
     uptr->hwmark = ba;
-	}													/* end if */
+    }                                                    /* end if */
 else if (uptr->flags & UNIT_11FMT) {                    /* 16b? */
     for (ba = 0; ba < uptr->capac; ) {                  /* loop thru file */
         k = fxread (pdp11b, sizeof (uint16), D18_BSIZE, uptr->fileref);
-        if (k == 0) break;
-        for ( ; k < D18_BSIZE; k++) pdp11b[k] = 0;
+        if (k == 0)
+            break;
+        for ( ; k < D18_BSIZE; k++)
+            pdp11b[k] = 0;
         for (k = 0; k < D18_BSIZE; k++)
             fbuf[ba++] = pdp11b[k];
             }
@@ -1437,18 +1505,19 @@ uint16 pdp11b[D18_BSIZE];
 uint32 ba, k, *fbuf;
 int32 u = uptr - dt_dev.units;
 
-if (!(uptr->flags & UNIT_ATT)) return SCPE_OK;
+if (!(uptr->flags & UNIT_ATT))
+    return SCPE_OK;
 if (sim_is_active (uptr)) {
     sim_cancel (uptr);
     if ((u == DTA_GETUNIT (dtsa)) && (dtsa & DTA_STSTP)) {
         dtsb = dtsb | DTB_ERF | DTB_SEL | DTB_DTF;
         DT_UPDINT;
-		}
+        }
     uptr->STATE = uptr->pos = 0;
     }
 fbuf = (uint32 *) uptr->filebuf;                        /* file buffer */
 if (uptr->hwmark && ((uptr->flags & UNIT_RO) == 0)) {   /* any data? */
-    printf ("%s%d: writing buffer to file\n", sim_dname (&dt_dev), u);
+    sim_printf ("%s%d: writing buffer to file\n", sim_dname (&dt_dev), u);
     rewind (uptr->fileref);                             /* start of file */
     if (uptr->flags & UNIT_8FMT) {                      /* 12b? */
         for (ba = 0; ba < uptr->hwmark; ) {             /* loop thru file */
@@ -1458,9 +1527,10 @@ if (uptr->hwmark && ((uptr->flags & UNIT_RO) == 0)) {   /* any data? */
                     ((fbuf[ba + 1] >> 12) & 077);
                 pdp8b[k + 2] = fbuf[ba + 1] & 07777;
                 ba = ba + 2;
-				}			                         /* end loop blk */
+                }                                     /* end loop blk */
             fxwrite (pdp8b, sizeof (uint16), D8_NBSIZE, uptr->fileref);
-            if (ferror (uptr->fileref)) break;
+            if (ferror (uptr->fileref))
+                break;
             }                                           /* end loop file */
         }                                               /* end if 12b */
     else if (uptr->flags & UNIT_11FMT) {                /* 16b? */
@@ -1468,12 +1538,14 @@ if (uptr->hwmark && ((uptr->flags & UNIT_RO) == 0)) {   /* any data? */
             for (k = 0; k < D18_BSIZE; k++)             /* loop blk */
                 pdp11b[k] = fbuf[ba++] & 0177777;
             fxwrite (pdp11b, sizeof (uint16), D18_BSIZE, uptr->fileref);
-            if (ferror (uptr->fileref)) break;
+            if (ferror (uptr->fileref))
+                break;
             }                                           /* end loop file */
         }                                               /* end if 16b */
     else fxwrite (uptr->filebuf, sizeof (uint32),       /* write file */
         uptr->hwmark, uptr->fileref);
-    if (ferror (uptr->fileref)) perror ("I/O error");
+    if (ferror (uptr->fileref))
+        perror ("I/O error");
     }                                                   /* end if hwmark */
 free (uptr->filebuf);                                   /* release buf */
 uptr->flags = uptr->flags & ~UNIT_BUF;                  /* clear buf flag */
