@@ -1,6 +1,6 @@
 /* pdp18b_lp.c: 18b PDP's line printer simulator
 
-   Copyright (c) 1993-2008, Robert M Supnik
+   Copyright (c) 1993-2017, Robert M Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -28,6 +28,10 @@
    lp09         (PDP-9,15) LP09 line printer
    lp15         (PDP-15)   LP15 line printer
 
+   13-Mar-17    RMS     Annotated fall throughs in switch
+   10-Mar-16    RMS     Added 3-cycle databreak set/show entry
+   07-Mar-16    RMS     Revised for dynamically allocated memory
+   13-Sep-15    RMS     Added APIVEC register
    19-Jan-07    RMS     Added UNIT_TEXT flag
    11-Jun-06    RMS     Made character translation table global scope
    14-Jan-04    RMS     Revised IO device call interface
@@ -36,7 +40,7 @@
    05-Feb-03    RMS     Added LP09, fixed conditionalization
    05-Oct-02    RMS     Added DIB, device number support
    30-May-02    RMS     Widened POS to 32b
-   03-Feb-02    RMS     Fixed typo (found by Robert Alan Byer)
+   03-Feb-02    RMS     Fixed typo (Robert Alan Byer)
    25-Nov-01    RMS     Revised interrupt structure
    19-Sep-01    RMS     Fixed bug in 647
    13-Feb-01    RMS     Revised for register arrays
@@ -47,7 +51,10 @@
 */
 
 #include "pdp18b_defs.h"
+
 extern int32 int_hwre[API_HLVL+1];
+extern int32 api_vec[API_HLVL][32];
+
 const char fio_to_asc[64] = {
     ' ','1','2','3','4','5','6','7','8','9','\'','~','#','V','^','<',
     '0','/','S','T','U','V','W','X','Y','Z','"',',','>','^','-','?',
@@ -352,13 +359,13 @@ if (pulse & 004) {                                      /* LPDI */
             lp647_buf[lp647_bp] = lp647_buf[lp647_bp] | ((dat >> 12) & 077);
             lp647_bp = lp647_bp + 1;
             }
-
+                                                        /* fall through */
     case 020:                                           /* LPB2 */
         if (lp647_bp < LP647_BSIZE) {
             lp647_buf[lp647_bp] = lp647_buf[lp647_bp] | ((dat >> 6) & 077);
             lp647_bp = lp647_bp + 1;
             }
-
+                                                        /* fall through */
     case 060:                                           /* LPB1 */
         if (lp647_bp < LP647_BSIZE) {
             lp647_buf[lp647_bp] = lp647_buf[lp647_bp] | (dat & 077);
@@ -534,6 +541,7 @@ REG lp09_reg[] = {
     { DRDATA (TIME, lp09_unit.wait, 24), PV_LEFT },
     { FLDATA (STOP_IOE, lp09_stopioe, 0) },
     { ORDATA (DEVNO, lp09_dib.dev, 6), REG_HRO },
+    { ORDATA (APIVEC, api_vec[API_LPT][INT_V_LPT], 6), REG_HRO },
     { NULL }
     };
 
@@ -661,7 +669,6 @@ return detach_unit (uptr);
 /* LP15 line printer */
 
 #define LP15_BSIZE      132                             /* line size */
-#define LPT_WC          034                             /* word count */
 #define LPT_CA          035                             /* current addr */
 
 /* Status register */
@@ -676,7 +683,7 @@ return detach_unit (uptr);
 #define STA_EFLGS       (STA_ALM | STA_OVF | STA_IHT | STA_ILK)
 #define STA_CLR         0003777                         /* always clear */
 
-extern int32 M[];
+extern int32 *M;
 int32 lp15_sta = 0;
 int32 lp15_ie = 1;
 int32 lp15_stopioe = 0;
@@ -709,7 +716,6 @@ UNIT lp15_unit = {
 
 REG lp15_reg[] = {
     { ORDATA (STA, lp15_sta, 18) },
-    { ORDATA (CA, M[LPT_CA], 18) },
     { FLDATA (INT, int_hwre[API_LPT], INT_V_LPT) },
     { FLDATA (ENABLE, lp15_ie, 0) },
     { DRDATA (LCNT, lp15_lc, 9) },
@@ -720,10 +726,12 @@ REG lp15_reg[] = {
     { FLDATA (STOP_IOE, lp15_stopioe, 0) },
     { BRDATA (LBUF, lp15_buf, 8, 8, LP15_BSIZE) },
     { ORDATA (DEVNO, lp15_dib.dev, 6), REG_HRO },
+    { ORDATA (APIVEC, api_vec[API_LPT][INT_V_LPT], 6), REG_HRO },
     { NULL }
     };
 
 MTAB lp15_mod[] = {
+    { MTAB_XTD|MTAB_VDV|MTAB_NMO, LPT_CA, "CA", "CA", &set_3cyc_reg, &show_3cyc_reg, "CA" },
     { MTAB_XTD|MTAB_VDV, 0, "DEVNO", "DEVNO", &set_devno, &show_devno },
     { 0 }
     };
