@@ -1,6 +1,6 @@
 /* pdp18b_rf.c: fixed head disk simulator
 
-   Copyright (c) 1993-2008, Robert M Supnik
+   Copyright (c) 1993-2016, Robert M Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -26,8 +26,12 @@
    rf           (PDP-9) RF09/RF09
                 (PDP-15) RF15/RS09
 
+   10-Mar-16    RMS     Added 3-cycle databreak set/show entries
+   07-Mar-16    RMS     Revised for dynamically allocated memory
+   13-Sep-15    RMS     Added APIVEC register
+   03-Sep-13    RMS     Added explicit void * cast
    04-Oct-06    RMS     Fixed bug, DSCD does not clear function register
-   15-May-06    RMS     Fixed bug in autosize attach (reported by David Gesswein)
+   15-May-06    RMS     Fixed bug in autosize attach (David Gesswein)
    14-Jan-04    RMS     Revised IO device call interface
                         Changed sim_fsize calling sequence
    26-Oct-03    RMS     Cleaned up buffer copy code
@@ -100,15 +104,16 @@
 
 #define RFS_CLR         0000170                         /* always clear */
 #define RFS_EFLGS       (RFS_HDW | RFS_APE | RFS_MXF | RFS_WCE | \
-						 RFS_DPE | RFS_WLO | RFS_NED )  /* error flags */
+                         RFS_DPE | RFS_WLO | RFS_NED )  /* error flags */
 #define RFS_FR          (RFS_FNC|RFS_IE)
 #define GET_FNC(x)      (((x) >> RFS_V_FNC) & RFS_M_FNC)
 #define GET_POS(x)      ((int) fmod (sim_gtime () / ((double) (x)), \
-						((double) RF_NUMWD)))
+                        ((double) RF_NUMWD)))
 #define RF_BUSY         (sim_is_active (&rf_unit))
 
-extern int32 M[];
+extern int32 *M;
 extern int32 int_hwre[API_HLVL+1];
+extern int32 api_vec[API_HLVL][32];
 extern UNIT cpu_unit;
 
 int32 rf_sta = 0;                                       /* status register */
@@ -125,7 +130,7 @@ int32 rf72 (int32 dev, int32 pulse, int32 dat);
 int32 rf_iors (void);
 t_stat rf_svc (UNIT *uptr);
 t_stat rf_reset (DEVICE *dptr);
-int32 rf_updsta (int32 new);
+int32 rf_updsta (int32 newst);
 t_stat rf_attach (UNIT *uptr, char *cptr);
 t_stat rf_set_size (UNIT *uptr, int32 val, char *cptr, void *desc);
 
@@ -146,8 +151,6 @@ UNIT rf_unit = {
 REG rf_reg[] = {
     { ORDATA (STA, rf_sta, 18) },
     { ORDATA (DA, rf_da, 22) },
-    { ORDATA (WC, M[RF_WC], 18) },
-    { ORDATA (CA, M[RF_CA], 18) },
     { ORDATA (BUF, rf_dbuf, 18) },
     { FLDATA (INT, int_hwre[API_RF], INT_V_RF) },
     { BRDATA (WLK, rf_wlk, 8, 16, RF_NUMDK) },
@@ -156,6 +159,7 @@ REG rf_reg[] = {
     { FLDATA (STOP_IOE, rf_stopioe, 0) },
     { DRDATA (CAPAC, rf_unit.capac, 31), PV_LEFT + REG_HRO },
     { ORDATA (DEVNO, rf_dib.dev, 6), REG_HRO },
+    { ORDATA (APIVEC, api_vec[API_RF][INT_V_RF], 6), REG_HRO },
     { NULL }
     };
 
@@ -169,6 +173,8 @@ MTAB rf_mod[] = {
     { UNIT_PLAT, (6 << UNIT_V_PLAT), NULL, "7P", &rf_set_size },
     { UNIT_PLAT, (7 << UNIT_V_PLAT), NULL, "8P", &rf_set_size },
     { UNIT_AUTO, UNIT_AUTO, "autosize", "AUTOSIZE", NULL },
+    { MTAB_XTD|MTAB_VDV|MTAB_NMO, RF_WC, "WC", "WC", &set_3cyc_reg, &show_3cyc_reg, "WC" },
+    { MTAB_XTD|MTAB_VDV|MTAB_NMO, RF_CA, "CA", "CA", &set_3cyc_reg, &show_3cyc_reg, "CA" },
     { MTAB_XTD|MTAB_VDV, 0, "DEVNO", "DEVNO", &set_devno, &show_devno },
     { 0 }
     };
@@ -265,7 +271,7 @@ return dat;
 t_stat rf_svc (UNIT *uptr)
 {
 int32 f, pa, d, t;
-int32 *fbuf = uptr->filebuf;
+int32 *fbuf = (int32 *) uptr->filebuf;
 
 if ((uptr->flags & UNIT_BUF) == 0) {                    /* not buf? abort */
     rf_updsta (RFS_NED | RFS_DON);                      /* set nxd, done */
@@ -294,7 +300,7 @@ do {
             break;
             }
         else {                                          /* not locked */
-            fbuf[rf_da] = M[pa];						/* write word */
+            fbuf[rf_da] = M[pa];                        /* write word */
             if (((uint32) rf_da) >= uptr->hwmark)
                 uptr->hwmark = rf_da + 1;
             }
@@ -310,9 +316,9 @@ return SCPE_OK;
 
 /* Update status */
 
-int32 rf_updsta (int32 new)
+int32 rf_updsta (int32 newst)
 {
-rf_sta = (rf_sta | new) & ~(RFS_ERR | RFS_CLR);
+rf_sta = (rf_sta | newst) & ~(RFS_ERR | RFS_CLR);
 if (rf_sta & RFS_EFLGS)
     rf_sta = rf_sta | RFS_ERR;
 if ((rf_sta & (RFS_ERR | RFS_DON)) && (rf_sta & RFS_IE))
