@@ -1,6 +1,6 @@
 /* pdp8_rx.c: RX8E/RX01, RX28/RX02 floppy disk simulator
 
-   Copyright (c) 1993-2008, Robert M Supnik
+   Copyright (c) 1993-2013, Robert M Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -25,9 +25,11 @@
 
    rx           RX8E/RX01, RX28/RX02 floppy disk
 
-   15-May-06    RMS     Fixed bug in autosize attach (reported by Dave Gesswein)
+   17-Sep-13    RMS     Changed to use central set_bootpc routine
+   03-Sep-13    RMS     Added explicit void * cast
+   15-May-06    RMS     Fixed bug in autosize attach (Dave Gesswein)
    04-Jan-04    RMS     Changed sim_fsize calling sequence
-   05-Nov-03    RMS     Fixed bug in RX28 read status (found by Charles Dickman)
+   05-Nov-03    RMS     Fixed bug in RX28 read status (Charles Dickman)
    26-Oct-03    RMS     Cleaned up buffer copy code, fixed double density write
    25-Apr-03    RMS     Revised for extended file support
    14-Mar-03    RMS     Fixed variable size interaction with save/restore
@@ -366,7 +368,7 @@ return;
 t_stat rx_svc (UNIT *uptr)
 {
 int32 i, func, byptr, bps, wps;
-int8 *fbuf = uptr->filebuf;
+int8 *fbuf = (int8 *) uptr->filebuf;
 uint32 da;
 #define PTR12(x) (((x) + (x) + (x)) >> 1)
 
@@ -733,8 +735,7 @@ static const uint16 boot2_rom[] = {
 
 t_stat rx_boot (int32 unitno, DEVICE *dptr)
 {
-int32 i;
-extern int32 saved_PC;
+size_t i;
 extern uint16 M[];
 
 if (rx_dib.dev != DEV_RX)                               /* only std devno */
@@ -742,13 +743,13 @@ if (rx_dib.dev != DEV_RX)                               /* only std devno */
 if (rx_28) {
     for (i = 0; i < BOOT2_LEN; i++)
         M[BOOT2_START + i] = boot2_rom[i];
-    saved_PC = BOOT2_ENTRY;
+    cpu_set_bootpc (BOOT2_ENTRY);
     }
 else {
     for (i = 0; i < BOOT_LEN; i++)
         M[BOOT_START + i] = boot_rom[i];
     M[BOOT_INST] = unitno? 07024: 07004;
-    saved_PC = BOOT_ENTRY;
+    cpu_set_bootpc (BOOT_ENTRY);
     }
 return SCPE_OK;
 }
