@@ -1,6 +1,6 @@
 /* vax_syscm.c: PDP-11 compatibility mode symbolic decode and parse
 
-   Copyright (c) 1993-2008, Robert M Supnik
+   Copyright (c) 1993-2012, Robert M Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -23,8 +23,11 @@
    used in advertising or otherwise to promote the sale, use or other dealings
    in this Software without prior written authorization from Robert M Supnik.
 
-   12-Nov-06    RMS     Fixed operand order in EIS instructions (found by W.F.J. Mueller)
-   27-Sep-05	RMS     Fixed warnings compiling with 64b addresses
+   29-Apr-12    RMS     Fixed compiler warning (Mark Pizzolato)
+   22-May-10    RMS     Fixed t_addr printouts for 64b big-endian systems
+                        (Mark Pizzolato)
+   12-Nov-06    RMS     Fixed operand order in EIS instructions (W.F.J. Mueller)
+   27-Sep-05    RMS     Fixed warnings compiling with 64b addresses
    15-Sep-04    RMS     Cloned from pdp11_sys.c
 */
 
@@ -242,13 +245,13 @@ switch (mode) {
     case 6:
         if (reg != 7)
             fprintf (of, "%-X(%s)", nval, rname[reg]);
-        else fprintf (of, "%-X", (nval + addr + 4) & 0177777);
+        else fprintf (of, "%-X", (int32)((nval + addr + 4) & 0177777));
         break;
 
     case 7:
         if (reg != 7)
             fprintf (of, "@%-X(%s)", nval, rname[reg]);
-        else fprintf (of, "@%-X", (nval + addr + 4) & 0177777);
+        else fprintf (of, "@%-X", (int32)((nval + addr + 4) & 0177777));
         break;
         }                                               /* end case */
 
@@ -326,7 +329,7 @@ for (i = 0; opc_val[i] >= 0; i++) {                     /* loop thru ops */
             case I_V_BR:                                /* cond branch */
                 fprintf (of, "%s ", opcode[i]);
                 brdisp = (l8b + l8b + ((l8b & 0200)? 0177002: 2)) & 0177777;
-                fprintf (of, "%-X", (addr + brdisp) & 0177777);
+                fprintf (of, "%-X", (int32)((addr + brdisp) & 0177777));
                 break;
 
             case I_V_8B:                                /* 8b */
@@ -336,7 +339,7 @@ for (i = 0; opc_val[i] >= 0; i++) {                     /* loop thru ops */
             case I_V_SOB:                               /* sob */
                 fprintf (of, "%s %s,", opcode[i], rname[srcr]);
                 brdisp = (dstm * 2) - 2;
-                fprintf (of, "%-X", (addr - brdisp) & 0177777);
+                fprintf (of, "%-X", (int32)((addr - brdisp) & 0177777));
                 break;
 
             case I_V_RSOP:                              /* rsop */
@@ -461,7 +464,7 @@ return tptr;
 
 t_stat get_spec (char *cptr, int32 addr, int32 n1, int32 *sptr, int32 *dptr)
 {
-int32 reg, indir, pflag, disp;
+int32 reg, indir, pflag, disp = 0;
 
 indir = 0;                                              /* no indirect */
 pflag = 0;
@@ -608,7 +611,7 @@ switch (j) {                                            /* case on class */
             return SCPE_ARG;
         if ((pflag & A_REL) == 0)
             disp = (disp - ad32) & 0177777;
-        if ((disp & 1) || (disp > 0400) && (disp < 0177402))
+        if ((disp & 1) || ((disp > 0400) && (disp < 0177402)))
             return SCPE_ARG;
         val[0] = val[0] | (((disp - 2) >> 1) & 0377);
         break;
@@ -683,6 +686,6 @@ if (*cptr != 0) return SCPE_ARG;                        /* junk at end? */
 for (i = j = 0; i < 3; i++, j = j + 2) {
     bytes[j] = val[i] & BMASK;
     bytes[j + 1] = (val[i] >> 8) & BMASK;
-	}
+    }
 return ((2 * (n1 + n2)) - 1);
 }
