@@ -1,6 +1,6 @@
-/* vax_sys.c: VAX simulator interface
+/* vax_syslist.c: VAX device list
 
-   Copyright (c) 1998-2008, Robert M Supnik
+   Copyright (c) 1998-2020, Robert M Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -23,8 +23,9 @@
    used in advertising or otherwise to promote the sale, use or other dealings
    in this Software without prior written authorization from Robert M Supnik.
 
-   17-Oct-06    RMS     Re-ordered device list
-   17-May-06    RMS     Added CR11/CD11 support (from John Dundas)
+   17-Feb-2020  RMS     Disabled VH11
+   17-Oct-2006  RMS     Re-ordered device list
+   17-May-2006  RMS     Added CR11/CD11 support (John Dundas)
    01-Oct-2004  RMS     Cloned from vax_sys.c
 */
 
@@ -44,7 +45,6 @@ extern DEVICE lpt_dev;
 extern DEVICE clk_dev;
 extern DEVICE rq_dev, rqb_dev, rqc_dev, rqd_dev;
 extern DEVICE rl_dev;
-extern DEVICE ry_dev;
 extern DEVICE ts_dev;
 extern DEVICE tq_dev;
 extern DEVICE dz_dev;
@@ -52,7 +52,6 @@ extern DEVICE csi_dev, cso_dev;
 extern DEVICE xq_dev, xqb_dev;
 extern DEVICE vh_dev;
 
-extern int32 sim_switches;
 extern void WriteB (uint32 pa, int32 val);
 extern void rom_wr_B (int32 pa, int32 val);
 extern UNIT cpu_unit;
@@ -70,7 +69,7 @@ DEVICE *sim_devices[] = {
     &csi_dev,
     &cso_dev,
     &dz_dev,
-    &vh_dev,
+//    &vh_dev,
     &cr_dev,
     &lpt_dev,
     &rl_dev,
@@ -78,7 +77,6 @@ DEVICE *sim_devices[] = {
     &rqb_dev,
     &rqc_dev,
     &rqd_dev,
-    &ry_dev,
     &ts_dev,
     &tq_dev,
     &xq_dev,
