@@ -1,6 +1,6 @@
 /* sim_tmxr.c: Telnet terminal multiplexor library
 
-   Copyright (c) 2001-2008, Robert M Supnik
+   Copyright (c) 2001-2019, Robert M Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -26,6 +26,16 @@
    Based on the original DZ11 simulator by Thord Nilson, as updated by
    Arthur Krewat.
 
+   19-Dec-19    JDB     Added tmxr_is_extended global hook
+   19-Mar-19    JDB     Added tmxr_read, tmxr_write, tmxr_show, tmxr_close
+                        global hooks and associated local hook routines;
+                        added tmxr_init_line, tmxr_report_connection,
+                        and tmxr_disconnect_line global routines
+   06-Mar-18    RMS     Revised for new IP address format in sim_sock
+   08-Jul-17    JDB     Corrected misleading indentation in tmxr_poll_tx
+   06-Aug-15    JDB     [4.0] Added modem control functions
+   28-Mar-15    RMS     Revised to use sim_printf
+   16-Jan-11    MP      Made option negotiation more reliable
    20-Nov-08    RMS     Added three new standardized SHOW routines
    30-Sep-08    JDB     Reverted tmxr_find_ldsc to original implementation
    27-May-08    JDB     Added line connection order to tmxr_poll_conn,
@@ -59,6 +69,8 @@
    tmxr_poll_rx -       poll receive
    tmxr_putc_ln -       put character for line
    tmxr_poll_tx -       poll transmit
+   tmxr_set_modem_control_passthru -    enable modem control on a multiplexer
+   tmxr_set_get_modem_bits -            set and/or get a line modem bits
    tmxr_open_master -   open master connection
    tmxr_close_master -  close master connection
    tmxr_attach  -       attach terminal multiplexor
@@ -74,6 +86,9 @@
    tmxr_tqln    -       number of buffered characters for line
    tmxr_set_lnorder -   set line connection order
    tmxr_show_lnorder -  show line connection order
+   tmxr_init_line -     initialize the line data
+   tmxr_report_connection - report a line connection to the port
+   tmxr_disconnect_line - disconnect a line
 
    All routines are OS-independent.
 */
@@ -86,12 +101,26 @@
 
 /* Telnet protocol constants - negatives are for init'ing signed char data */
 
+/* Commands */
 #define TN_IAC          -1                              /* protocol delim */
 #define TN_DONT         -2                              /* dont */
 #define TN_DO           -3                              /* do */
 #define TN_WONT         -4                              /* wont */
 #define TN_WILL         -5                              /* will */
+#define TN_SB           -6                              /* sub-option negotiation */
+#define TN_GA           -7                              /* go ahead */
+#define TN_EL           -8                              /* erase line */
+#define TN_EC           -9                              /* erase character */
+#define TN_AYT          -10                             /* are you there */
+#define TN_AO           -11                             /* abort output */
+#define TN_IP           -12                             /* interrupt process */
 #define TN_BRK          -13                             /* break */
+#define TN_DATAMK       -14                             /* data mark */
+#define TN_NOP          -15                             /* no operation */
+#define TN_SE           -16                             /* end sub-option negot */
+
+/* Options */
+
 #define TN_BIN          0                               /* bin */
 #define TN_ECHO         1                               /* echo */
 #define TN_SGA          3                               /* sga */
@@ -108,16 +137,24 @@
 #define TNS_WONT        003                             /* WONT seen */
 #define TNS_SKIP        004                             /* skip next cmd */
 #define TNS_CRPAD       005                             /* CR padding */
+#define TNS_DO          006                             /* DO request pending rejection */
 
 void tmxr_rmvrc (TMLN *lp, int32 p);
-int32 tmxr_send_buffered_data (TMLN *lp);
-TMLN *tmxr_find_ldsc (UNIT *uptr, int32 val, TMXR *mp);
 
-extern int32 sim_switches;
 extern char sim_name[];
-extern FILE *sim_log;
 extern uint32 sim_os_msec (void);
 
+static int32 tmxr_local_read  (TMLN *lp, int32 length);
+static int32 tmxr_local_write (TMLN *lp, int32 length);
+static void  tmxr_local_show  (TMLN *lp, FILE *stream);
+static void  tmxr_local_close (TMLN *lp);
+
+int32  (*tmxr_read)        (TMLN *lp, int32 length) = tmxr_local_read;
+int32  (*tmxr_write)       (TMLN *lp, int32 length) = tmxr_local_write;
+void   (*tmxr_show)        (TMLN *lp, FILE *stream) = tmxr_local_show;
+void   (*tmxr_close)       (TMLN *lp)               = tmxr_local_close;
+t_bool (*tmxr_is_extended) (TMLN *lp)               = NULL;
+
 /* Poll for new connection
 
    Called from unit service routine to test for new connection
@@ -138,7 +175,8 @@ SOCKET newsock;
 TMLN *lp;
 int32 *op;
 int32 i, j;
-uint32 ipaddr;
+char *ipaddr = NULL;
+
 static char mantra[] = {
     TN_IAC, TN_WILL, TN_LINE,
     TN_IAC, TN_WILL, TN_SGA,
@@ -165,40 +203,16 @@ if (newsock != INVALID_SOCKET) {                        /* got a live one? */
 
     if (i >= mp->lines) {                               /* all busy? */
         tmxr_msg (newsock, "All connections busy\r\n");
-        sim_close_sock (newsock, 0);
+        sim_close_sock (newsock);
         }
     else {
         lp = mp->ldsc + i;                              /* get line desc */
         lp->conn = newsock;                             /* record connection */
         lp->ipad = ipaddr;                              /* ip address */
         lp->cnms = sim_os_msec ();                      /* time of conn */
-        lp->rxbpr = lp->rxbpi = 0;                      /* init buf pointers */
-        lp->txbpr = lp->txbpi = 0;
-        lp->rxcnt = lp->txcnt = 0;                      /* init counters */
-        lp->tsta = 0;                                   /* init telnet state */
-        lp->xmte = 1;                                   /* enable transmit */
-        lp->dstb = 0;                                   /* default bin mode */
-        sim_write_sock (newsock, mantra, 15);
-        tmxr_linemsg (lp, "\n\r\nConnected to the ");
-        tmxr_linemsg (lp, sim_name);
-        tmxr_linemsg (lp, " simulator ");
-
-        if (mp->dptr) {                                 /* device defined? */
-            tmxr_linemsg (lp, sim_dname (mp->dptr));    /* report device name */
-            tmxr_linemsg (lp, " device");
-
-            if (mp->lines > 1) {                        /* more than one line? */
-                char line[20];
-
-                tmxr_linemsg (lp, ", line ");           /* report the line number */
-                sprintf (line, "%i", i);
-                tmxr_linemsg (lp, line);
-                }
-            }
-
-        tmxr_linemsg (lp, "\r\n\n");
-
-        tmxr_poll_tx (mp);                              /* flush output */
+        tmxr_init_line (lp);                            /* initialize the line */
+        sim_write_sock (newsock, mantra, sizeof (mantra));
+        tmxr_report_connection (mp, lp, i);             /* report the connection */
         return i;
         }
     }                                                   /* end if newsock */
@@ -212,15 +226,91 @@ void tmxr_reset_ln (TMLN *lp)
 if (lp->txlog)                                          /* dump log */
     fflush (lp->txlog);
 tmxr_send_buffered_data (lp);                           /* send buffered data */
-sim_close_sock (lp->conn, 0);                           /* reset conn */
-lp->conn = lp->tsta = 0;                                /* reset state */
-lp->rxbpr = lp->rxbpi = 0;
-lp->txbpr = lp->txbpi = 0;
-lp->xmte = 1;
-lp->dstb = 0;
+free (lp->ipad);
+lp->ipad = NULL;
+tmxr_close (lp);                                        /* reset the connection */
+tmxr_init_line (lp);                                    /* initialize the line */
+lp->conn = 0;                                           /*   and clear the connection */
 return;
 }
 
+/* Enable modem control pass thru
+
+   Inputs:
+        none
+
+   Output:
+        none
+
+   Implementation note:
+
+    1  Calling this API disables any actions on the part of this
+       library to directly manipulate DTR (&RTS) on serial ports.
+
+    2  Calling this API enables the tmxr_set_get_modem_bits and
+       tmxr_set_config_line APIs.
+
+*/
+static t_stat tmxr_clear_modem_control_passthru_state (TMXR *mp, t_bool state)
+{
+if (mp->master)
+    return SCPE_ALATT;
+else
+    return SCPE_OK;
+}
+
+t_stat tmxr_set_modem_control_passthru (TMXR *mp)
+{
+return tmxr_clear_modem_control_passthru_state (mp, TRUE);
+}
+
+/* Manipulate the modem control bits of a specific line
+
+   Inputs:
+        *lp     =       pointer to terminal line descriptor
+        bits_to_set     TMXR_MDM_DTR and/or TMXR_MDM_RTS as desired
+        bits_to_clear   TMXR_MDM_DTR and/or TMXR_MDM_RTS as desired
+
+   Output:
+        incoming_bits   if non NULL, returns the current stat of DCD,
+                        RNG, CTS and DSR along with the current state
+                        of DTR and RTS
+
+   Implementation note:
+
+       If a line is connected to a serial port, then these values affect
+       and reflect the state of the serial port.  If the line is connected
+       to a network socket (or could be) then the network session state is
+       set, cleared and/or returned.
+*/
+t_stat tmxr_set_get_modem_bits (TMLN *lp, int32 bits_to_set, int32 bits_to_clear, int32 *incoming_bits)
+{
+int32 incoming_state;
+
+if ((bits_to_set & ~(TMXR_MDM_OUTGOING)) ||         /* Assure only settable bits */
+    (bits_to_clear & ~(TMXR_MDM_OUTGOING)) ||
+    (bits_to_set & bits_to_clear))                  /* and can't set and clear the same bits */
+    return SCPE_ARG;
+
+if (lp->conn)
+    incoming_state = TMXR_MDM_DCD | TMXR_MDM_DSR;
+else
+    incoming_state = 0;
+
+if (incoming_bits)
+    *incoming_bits = incoming_state;
+
+if (lp->conn && (bits_to_clear & TMXR_MDM_DTR)) {           /* drop DTR? */
+    tmxr_linemsg (lp, "\r\nDisconnected from the ");
+    tmxr_linemsg (lp, sim_name);
+    tmxr_linemsg (lp, " simulator\r\n\n");
+
+    tmxr_reset_ln (lp);
+    }
+
+return SCPE_OK;
+}
+
 /* Get character from specific line
 
    Inputs:
@@ -239,8 +329,10 @@ if (lp->conn && lp->rcve) {                             /* conn & enb? */
     if (j) {                                            /* any? */
         tmp = lp->rxb[lp->rxbpr];                       /* get char */
         val = TMXR_VALID | (tmp & 0377);                /* valid + chr */
-        if (lp->rbr[lp->rxbpr])                         /* break? */
+        if (lp->rbr[lp->rxbpr]) {                       /* break? */
+            lp->rbr[lp->rxbpr] = 0;                     /* clear status */
             val = val | SCPE_BREAK;
+            }
         lp->rxbpr = lp->rxbpr + 1;                      /* adv pointer */
         }
     }                                                   /* end if conn */
@@ -268,21 +360,24 @@ for (i = 0; i < mp->lines; i++) {                       /* loop thru lines */
 
     nbytes = 0;
     if (lp->rxbpi == 0)                                 /* need input? */
-        nbytes = sim_read_sock (lp->conn,               /* yes, read */
-            &(lp->rxb[lp->rxbpi]),                      /* leave spc for */
-            TMXR_MAXBUF - TMXR_GUARD);                  /* Telnet cruft */
+        nbytes = tmxr_read (lp,                         /* yes, read */
+            TMXR_MAXBUF - TMXR_GUARD);                  /* leave spc for Telnet cruft */
     else if (lp->tsta)                                  /* in Telnet seq? */
-        nbytes = sim_read_sock (lp->conn,               /* yes, read to end */
-            &(lp->rxb[lp->rxbpi]),
+        nbytes = tmxr_read (lp,                         /* yes, read to end */
             TMXR_MAXBUF - lp->rxbpi);
     if (nbytes < 0)                                     /* closed? reset ln */
         tmxr_reset_ln (lp);
     else if (nbytes > 0) {                              /* if data rcvd */
         j = lp->rxbpi;                                  /* start of data */
-        memset (&lp->rbr[j], 0, nbytes);                /* clear status */
         lp->rxbpi = lp->rxbpi + nbytes;                 /* adv pointers */
         lp->rxcnt = lp->rxcnt + nbytes;
 
+        if (tmxr_is_extended != NULL                    /* if the line */
+          && tmxr_is_extended (lp) == TRUE)             /*   is extended */
+            continue;                                   /*     then skip the Telnet processing */
+
+        memset (&lp->rbr[j], 0, nbytes);                /* clear status */
+
 /* Examine new data, remove TELNET cruft before making input available */
 
         for (; j < lp->rxbpi; ) {                       /* loop thru char */
@@ -301,7 +396,7 @@ for (i = 0; i < mp->lines; i++) {                       /* loop thru lines */
                 break;
 
             case TNS_IAC:                               /* IAC prev */
-                if ((tmp == TN_IAC) & !lp->dstb) {      /* IAC + IAC, bin? */
+                if (tmp == TN_IAC) {                    /* IAC + IAC */
                     lp->tsta = TNS_NORM;                /* treat as normal */
                     j = j + 1;                          /* advance j */
                     break;                              /* keep IAC */
@@ -313,11 +408,31 @@ for (i = 0; i < mp->lines; i++) {                       /* loop thru lines */
                     j = j + 1;                          /* advance j */
                     break;
                     }
-                if (tmp == TN_WILL)                     /* IAC + WILL? */
+                switch (tmp) {
+                case TN_WILL:                           /* IAC + WILL? */
                     lp->tsta = TNS_WILL;
-                else if (tmp == TN_WONT)                /* IAC + WONT? */
+                    break;
+                case TN_WONT:                           /* IAC + WONT? */
                     lp->tsta = TNS_WONT;
-                else lp->tsta = TNS_SKIP;               /* IAC + other */
+                    break;
+                case TN_DO:                             /* IAC + DO? */
+                    lp->tsta = TNS_DO;
+                    break;
+                case TN_DONT:                           /* IAC + DONT? */
+                    lp->tsta = TNS_SKIP;                /* IAC + other */
+                    break;
+                case TN_GA: case TN_EL:                 /* IAC + other 2 byte types */
+                case TN_EC: case TN_AYT:                
+                case TN_AO: case TN_IP:
+                case TN_NOP: 
+                    lp->tsta = TNS_NORM;                /* ignore */
+                    break;
+                case TN_SB:                             /* IAC + SB sub-opt negotiation */
+                case TN_DATAMK:                         /* IAC + data mark */
+                case TN_SE:                             /* IAC + SE sub-opt end */
+                    lp->tsta = TNS_NORM;                /* ignore */
+                    break;
+                    }
                 tmxr_rmvrc (lp, j);                     /* remove char */
                 break;
 
@@ -327,6 +442,9 @@ for (i = 0; i < mp->lines; i++) {                       /* loop thru lines */
                         lp->dstb = 0;
                     else lp->dstb = 1;
                     }
+                tmxr_rmvrc (lp, j);                     /* remove it */
+                lp->tsta = TNS_NORM;                    /* next normal */
+                break;
 
             /* Negotiation with the HP terminal emulator "QCTerm" is not working.
                QCTerm says "WONT BIN" but sends bare CRs.  RFC 854 says:
@@ -349,9 +467,18 @@ for (i = 0; i < mp->lines; i++) {                       /* loop thru lines */
                     tmxr_rmvrc (lp, j);                 /* remove it */
                 break;
 
-            case TNS_SKIP: default:                     /* skip char */
+            case TNS_DO:                                /* pending DO request */
+                if (tmp == TN_BIN) {                    /* reject all but binary mode */
+                    char accept[] = {TN_IAC, TN_WILL, TN_BIN};
+                    sim_write_sock (lp->conn, accept, sizeof(accept));
+                    }
+                tmxr_rmvrc (lp, j);                     /* remove it */
                 lp->tsta = TNS_NORM;                    /* next normal */
+                break;
+
+            case TNS_SKIP: default:                     /* skip char */
                 tmxr_rmvrc (lp, j);                     /* remove char */
+                lp->tsta = TNS_NORM;                    /* next normal */
                 break;
                 }                                       /* end case state */
             }                                           /* end for char */
@@ -380,6 +507,7 @@ for ( ; p < lp->rxbpi; p++) {
     lp->rxb[p] = lp->rxb[p + 1];
     lp->rbr[p] = lp->rbr[p + 1];
     }
+lp->rbr[p] = 0;                                         /* clear potential break from vacated slot */
 lp->rxbpi = lp->rxbpi - 1;
 return;
 }
@@ -435,9 +563,9 @@ for (i = 0; i < mp->lines; i++) {                       /* loop thru lines */
     lp = mp->ldsc + i;                                  /* get line desc */
     if (lp->conn == 0)                                  /* skip if !conn */
         continue;
-        nbytes = tmxr_send_buffered_data (lp);          /* buffered bytes */
-        if (nbytes == 0)                                /* buf empty? enab line */
-            lp->xmte = 1;
+    nbytes = tmxr_send_buffered_data (lp);              /* buffered bytes */
+    if (nbytes == 0)                                    /* buf empty? enab line */
+        lp->xmte = 1;
         }                                               /* end for */
 return;
 }
@@ -457,11 +585,11 @@ int32 nbytes, sbytes;
 nbytes = tmxr_tqln(lp);                                 /* avail bytes */
 if (nbytes) {                                           /* >0? write */
     if (lp->txbpr < lp->txbpi)                          /* no wrap? */
-        sbytes = sim_write_sock (lp->conn,              /* write all data */
-            &(lp->txb[lp->txbpr]), nbytes);
-    else sbytes = sim_write_sock (lp->conn,             /* write to end buf */
-            &(lp->txb[lp->txbpr]), TMXR_MAXBUF - lp->txbpr);
-    if (sbytes != SOCKET_ERROR) {                       /* ok? */
+        sbytes = tmxr_write (lp, nbytes);               /* write all data */
+    else
+        sbytes = tmxr_write (lp, TMXR_MAXBUF - lp->txbpr);  /* write to end buf */
+
+    if (sbytes > 0) {                                   /* ok? */
         lp->txbpr = (lp->txbpr + sbytes);               /* update remove ptr */
         if (lp->txbpr >= TMXR_MAXBUF)                   /* wrap? */
             lp->txbpr = 0;
@@ -469,8 +597,8 @@ if (nbytes) {                                           /* >0? write */
         nbytes = nbytes - sbytes;
         }
     if (nbytes && (lp->txbpr == 0))     {               /* more data and wrap? */
-        sbytes = sim_write_sock (lp->conn, lp->txb, nbytes);
-        if (sbytes != SOCKET_ERROR) {                   /* ok */
+        sbytes = tmxr_write (lp, nbytes);
+        if (sbytes > 0) {                               /* ok */
             lp->txbpr = (lp->txbpr + sbytes);           /* update remove ptr */
             if (lp->txbpr >= TMXR_MAXBUF)               /* wrap? */
                 lp->txbpr = 0;
@@ -501,22 +629,20 @@ t_stat r;
 port = (int32) get_uint (cptr, 10, 65535, &r);          /* get port */
 if ((r != SCPE_OK) || (port == 0))
     return SCPE_ARG;
-sock = sim_master_sock (port);                          /* make master socket */
+sock = sim_master_sock (cptr, &r);                      /* make master socket */
 if (sock == INVALID_SOCKET)                             /* open error */
     return SCPE_OPENERR;
-printf ("Listening on port %d (socket %d)\n", port, sock);
-if (sim_log)
-    fprintf (sim_log, "Listening on port %d (socket %d)\n", port, sock);
+sim_printf ("Listening on port %d (socket %d)\n", port, sock);
 mp->port = port;                                        /* save port */
 mp->master = sock;                                      /* save master socket */
 for (i = 0; i < mp->lines; i++) {                       /* initialize lines */
     lp = mp->ldsc + i;
-    lp->conn = lp->tsta = 0;
-    lp->rxbpi = lp->rxbpr = 0;
-    lp->txbpi = lp->txbpr = 0;
-    lp->rxcnt = lp->txcnt = 0;
-    lp->xmte = 1;
-    lp->dstb = 0;
+
+    if (tmxr_is_extended == NULL                        /* if the line  */
+      || tmxr_is_extended (lp) == FALSE) {              /*   is not extended */
+        tmxr_init_line (lp);                            /*     then initialize the line */
+        lp->conn = 0;                                   /*       and clear the connection */
+        }
     }
 return SCPE_OK;
 }
@@ -555,14 +681,13 @@ TMLN *lp;
 
 for (i = 0; i < mp->lines; i++) {                       /* loop thru conn */
     lp = mp->ldsc + i;
-    if (lp->conn) {
-        tmxr_linemsg (lp, "\r\nDisconnected from the ");
-        tmxr_linemsg (lp, sim_name);
-        tmxr_linemsg (lp, " simulator\r\n\n");
-        tmxr_reset_ln (lp);
-        }                                               /* end if conn */
+
+    if (lp->conn                                        /* if the line is connected */
+      && (tmxr_is_extended == NULL                      /*   and the line  */
+      || tmxr_is_extended (lp) == FALSE))               /*     is not extended */
+        tmxr_disconnect_line (lp);                      /*       then disconnect it */
     }                                                   /* end for */
-sim_close_sock (mp->master, 1);                         /* close master socket */
+sim_close_sock (mp->master);                            /* close master socket */
 mp->master = 0;
 return SCPE_OK;
 }
@@ -594,14 +719,14 @@ return SCPE_NOFNC;
 
 /* Output message to socket or line descriptor */
 
-void tmxr_msg (SOCKET sock, char *msg)
+void tmxr_msg (SOCKET sock, const char *msg)
 {
 if (sock)
     sim_write_sock (sock, msg, strlen (msg));
 return;
 }
 
-void tmxr_linemsg (TMLN *lp, char *msg)
+void tmxr_linemsg (TMLN *lp, const char *msg)
 {
 int32 len;
 
@@ -617,18 +742,14 @@ void tmxr_fconns (FILE *st, TMLN *lp, int32 ln)
 if (ln >= 0)
     fprintf (st, "line %d: ", ln);
 if (lp->conn) {
-    int32 o1, o2, o3, o4, hr, mn, sc;
+    int32 hr, mn, sc;
     uint32 ctime;
 
-    o1 = (lp->ipad >> 24) & 0xFF;
-    o2 = (lp->ipad >> 16) & 0xFF;
-    o3 = (lp->ipad >> 8) & 0xFF;
-    o4 = (lp->ipad) & 0xFF;
     ctime = (sim_os_msec () - lp->cnms) / 1000;
     hr = ctime / 3600;
     mn = (ctime / 60) % 60;
     sc = ctime % 60;
-    fprintf (st, "IP address %d.%d.%d.%d", o1, o2, o3, o4);
+    tmxr_show (lp, st);                                 /* display the port connection */
     if (ctime)
         fprintf (st, ", connected %02d:%02d:%02d\n", hr, mn, sc);
     }
@@ -996,3 +1117,166 @@ fprintf (st, "lines=%d", mp->lines);
 return SCPE_OK;
 }
 
+
+
+/* Global utility routines */
+
+
+/* Initialize the line state.
+
+   Reset the line state to represent an idle line.  Note that we do not clear
+   all of the line structure members, so a connected line remains connected
+   after this call.
+
+   Because a line break is represented by a flag in the "receive break status"
+   array, we must zero that array in order to clear any leftover break
+   indications.
+*/
+
+void tmxr_init_line (TMLN *lp)
+{
+lp->tsta = 0;                                           /* clear the Telnet negotiation state */
+lp->xmte = 1;                                           /* enable transmission */
+lp->dstb = 0;                                           /* default to binary data mode */
+lp->rxbpr = lp->rxbpi = lp->rxcnt = 0;                  /* clear the receive indexes */
+lp->txbpr = lp->txbpi = lp->txcnt = 0;                  /* clear the transmit indexes */
+
+memset (lp->rbr, 0, TMXR_MAXBUF);                       /* clear the break status array */
+
+return;
+}
+
+
+/* Report a connection to a line.
+
+   This routine sends a notification of the form:
+
+      Connected to the <sim> simulator <dev> device, line <n>
+
+   ...to the line number specified by the "line" parameter of the multiplexer
+   associated with the line descriptor pointer "lp" and the mux descriptor
+   pointer "mp".  If the device has only one line, the "line <n>" part is
+   omitted.  If the device has not been defined, the "<dev> device" part is
+   omitted.
+*/
+
+void tmxr_report_connection (TMXR *mp, TMLN *lp, int32 line)
+{
+int32 buffer_count;
+char  line_number [20];
+
+tmxr_linemsg (lp, "\n\r\nConnected to the ");           /* report the connection */
+tmxr_linemsg (lp, sim_name);                            /*   to the simulator */
+tmxr_linemsg (lp, " simulator ");
+
+if (mp->dptr) {                                         /* if the device pointer has been set */
+    tmxr_linemsg (lp, sim_dname (mp->dptr));            /*   then report the name */
+    tmxr_linemsg (lp, " device");                       /*     of the connected device */
+
+    if (mp->lines > 1) {                                /* if the multiplexer has more than one line */
+        tmxr_linemsg (lp, ", line ");                   /*   then report the line number */
+        sprintf (line_number, "%i", line);              /*     of the connection */
+        tmxr_linemsg (lp, line_number);
+        }
+    }
+
+tmxr_linemsg (lp, "\r\n\n");
+
+buffer_count = tmxr_send_buffered_data (lp);            /* write the message */
+
+if (buffer_count == 0)                                  /* if the write buffer is now empty */
+    lp->xmte = 1;                                       /*   then reenable transmission if it was paused */
+
+return;
+}
+
+
+/* Report a disconnection from a line.
+
+   This routine sends a notification of the form:
+
+      Disconnected from the <sim> simulator
+
+   ...to the line number associated with the line descriptor pointer "lp" and
+   then disconnects the line.
+
+
+   Implementation notes:
+
+    1. We do not write the buffer here, because the disconnect routine will do
+       that for us.
+*/
+
+void tmxr_disconnect_line (TMLN *lp)
+{
+tmxr_linemsg (lp, "\r\nDisconnected from the ");        /* report the disconnection */
+tmxr_linemsg (lp, sim_name);                            /*   from the simulator */
+tmxr_linemsg (lp, " simulator\r\n\n");
+
+tmxr_reset_ln (lp);                                     /* disconnect the line */
+
+return;
+}
+
+
+
+/* Local hook routines */
+
+
+/* Read from a line.
+
+   This routine reads up to "length" bytes into the buffer associated with line
+   "lp".  The actual number of bytes read is returned.  If no bytes are
+   available, 0 is returned.  If an error occurred while reading, -1 is
+   returned.
+*/
+
+static int32 tmxr_local_read (TMLN *lp, int32 length)
+{
+return sim_read_sock (lp->conn, &(lp->rxb [lp->rxbpi]), length);
+}
+
+
+/* Write to a line.
+
+   This routine writes up to "length" bytes from the buffer associated with
+   "lp".  The actual number of bytes written is returned.  If an error occurred
+   while writing, -1 is returned.
+*/
+
+static int32 tmxr_local_write (TMLN *lp, int32 length)
+{
+int32 written;
+
+written = sim_write_sock (lp->conn, &(lp->txb [lp->txbpr]), length);
+
+if (written == SOCKET_ERROR)                        /* did an error occur? */
+    return -1;                                      /* return error indication */
+else
+    return written;
+}
+
+
+/* Show a line.
+
+   This routine writes the port description to the file indicated by the
+   "stream" parameter.
+*/
+
+static void tmxr_local_show (TMLN *lp, FILE *stream)
+{
+fprintf (stream, "IP address %s", lp->ipad);
+return;
+}
+
+
+/* Close a line.
+
+   This routine closes the line indicated by the "lp" parameter.
+*/
+
+static void tmxr_local_close (TMLN *lp)
+{
+sim_close_sock (lp->conn);                              /* reset conn */
+return;
+}
